/****************************************************************************
 
SC 29 Software Copyright Licencing Disclaimer:

This software module was originally developed by
  Coding Technologies

and edited by
  -

in the course of development of the ISO/IEC 13818-7 and ISO/IEC 14496-3 
standards for reference purposes and its performance may not have been 
optimized. This software module is an implementation of one or more tools as 
specified by the ISO/IEC 13818-7 and ISO/IEC 14496-3 standards.
ISO/IEC gives users free license to this software module or modifications 
thereof for use in products claiming conformance to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International 
Standards. ISO/IEC gives users the same free license to this software module or 
modifications thereof for research purposes and further ISO/IEC standardisation.
Those intending to use this software module in products are advised that its 
use may infringe existing patents. ISO/IEC have no liability for use of this 
software module or modifications thereof. Copyright is not released for 
products that do not conform to audiovisual and image-coding related ITU 
Recommendations and/or ISO/IEC International Standards.
The original developer retains full right to modify and use the code for its 
own purpose, assign or donate the code to a third party and to inhibit third 
parties from using the code for products that do not conform to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International Standards.
This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2002.



*******************************************************************************/
/*!
  \file
  \brief  SBR conformance Tool down sampler
*/



#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "adifdec.h"
#include "block.h"
#include "config.h"


#include "ct_sbrConfToolDownSampler.h"





#define SBR_UPSAMPLER_DELAY 481
#define POLYPHASE_FILTERS 32



static void filterPoly(double *input, const double * b_real,  double * output, int filter_length, int in_length)
{
	int i,j;
		
	double acc_real;
	const double *coef_ptr_b;
	double *in_ptr,  *data_ptr_in;
	double *out_ptr;
	

	in_ptr  = input;
	out_ptr = output;
		
	for(i=0; i<in_length;i++)
	{
		data_ptr_in  = in_ptr  -1;			/* Set pointers...*/
		coef_ptr_b   = b_real;
				
		acc_real = *in_ptr*(*coef_ptr_b++);

		for (j=1; j< filter_length; j++){
			acc_real += (*coef_ptr_b++) * (*data_ptr_in--);
		}

    *out_ptr++ = acc_real;
		in_ptr++;
	}
}




int downSampleSbr(double* timeIn[Chans], int downSample, int ch)
{

#define POLYPHASE_FILTER_LENGTH 25


  static const double H[POLYPHASE_FILTERS][POLYPHASE_FILTER_LENGTH] = {
    {3.476697953131265e-008, 0,	-6.763157773587183e-008, 0, -6.974973600673098e-006, 0, -1.924219571106937e-004, 0, 1.629822651831791e-004, 0, 3.540528761884519e-004, 0, 9.999940276586996e-001, 0, 3.540528761884519e-004, 0,	1.629822651831791e-004, 0, -1.924219571106937e-004, 0, -6.974973600673098e-006, 0, -6.763157773587183e-008, 0, 3.476697953131265e-008},
    {1.396471916031115e-007, 0,	-2.974245852183672e-007, 0, -1.318138373054617e-005, 0, -3.865443699577471e-004, 0, 3.721410827846135e-004, 0, 3.666791540864976e-004, 0, 9.999958594813760e-001, 0, 3.666791540864976e-004, 0,	3.721410827846135e-004, 0, -3.865443699577471e-004, 0, -1.318138373054617e-005, 0, -2.974245852183672e-007, 0, 1.396471916031115e-007},
    {1.224963549817200e-007, 0,	-2.646404681011980e-007, 0, -1.108178601327391e-005, 0, -3.610222352290030e-004, 0, 3.554111749102114e-004, 0, 3.596349479868729e-004, 0, 1.000005611329237e+000, 0, 3.596349479868729e-004, 0,	3.554111749102114e-004, 0, -3.610222352290030e-004, 0, -1.108178601327391e-005, 0, -2.646404681011980e-007, 0, 1.224963549817200e-007},
    {1.326399319916421e-007, 0,	-2.700352437567778e-007, 0, -9.882252437042349e-006, 0, -3.735656036780282e-004, 0, 3.517646637852679e-004, 0, 3.633285653067748e-004, 0, 1.000010164947417e+000, 0, 3.633285653067748e-004, 0,	3.517646637852679e-004, 0, -3.735656036780282e-004, 0, -9.882252437042349e-006, 0, -2.700352437567778e-007, 0, 1.326399319916421e-007},
    {1.522562232650754e-007, 0,	-2.984765811933622e-007, 0, -8.632246089961839e-006, 0, -3.981722856143535e-004, 0, 3.668277251617622e-004, 0, 3.652031799326229e-004, 0, 1.000002336408856e+000, 0, 3.652031799326229e-004, 0,	3.668277251617622e-004, 0, -3.981722856143535e-004, 0, -8.632246089961839e-006, 0, -2.984765811933622e-007, 0, 1.522562232650754e-007},
    {1.770077632287240e-007, 0,	-3.191765536892460e-007, 0, -7.252506589625763e-006, 0, -4.269398602904086e-004, 0, 3.673910838620092e-004, 0, 3.739888363247150e-004, 0, 1.000002916620203e+000, 0, 3.739888363247150e-004, 0,	3.673910838620092e-004, 0, -4.269398602904086e-004, 0, -7.252506589625763e-006, 0, -3.191765536892460e-007, 0, 1.770077632287240e-007},
    {2.068765829715687e-007, 0,	-3.511660788410439e-007, 0, -5.652904186405254e-006, 0, -4.594357796680696e-004, 0, 3.776142052929487e-004, 0, 3.659398368134911e-004, 0, 1.000009380633999e+000, 0, 3.659398368134911e-004, 0,	3.776142052929487e-004, 0, -4.594357796680696e-004, 0, -5.652904186405254e-006, 0, -3.511660788410439e-007, 0, 2.068765829715687e-007},
    {2.421414005440356e-007, 0,	-3.782805603719183e-007, 0, -3.994773586637473e-006, 0, -4.950706931314535e-004, 0, 3.790387553154356e-004, 0, 3.671113185055310e-004, 0, 1.000014455098329e+000, 0, 3.671113185055310e-004, 0,	3.790387553154356e-004, 0, -4.950706931314535e-004, 0, -3.994773586637473e-006, 0, -3.782805603719183e-007, 0, 2.421414005440356e-007},
    {2.788460966259352e-007, 0,	-4.079961401854086e-007, 0, -2.260958885598900e-006, 0, -5.296088142178838e-004, 0, 3.835318699545681e-004, 0, 3.763419839421675e-004, 0, 9.999940885015929e-001, 0, 3.763419839421675e-004, 0,	3.835318699545681e-004, 0, -5.296088142178838e-004, 0, -2.260958885598900e-006, 0, -4.079961401854086e-007, 0, 2.788460966259352e-007},
    {3.161911779625002e-007, 0,	-4.313203422567221e-007, 0, -5.273303865290991e-007, 0, -5.625765678906092e-004, 0, 3.829529676084116e-004, 0, 3.673391073119114e-004, 0, 9.999951684002906e-001, 0, 3.673391073119114e-004, 0,	3.829529676084116e-004, 0, -5.625765678906092e-004, 0, -5.273303865290991e-007, 0, -4.313203422567221e-007, 0, 3.161911779625002e-007},
    {3.543122532697675e-007, 0,	-4.608625488256222e-007, 0,  1.184078570054111e-006, 0, -5.943681506843667e-004, 0, 3.884059372906167e-004, 0, 3.792552922175888e-004, 0, 9.999972692390483e-001, 0, 3.792552922175888e-004, 0,	3.884059372906167e-004, 0, -5.943681506843667e-004, 0,  1.184078570054111e-006, 0, -4.608625488256222e-007, 0, 3.543122532697675e-007},
    {3.909402775633086e-007, 0,	-4.739340938681313e-007, 0,  2.706078490008691e-006, 0, -6.234965573461278e-004, 0, 3.816668617870812e-004, 0, 3.772857914767589e-004, 0, 1.000002886171420e+000, 0, 3.772857914767589e-004, 0,	3.816668617870812e-004, 0, -6.234965573461278e-004, 0,  2.706078490008691e-006, 0, -4.739340938681313e-007, 0, 3.909402775633086e-007},
    {4.222809367831911e-007, 0,	-4.878510064125678e-007, 0,  4.036795191477909e-006, 0, -6.473621895606138e-004, 0, 3.791705819779010e-004, 0, 3.847083857331159e-004, 0, 9.999969286735556e-001, 0, 3.847083857331159e-004, 0,	3.791705819779010e-004, 0, -6.473621895606138e-004, 0,  4.036795191477909e-006, 0, -4.878510064125678e-007, 0, 4.222809367831911e-007},
    {4.502138847524999e-007, 0,	-4.999326042535363e-007, 0,  5.135058417623972e-006, 0, -6.679743577822745e-004, 0, 3.771635262404024e-004, 0, 3.866181819880556e-004, 0, 9.999972973017242e-001, 0, 3.866181819880556e-004, 0,	3.771635262404024e-004, 0, -6.679743577822745e-004, 0,  5.135058417623972e-006, 0, -4.999326042535363e-007, 0, 4.502138847524999e-007},
    {4.709334223508883e-007, 0,	-5.078830669590950e-007, 0,  5.962080730857686e-006, 0, -6.828692913148293e-004, 0, 3.751956167593989e-004, 0, 3.827003698106720e-004, 0, 1.000008336100520e+000, 0, 3.827003698106720e-004, 0,	3.751956167593989e-004, 0, -6.828692913148293e-004, 0,  5.962080730857686e-006, 0, -5.078830669590950e-007, 0, 4.709334223508883e-007},
    {4.843159761574267e-007, 0,	-5.062025248855524e-007, 0,  6.458113652318982e-006, 0, -6.923844198753944e-004, 0, 3.691031022061020e-004, 0, 3.826743784427686e-004, 0, 1.000007446899661e+000, 0, 3.826743784427686e-004, 0,	3.691031022061020e-004, 0, -6.923844198753944e-004, 0,  6.458113652318982e-006, 0, -5.062025248855524e-007, 0, 4.843159761574267e-007},
    {4.881790944646534e-007, 0,	-5.091494784777248e-007, 0,  6.616136460265565e-006, 0, -6.950689516769845e-004, 0, 3.698758977096631e-004, 0, 3.826493196713532e-004, 0, 9.999860218829778e-001, 0, 3.826493196713532e-004, 0,	3.698758977096631e-004, 0, -6.950689516769845e-004, 0,  6.616136460265565e-006, 0, -5.091494784777248e-007, 0, 4.881790944646534e-007},
    {4.843159761574265e-007, 0,	-5.062025248858790e-007, 0,  6.458113652319081e-006, 0, -6.923844198753948e-004, 0, 3.691031022060960e-004, 0, 3.826743784426941e-004, 0, 1.000007446899662e+000, 0, 3.826743784426941e-004, 0,	3.691031022060960e-004, 0, -6.923844198753948e-004, 0,  6.458113652319081e-006, 0, -5.062025248858790e-007, 0, 4.843159761574265e-007},
    {4.709334223509752e-007, 0,	-5.078830668471703e-007, 0,  5.962080730857398e-006, 0, -6.828692913148301e-004, 0, 3.751956167594234e-004, 0, 3.827003698109648e-004, 0, 1.000008336100521e+000, 0, 3.827003698109648e-004, 0,	3.751956167594234e-004, 0, -6.828692913148301e-004, 0,  5.962080730857398e-006, 0, -5.078830668471703e-007, 0, 4.709334223509752e-007},
    {4.502138847525010e-007, 0,	-4.999326042533778e-007, 0,  5.135058417624179e-006, 0, -6.679743577822758e-004, 0, 3.771635262403889e-004, 0, 3.866181819878970e-004, 0, 9.999972973017255e-001, 0, 3.866181819878970e-004, 0,	3.771635262403889e-004, 0, -6.679743577822758e-004, 0,  5.135058417624179e-006, 0, -4.999326042533778e-007, 0, 4.502138847525010e-007},
    {4.222809367831921e-007, 0,	-4.878510064126016e-007, 0,  4.036795191477679e-006, 0, -6.473621895606155e-004, 0, 3.791705819779300e-004, 0, 3.847083857334735e-004, 0, 9.999969286735594e-001, 0, 3.847083857334735e-004, 0,	3.791705819779300e-004, 0, -6.473621895606155e-004, 0,  4.036795191477679e-006, 0, -4.878510064126016e-007, 0, 4.222809367831921e-007},
    {3.909402775633092e-007, 0,	-4.739340938680947e-007, 0,  2.706078490009059e-006, 0, -6.234965573461287e-004, 0, 3.816668617870499e-004, 0, 3.772857914763712e-004, 0, 1.000002886171421e+000, 0, 3.772857914763712e-004, 0,	3.816668617870499e-004, 0, -6.234965573461287e-004, 0,  2.706078490009059e-006, 0, -4.739340938680947e-007, 0, 3.909402775633092e-007},
    {3.543122532697675e-007, 0,	-4.608625488256092e-007, 0,  1.184078570054243e-006, 0, -5.943681506843663e-004, 0, 3.884059372906052e-004, 0, 3.792552922174683e-004, 0, 9.999972692390470e-001, 0, 3.792552922174683e-004, 0,	3.884059372906052e-004, 0, -5.943681506843663e-004, 0,  1.184078570054243e-006, 0, -4.608625488256092e-007, 0, 3.543122532697675e-007},
    {3.161911779624999e-007, 0,	-4.313203422567522e-007, 0, -5.273303865294305e-007, 0, -5.625765678906092e-004, 0, 3.829529676084380e-004, 0, 3.673391073122271e-004, 0, 9.999951684002906e-001, 0, 3.673391073122271e-004, 0,	3.829529676084380e-004, 0, -5.625765678906092e-004, 0, -5.273303865294305e-007, 0, -4.313203422567522e-007, 0, 3.161911779624999e-007},
    {2.788460966259361e-007, 0,	-4.079961401853398e-007, 0, -2.260958885598140e-006, 0, -5.296088142178834e-004, 0, 3.835318699545062e-004, 0, 3.763419839414048e-004, 0, 9.999940885015904e-001, 0, 3.763419839414048e-004, 0,	3.835318699545062e-004, 0, -5.296088142178834e-004, 0, -2.260958885598140e-006, 0, -4.079961401853398e-007, 0, 2.788460966259361e-007},
    {2.421414005440362e-007, 0,	-3.782805603718790e-007, 0, -3.994773586636954e-006, 0, -4.950706931314529e-004, 0, 3.790387553153991e-004, 0, 3.671113185050744e-004, 0, 1.000014455098326e+000, 0, 3.671113185050744e-004, 0,	3.790387553153991e-004, 0, -4.950706931314529e-004, 0, -3.994773586636954e-006, 0, -3.782805603718790e-007, 0, 2.421414005440362e-007},
    {2.068765829715683e-007, 0,	-3.511660788410130e-007, 0, -5.652904186404924e-006, 0, -4.594357796680679e-004, 0, 3.776142052929195e-004, 0, 3.659398368131350e-004, 0, 1.000009380633996e+000, 0, 3.659398368131350e-004, 0,	3.776142052929195e-004, 0, -4.594357796680679e-004, 0, -5.652904186404924e-006, 0, -3.511660788410130e-007, 0, 2.068765829715683e-007},
    {1.770077632288873e-007, 0,	-3.191765536890821e-007, 0, -7.252506589403714e-006, 0, -4.269398602904083e-004, 0, 3.673910838620111e-004, 0, 3.739888363247353e-004, 0, 1.000002916620202e+000, 0, 3.739888363247353e-004, 0,	3.673910838620111e-004, 0, -4.269398602904083e-004, 0, -7.252506589403714e-006, 0, -3.191765536890821e-007, 0, 1.770077632288873e-007},
    {1.522562232650743e-007, 0,	-2.984765811933783e-007, 0, -8.632246089962066e-006, 0, -3.981722856143523e-004, 0, 3.668277251617779e-004, 0, 3.652031799328445e-004, 0, 1.000002336408854e+000, 0, 3.652031799328445e-004, 0,	3.668277251617779e-004, 0, -3.981722856143523e-004, 0, -8.632246089962066e-006, 0, -2.984765811933783e-007, 0, 1.522562232650743e-007},
    {1.326399319916421e-007, 0,	-2.700352437567779e-007, 0, -9.882252437042350e-006, 0, -3.735656036780283e-004, 0, 3.517646637852678e-004, 0, 3.633285653067912e-004, 0, 1.000010164947417e+000, 0, 3.633285653067912e-004, 0,	3.517646637852678e-004, 0, -3.735656036780283e-004, 0, -9.882252437042350e-006, 0, -2.700352437567779e-007, 0, 1.326399319916421e-007},
    {1.224963549817213e-007, 0,	-2.646404681008590e-007, 0, -1.108178601327352e-005, 0, -3.610222352290024e-004, 0, 3.554111749101667e-004, 0, 3.596349479863394e-004, 0, 1.000005611329234e+000, 0, 3.596349479863394e-004, 0,	3.554111749101667e-004, 0, -3.610222352290024e-004, 0, -1.108178601327352e-005, 0, -2.646404681008590e-007, 0, 1.224963549817213e-007},
    {1.396471916031121e-007, 0,	-2.974245852182784e-007, 0, -1.318138373054608e-005, 0, -3.865443699577474e-004, 0, 3.721410827846046e-004, 0, 3.666791540863757e-004, 0, 9.999958594813758e-001, 0, 3.666791540863757e-004, 0,	3.721410827846046e-004, 0, -3.865443699577474e-004, 0, -1.318138373054608e-005, 0, -2.974245852182784e-007, 0, 1.396471916031121e-007},
   /* {3.476697953131265e-008, 0,	-6.763157773587183e-008, 0, -6.974973600673098e-006, 0, -1.924219571106937e-004, 0, 1.629822651831791e-004, 0, 3.540528761884519e-004, 0, 9.999940276586996e-001, 0, 3.540528761884519e-004, 0,	1.629822651831791e-004, 0, -1.924219571106937e-004, 0, -6.974973600673098e-006, 0, -6.763157773587183e-008, 0, 3.476697953131265e-008} */

    };
 			
  

  static double maxDiff[Chans] = {0,0,0,0,0,0,0,0,0,0};
  static double filterInput [Chans][POLYPHASE_FILTERS][POLYPHASE_FILTER_LENGTH + 32];
  static double filterOutput[Chans][POLYPHASE_FILTERS][32];
  int k, i;
  int shift = 0;

  int delay;
  

  static int init[10] = {0,0,0,0,0,0,0,0,0,0};
  if(init[ch]==0){
    init[ch] = 1;
    for(k=0;k<POLYPHASE_FILTERS;k++){
      memset(filterInput[ch][k],0,(POLYPHASE_FILTER_LENGTH+32)*sizeof(double));
      memset(filterOutput[ch][k],0,32*sizeof(double));
    }
  }


  /* Buffer data*/
  for(k=0; k< 32; k++){
    memcpy(filterInput[ch][k],filterInput[ch][k] + 32, POLYPHASE_FILTER_LENGTH * sizeof(double)); 
  }


  /* Decimates signal, and get the polyphase representation*/
  if(downSample){
    for(k=0; k < 32; k++){
      for(i = 0; i < 32; i++){
        filterInput[ch][k][POLYPHASE_FILTER_LENGTH + i] = timeIn[ch][2*k + 2*i*32];
      }
    }
  }
  else{
    for(k=0; k < 32; k++){
      for(i = 0; i < 32; i++){
        filterInput[ch][k][POLYPHASE_FILTER_LENGTH + i] = timeIn[ch][k + i*32];
      }
    }
  }


  /* Now do the poly phase filtering*/
  for(k = 0; k < 32; k++){
    filterPoly(filterInput[ch][k] + POLYPHASE_FILTER_LENGTH, H[(k+shift)&31],  filterOutput[ch][k], POLYPHASE_FILTER_LENGTH, 32);
  }


  /* Create the output from the filtered polyphase components.*/
  for(k=0; k < 32; k++){
    for(i = 0; i < 32; i++){
      timeIn[ch][k+i*32] = (float) filterOutput[ch][k][i];
    }
  }


  delay = SBR_UPSAMPLER_DELAY + 32*((POLYPHASE_FILTER_LENGTH - 1)/2); /* 769 samples OR 865 samples*/
  
  return (delay);
}














































#if 0

/* OLD CODE LEFT FOR REFERENCE*/

#define BUFFER_DELAY 0
#define SBR_UPSAMPLER_DELAY 481
#define POLYPHASE_FILTER_LENGTH 19
#define POLYPHASE_FILTERS 32

void downSampleSbr(double* timeIn[Chans], int ch)
{
  static const double H[POLYPHASE_FILTERS][POLYPHASE_FILTER_LENGTH] = {
    {                       0, -6.865234390739336e-006,   2.021734177878334e-019, -1.925323718485778e-004,  1.082900037487784e-017,  1.629953963851696e-004,  2.068603282380424e-016, 3.539974165991966e-004,  1.618363057262597e-015, 9.999936496672969e-001, -1.618363057262596e-015, 3.539974165991959e-004, -2.068603282380423e-016, 1.629953963851696e-004, -1.082900037487784e-017, -1.925323718485778e-004, -2.021734177878334e-019, -6.865234390739336e-006,                       0},
    {  1.243571918183249e-021, -1.303629314034515e-005,   1.007482107899088e-019, -3.868072386214555e-004,  3.600233387327305e-018,  3.722995327706143e-004,  5.188673120576248e-017, 3.666835871299664e-004,  4.710738998470909e-016, 9.999950140908107e-001, -6.312842655161113e-016, 3.666835871299578e-004, -8.726942540416822e-017, 3.722995327706143e-004, -2.781271810407721e-018, -3.868072386214555e-004,  4.626898146766910e-020, -1.303629314034515e-005,  3.791117052581419e-021},
    {  5.206963012926867e-021, -1.094833675574678e-005,   2.874049636689893e-020, -3.612695775745262e-004, -8.010641692504745e-018,  3.555490606688389e-004, -2.207218844534837e-016, 3.596277879683647e-004, -1.545698654732715e-015, 1.000004838835959e+000,  1.067613158669950e-015, 3.596277879683966e-004,  1.081523430052019e-016, 3.555490606688383e-004,  7.341855330725790e-018, -3.612695775745262e-004,  1.615087160061812e-019, -1.094833675574677e-005,  7.890857955611802e-022},
    { -6.279844210196819e-022, -9.734443041653594e-006,  -2.146899900415647e-020, -3.738136990518428e-004,  2.654578473833463e-019,  3.519107228811528e-004,  1.981433149403573e-017, 3.633282356300822e-004,  8.632752803290187e-017, 1.000009373879989e+000,  8.989823143855850e-017, 3.633282356300748e-004,  2.030464494320142e-017, 3.519107228811515e-004,  2.598490450980780e-019, -3.738136990518428e-004, -2.194515370753232e-020, -9.734443041653594e-006, -6.223574543038185e-022},
    { -1.469447278277998e-021, -8.475881361167675e-006,  -7.316967428058749e-020, -3.984335262224805e-004,  2.846431188389397e-019,  3.669911214045873e-004,  5.865351758623558e-017, 3.652203343358905e-004,  1.814392257497429e-016, 1.000001482969120e+000,  5.024265990751603e-016, 3.652203343359212e-004,  9.916474167934498e-017, 3.669911214045880e-004,  2.168292569851150e-018, -3.984335262224805e-004, -6.647911524925469e-020, -8.475881361167680e-006, -1.905177114713605e-021},
    { -1.941033631853410e-021, -7.068036755120318e-006,  -8.871357722701897e-021, -4.272088013492551e-004,  8.871452121365235e-018,  3.675754508682389e-004,  2.018424117943192e-016, 3.740213799943714e-004,  1.551776937162320e-015, 1.000002001887051e+000, -1.431013982379615e-015, 3.740213799943787e-004, -1.743121604568072e-016, 3.675754508682384e-004, -8.396146702243366e-018, -4.272088013492551e-004, -1.166361009063702e-020, -7.068036755120319e-006,  1.529577240181109e-021},
    { -2.722237728073619e-023, -5.459141645447217e-006,   4.073014742342162e-020, -4.597075026190174e-004,  4.410350475970104e-018,  3.778203171334240e-004,  3.952650248180134e-017, 3.660050784712360e-004,  6.530480040433808e-016, 1.000008404956014e+000, -1.483233066633994e-015, 3.660050784712164e-004, -2.298807682035021e-016, 3.778203171334222e-004, -7.866451083007969e-018, -4.597075026190174e-004,  8.764896816032744e-020, -5.459141645447218e-006,  2.018078483626308e-021},
    { -1.096892448745143e-021, -3.774846439292608e-006,  -8.340348598068546e-020, -4.953454733444586e-004,  5.613020696306100e-018,  3.792699484826004e-004,  1.697389909882167e-016, 3.672041851999384e-004,  1.069567679879837e-015, 1.000013407494517e+000, -4.011631918404564e-016, 3.672041851999113e-004, -1.580190228107445e-017, 3.792699484825980e-004, -2.740546005026600e-018, -4.953454733444587e-004, -1.491852342532256e-020, -3.774846439292607e-006, -5.231980957418259e-024},
    { -6.220457096951751e-022, -2.009308400113623e-006,  -8.651292992119993e-020, -5.298951205120420e-004,  4.968733524344841e-018,  3.837899775017953e-004,  1.486945429604031e-016, 3.764569636139665e-004,  9.409678165910275e-016, 9.999929494657270e-001, -3.650336880081740e-016, 3.764569636139641e-004, -1.619475956754203e-017, 3.837899775017960e-004, -2.417107116986603e-018, -5.298951205120420e-004,  6.590184988248803e-021, -2.009308400113623e-006,  4.485637428955138e-023},
    {  6.642628441609002e-022, -2.605514716721967e-007,   1.987671398224433e-019, -5.628567695118514e-004, -8.626392475443331e-018,  3.832310765090504e-004, -3.144882833675799e-016, 3.674878630560550e-004, -1.720990435627330e-015, 9.999939715884588e-001, -1.483402370621734e-016, 3.674878630560500e-004, -1.161934714582663e-016, 3.832310765090522e-004,  1.860786941223692e-019, -5.628567695118515e-004,  5.094939372853976e-020, -2.605514716721969e-007,  2.435034624253366e-022},
    { -4.081652389146168e-023,  1.484184513811135e-006,  -2.634648310587499e-019, -5.946627942997303e-004,  1.266216856882457e-017,  3.887112710890864e-004,  2.802768052410791e-016, 3.794236048471258e-004,  2.270508845645257e-015, 9.999959725807082e-001, -2.377165673536332e-015, 3.794236048471405e-004, -3.052312484884072e-016, 3.887112710890851e-004, -1.315317239721914e-017, -5.946627942997301e-004,  2.781188977195959e-019,  1.484184513811135e-006,  4.413127837137788e-023},
    {  3.343285716071579e-023,  3.031012980461416e-006,  -6.032751065512213e-020, -6.237856429018064e-004,  2.725077644368407e-018,  3.819921409467687e-004,  7.724056020563691e-018, 3.774769176998343e-004,  4.180876227051939e-016, 1.000001531879849e+000, -1.279184462610244e-015, 3.774769176998490e-004, -2.061681570369677e-016, 3.819921409467668e-004, -6.670849226072696e-018, -6.237856429018065e-004,  1.719244428772636e-019,  3.031012980461416e-006, -3.071906947629278e-022},
    {  5.900806731895393e-022,  4.391256566413461e-006,  -1.541768992232937e-019, -6.476560049399591e-004,  5.593405032839932e-018,  3.795167862341264e-004,  2.142232661103879e-016, 3.849126045213137e-004,  1.130935067104583e-015, 9.999955061342802e-001,  2.489188618574501e-016, 3.849126045213579e-004,  1.035705218682515e-016, 3.795167862341251e-004,  7.791681484178382e-019, -6.476560049399589e-004, -1.999298154204988e-020,  4.391256566413460e-006,  2.425684129992244e-022},
    { -1.643250768805279e-021,  5.509453213431000e-006,   3.108446050835426e-019, -6.682688582697877e-004, -1.093751290049027e-017,  3.775258209985615e-004, -4.382600952185857e-016, 3.868370911920327e-004, -2.241229607019397e-015, 9.999958205716027e-001, -7.741996017487982e-016, 3.868370911920523e-004, -2.536724678013554e-016, 3.775258209985590e-004, -2.891776828410536e-018, -6.682688582697877e-004,  6.539894991798637e-020,  5.509453213431000e-006, -8.206810370965166e-022},
    {  6.359242804336877e-023,  6.344120143117411e-006,   2.785173277038877e-020, -6.831598621117222e-004, -5.649537171956114e-019,  3.755664687703318e-004,  2.886859940605959e-017, 3.829335761992473e-004, -6.011356946954504e-017, 1.000006828097960e+000,  7.422677094585316e-016, 3.829335761992620e-004,  1.289522466756778e-016, 3.755664687703306e-004,  3.771962367213314e-018, -6.831598621117223e-004, -1.136901755827648e-019,  6.344120143117413e-006,  6.057563311484554e-022},
    {  8.287859317592759e-022,  6.851935050688066e-006,  -2.442526574084260e-019, -6.926706591139295e-004,  6.969035992028906e-018,  3.694809262512320e-004,  1.217075180032954e-016, 3.829117789643521e-004,  1.221847307287007e-015, 1.000005921817279e+000, -1.842136184614329e-015, 3.829117789643619e-004, -2.642462354451600e-016, 3.694809262512302e-004, -9.816309860337781e-018, -6.926706591139295e-004,  3.209804874202833e-019,  6.851935050688066e-006, -1.490190072091124e-021},
    {  1.495815923016830e-021,  7.011416021343352e-006,  -2.970761826648268e-019, -6.953558775774037e-004,  9.193565063630031e-018,  3.702556349118133e-004,  2.655385150632393e-016, 3.828894626439082e-004,  1.742769782162350e-015, 9.999844882037938e-001, -8.615166608405658e-016, 3.828894626439127e-004, -6.275188385911457e-017, 3.702556349118132e-004, -5.094427960724789e-018, -6.953558775774037e-004,  1.879513587457154e-019,  7.011416021343352e-006, -5.332771897204549e-022},
    { -3.603273717961868e-022,  6.851935050688065e-006,   1.793200657233428e-020, -6.926706591139293e-004, -9.986287480231466e-019,  3.694809262512309e-004, -8.669852276355144e-017, 3.829117789643364e-004, -2.661718329002662e-016, 1.000005921817279e+000, -7.622607378509191e-016, 3.829117789643512e-004, -1.494867874071155e-016, 3.694809262512296e-004, -3.743792605765745e-018, -6.926706591139293e-004,  1.098506037392192e-019,  6.851935050688065e-006, -7.356542053642170e-022},
    {  1.567169139603102e-021,  6.344120143117412e-006,  -3.329315943192525e-019, -6.831598621117225e-004,  1.071774868631668e-017,  3.755664687703317e-004,  3.186653648498202e-016, 3.829335761992442e-004,  2.044890616759323e-015, 1.000006828097960e+000, -8.672867598039965e-016, 3.829335761992442e-004, -4.880814316978073e-017, 3.755664687703323e-004, -5.317471869489461e-018, -6.831598621117225e-004,  1.868975063272714e-019,  6.344120143117412e-006, -4.250770456090431e-022},
    {  5.376368300405154e-022,  5.509453213431000e-006,  -1.228054142842467e-019, -6.682688582697876e-004,  4.188127828196367e-018,  3.775258209985606e-004,  1.354594727981446e-016, 3.868370911920400e-004,  8.118258573683432e-016, 9.999958205716027e-001, -1.755792009088489e-016, 3.868370911920497e-004,  1.139881835321273e-017, 3.775258209985615e-004, -1.199346818180743e-018, -6.682688582697876e-004,  4.222477900179179e-020,  5.509453213430999e-006, -1.742454512376586e-023},
    {  1.838869843323161e-021,  4.391256566413458e-006,  -6.506791535965792e-019, -6.476560049399585e-004,  2.341381139739870e-017,  3.795167862341241e-004,  6.016113633590853e-016, 3.849126045213484e-004,  4.334253482021055e-015, 9.999955061342800e-001, -3.271215361442907e-015, 3.849126045212995e-004, -3.564790448519795e-016, 3.795167862341241e-004, -1.851925395415110e-017, -6.476560049399586e-004,  5.155926026492588e-019,  4.391256566413459e-006, -1.189976549655894e-021},
    {  1.911062246607499e-022,  3.031012980461417e-006,  -8.049031051494260e-020, -6.237856429018068e-004,  2.890244655644052e-018,  3.819921409467678e-004,  1.379845798744085e-016, 3.774769176998106e-004,  6.275889349277821e-016, 1.000001531879849e+000,  5.136633605365108e-016, 3.774769176998400e-004,  1.242494557057315e-016, 3.819921409467684e-004,  2.319401599367363e-018, -6.237856429018067e-004, -6.593433709163804e-020,  3.031012980461417e-006,  1.692758346768874e-022},
    { -7.791676254580272e-024,  1.484184513811135e-006,  -6.505243164395895e-020, -5.946627942997302e-004,  3.524141073894987e-018,  3.887112710890854e-004,  4.968585238180173e-017, 3.794236048471288e-004,  5.856953828771187e-016, 9.999959725807082e-001, -1.054035367258952e-015, 3.794236048471288e-004, -1.576035168396058e-016, 3.887112710890848e-004, -5.633390471002201e-018, -5.946627942997301e-004,  1.265471648473271e-019,  1.484184513811134e-006,  2.231322475503647e-023},
    { -1.833005963382686e-022, -2.605514716721971e-007,  -4.989760630184206e-020, -5.628567695118518e-004,  1.596548956079270e-018,  3.832310765090520e-004,  8.603290493478569e-017, 3.674878630560757e-004,  3.607642653627952e-016, 9.999939715884590e-001,  4.130516959635347e-016, 3.674878630560560e-004,  9.234632943654335e-017, 3.832310765090525e-004,  1.865977318727283e-018, -5.628567695118517e-004, -5.394344824170823e-020, -2.605514716721968e-007, -1.958695203010505e-022},
    { -4.644805208692255e-022, -2.009308400113623e-006,  -6.498133256639952e-020, -5.298951205120420e-004,  6.302998712534114e-018,  3.837899775017948e-004,  1.187760386523209e-016, 3.764569636139423e-004,  1.085987334352726e-015, 9.999929494657270e-001, -1.429481079788083e-015, 3.764569636139816e-004, -1.982267003841095e-016, 3.837899775017957e-004, -7.857243811820783e-018, -5.298951205120420e-004,  1.131729588025191e-019, -2.009308400113623e-006,  8.111030204984641e-022},
    {  8.647717096761325e-022, -3.774846439292608e-006,   8.056302586598797e-020, -4.953454733444587e-004, -1.533306590582830e-018,  3.792699484826005e-004, -1.169739815254321e-016, 3.672041851999107e-004, -4.180435557251579e-016, 1.000013407494517e+000, -8.185554555697827e-016, 3.672041851999205e-004, -1.672048835196434e-016, 3.792699484825993e-004, -3.793678605745485e-018, -4.953454733444587e-004,  9.935490202714275e-020, -3.774846439292608e-006,  1.164281103997435e-021},
    { -2.387251757585541e-021, -5.459141645447217e-006,  -9.047191716990166e-020, -4.597075026190173e-004,  1.066348906048648e-017,  3.778203171334231e-004,  2.831493178483795e-016, 3.660050784712002e-004,  1.953351619779160e-015, 1.000008404956014e+000, -1.249220197785010e-015, 3.660050784712393e-004, -1.209537201480910e-016, 3.778203171334242e-004, -7.721442235702127e-018, -4.597075026190173e-004, -1.937705773298938e-020, -5.459141645447217e-006,  6.820717863117788e-022},
    { -1.432546789702660e-021, -7.068036755120321e-006,   7.406990807248329e-021, -4.272088013492552e-004,  7.594595135363554e-018,  3.675754508682376e-004,  1.601429338157399e-016, 3.740213799943480e-004,  1.300869307136745e-015, 1.000002001887051e+000, -1.368723556653674e-015, 3.740213799943575e-004, -1.765568732954870e-016, 3.675754508682382e-004, -7.878492901666675e-018, -4.272088013492551e-004,  4.872109191555596e-021, -7.068036755120322e-006,  1.676602963166135e-021},
    { -5.345820400551283e-022, -8.475881361167672e-006,   1.342398432119811e-019, -3.984335262224803e-004,  1.032489396483405e-017,  3.669911214045876e-004,  1.550139652824568e-016, 3.652203343359025e-004,  1.597734150383360e-015, 1.000001482969120e+000, -2.495555664447337e-015, 3.652203343358927e-004, -3.605405361991918e-016, 3.669911214045873e-004, -1.350771526394202e-017, -3.984335262224802e-004,  4.727972753390470e-020, -8.475881361167670e-006,  4.930899289408786e-021},
    { -2.444975713336431e-022, -9.734443041653596e-006,  -3.920419169389628e-020, -3.738136990518427e-004, -1.641697682625061e-018,  3.519107228811540e-004, -1.779428030125157e-017, 3.633282356300758e-004, -2.211002190360434e-016, 1.000009373879989e+000,  4.309913362826457e-016, 3.633282356300752e-004,  6.551794828025898e-017, 3.519107228811528e-004,  2.275425499818871e-018, -3.738136990518429e-004, -1.331385185762041e-020, -9.734443041653598e-006, -1.241476893861987e-021},
    { -4.641820960033087e-021, -1.094833675574677e-005,   6.433709437739685e-020, -3.612695775745260e-004,  1.195608879969434e-017,  3.555490606688391e-004,  2.757259531502994e-016, 3.596277879683773e-004,  2.107285502645428e-015, 1.000004838835959e+000, -1.908309563441105e-015, 3.596277879683277e-004, -2.295625285686288e-016, 3.555490606688367e-004, -1.168514532763819e-017, -3.612695775745261e-004, -1.431717031892410e-019, -1.094833675574678e-005,  2.200520035072726e-021},
    {  3.545728385177969e-021, -1.303629314034516e-005,  -3.449805928923687e-021, -3.868072386214556e-004, -5.110490432103698e-018,  3.722995327706147e-004, -1.264262644551389e-016, 3.666835871299573e-004, -9.587517968088820e-016, 9.999950140908109e-001,  8.809885637258957e-016, 3.666835871299873e-004,  1.074733187229653e-016, 3.722995327706153e-004,  5.654839005894736e-018, -3.868072386214556e-004,  8.728537601846085e-020, -1.303629314034516e-005, -6.702368130468160e-022},
    /*{                       0, -6.865234390739336e-006,   2.021734177878334e-019, -1.925323718485778e-004,  1.082900037487784e-017,  1.629953963851696e-004,  2.068603282380424e-016, 3.539974165991966e-004,  1.618363057262597e-015, 9.999936496672969e-001, -1.618363057262596e-015, 3.539974165991959e-004, -2.068603282380423e-016, 1.629953963851696e-004, -1.082900037487784e-017, -1.925323718485778e-004, -2.021734177878334e-019, -6.865234390739336e-006,                       0}
      Moved to the top, qmfOffSet -> zero, otherwise one.
      */
  };
   
  static double maxDiff[Chans] = {0,0,0,0,0,0,0,0,0,0};
  static double filterInput [Chans][POLYPHASE_FILTERS][POLYPHASE_FILTER_LENGTH + 32];
  static double filterOutput[Chans][POLYPHASE_FILTERS][32];
  int k, i;

  static double temp[Chans][1024 + BUFFER_DELAY];
  static double tempDataRef[Chans][2048 + 1024];

  int delay = 1024;
  int qmfOffset = 0;

  static int init[10] = {0,0,0,0,0,0,0,0,0,0};
  if(init[ch]==0){
    init[ch] = 1;
    for(k=0;k<POLYPHASE_FILTERS;k++){
      memset(filterInput[ch][k],0,(POLYPHASE_FILTER_LENGTH+32)*sizeof(double));
      memset(filterOutput[ch][k],0,32*sizeof(double));
    }
    memset(temp[ch],0,(1024 + BUFFER_DELAY)*sizeof(double));
  }

  /* Buffer data*/
  
  memcpy(temp[ch],temp[ch]+1024, BUFFER_DELAY * sizeof(double)); 

  
  memcpy(tempDataRef[ch],tempDataRef[ch]+1024, 2048 * sizeof(double)); 
  memcpy(tempDataRef[ch] + 2048, tempDataDS[ch],1024*sizeof(double));


  for(k=0; k< 32; k++){
    memcpy(filterInput[ch][k],filterInput[ch][k] + 32, 19 * sizeof(double)); 
  }


  /* Decimates signal, and get the polyphase representation*/
  for(k=0;k<1024;k++){
    temp[ch][BUFFER_DELAY+k] = timeIn[ch][2*k];
  }

  for(k=0; k < 32; k++){
    for(i = 0; i < 32; i++){
      filterInput[ch][k][19 + i] = temp[ch][k + i*32 + qmfOffset];
    }
  }


  /* Now do the poly phase filtering*/
  for(k = 0; k < 32; k++){
    filterPoly(filterInput[ch][k] + 19, H[k],  filterOutput[ch][k], 19, 32);
  }


  /* Create the output from the filtered polyphase components.*/

  for(k=0; k < 32; k++){
    for(i = 0; i < 32; i++){
      timeIn[ch][k+i*32] = (float) filterOutput[ch][k][i];
    }
  }


  delay = BUFFER_DELAY + SBR_UPSAMPLER_DELAY + 32*((POLYPHASE_FILTER_LENGTH - 1)/2) - qmfOffset;

 /* for(k=0; k < 1024; k++){
    timeIn[ch][k] = tempDataRef[ch][k + 2048 - delay];
  }
*/
  /* Compare*/
  {

    for(i=0; i < 1024;i++){
      double diff = fabs(timeIn[ch][i] - tempDataRef[ch][i+ 2048 - (delay)]);
      if (diff > maxDiff[ch])
        maxDiff[ch] = diff;
    }

    printf("\n MAXDIFF: %f\n",maxDiff[ch]);
  }

}


#endif




  
  

