/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 * Matejko Michael (Fraunhofer IIS) 
 * 
 * and edited by 
 * 
 * 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#ifndef BITBUFFER_H
#define BITBUFFER_H

/** This is a ring buffer! */

#include "debug.h"

/** bit buffer handler */
typedef struct TAG_bitBuffer* HANDLE_bitBuffer;


/**
 * Initialize new bit buffer.
 * @param size Buffer size in bits
 * @param name Name for debug output
 * @param dbg Debug handler
 * @return bitBuffer handler
 */
HANDLE_bitBuffer bitBufferInit (unsigned long size, char* name, HANDLE_debug dbg);

/**
 * Open binary file as data source for bitBuffer.
 * @param buf bitBuffer handler
 * @param filename File to be opened 
 * @return True if succeed
 */
int bitBufferOpenInputFile(HANDLE_bitBuffer buf, char *filename);

/**
 * Put bits into bit buffer.
 * @param buf bitBuffer handler
 * @param value Contains the bits to be put into buffer
 * @param bits Number of bits to be written
 * @return Count of written bits 
 * @attention Return value will be 0 if not enough space available!
 */
unsigned long bitBufferPut (HANDLE_bitBuffer buf, unsigned long value, unsigned long bits);

/**
 * Read bits from bit buffer. 
 * @param buf bitBuffer handler
 * @param value Will be filled with bits from bit buffer
 * @param bits Number of bits to be read from buffer 
 * @return Count of read bits
 * @attention Return value may differ from bits if not enough bits available!
 */
unsigned long bitBufferRead (HANDLE_bitBuffer buf, unsigned long *value, unsigned long bits);

/**
 * Read bits from bit buffer and return them.
 * @attention No error checking here
 * @param buf bitBuffer handler
 * @param bits Number of bits to be read from buffer
 * @return Read bits
 */
unsigned long bitBufferGet (HANDLE_bitBuffer buf, unsigned long bits);

/**
 * Move the read pointer of bit buffer.
 * @attention Return value may differ from n if not enough bits available! 
 * @param buf bitBuffer handler
 * @param bits May be positive or negative and will move read pointer forward or backward
 * @return Count of bits moved
 */
long bitBufferWind (HANDLE_bitBuffer buf, long bits);

/**
 * Move the read pointer of bit buffer to given position.
 * @param buf bitBuffer handler
 * @param pos Desired position in the bitstream
 * @return on success true else false
 */
int bitBufferGoTo (HANDLE_bitBuffer buf, unsigned long pos);

/**
 * Get amount of read bits 
 * @param buf bitBuffer handler
 * @return Count of bits read from bit buffer
 */
unsigned long bitBufferGetRead (HANDLE_bitBuffer buf);

/**
 * Get amount of written bits
 * @param buf bitBuffer handler
 * @return Count of bits written to bit buffer
 */
unsigned long bitBufferGetWritten (HANDLE_bitBuffer buf);

/** 
 * Get amount of available free space in bit buffer 
 * @param buf bitBuffer handler
 * @return Available free space
 */
unsigned long bitBufferGetFree (HANDLE_bitBuffer buf);


/**
 * Get amount of available data bits in buffer 
 * @param buf bitBuffer handler
 * @return Count of available bits in bit buffer
 */
unsigned long bitBufferGetAvailable (HANDLE_bitBuffer buf);


/**
 * Copy bits from in_buf to out_buf until \n 
 * 1) \c bits bits transferred or \n
 * 2) \c in_buf signalizes underrun  or \n
 * 3) \c out_buf signalizes overfill 
 * 
 * @param in_buf bitBuffer handler of buffer containing data to copy
 * @param out_buf bitBuffer handler of buffer to be filled
 * @param bits Number of bits to be copied
 * @return Count of transferred bits
 * @attention Return value may differ from n in case 2 and 3! 
 */
unsigned long bitBufferTransfer (HANDLE_bitBuffer in_buf, HANDLE_bitBuffer out_buf, unsigned long bits);


/**
 * @param buf bitBuffer handler
 * @return Count of errors caused by bit buffer
 */
unsigned long bitBufferGetErrors (HANDLE_bitBuffer buf);


/**
 * Free all resources allocated by bitBuffer. Close associated files. 
 * @param buf bitBuffer handler
 */
void bitBufferFree (HANDLE_bitBuffer buf);


/**
 * Clear buffer
 * @param buf bitBuffer handler
 */
void bitBufferReset (HANDLE_bitBuffer buf);


#endif
