/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   Matejko Michael (Fraunhofer IIS) 
 * 
 * and edited by 
 * 
 * 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#ifndef PCMBUFFER_H
#define PCMBUFFER_H

/* pcm buffer status codes */
typedef enum {
  PCM_OK = 0,    /* operation ok */
  PCM_EMPTY,     /* no samples available */
  PCM_FULL,      /* buffer full */
  PCM_CLIPPED    /* sample ok, but clipped to maximum */
} STATUS_pcmBuffer;

/** pcm buffer handler */
typedef struct TAG_pcmBuffer* HANDLE_pcmBuffer;

/** Init pcm buffer. Return handler. */
HANDLE_pcmBuffer  pcmBufferInit    (unsigned int size);

/** Put pcm sample to buffer */
STATUS_pcmBuffer pcmBufferPut     (HANDLE_pcmBuffer buf, double sample);

/** Read one pcm sample to sample variable */
STATUS_pcmBuffer pcmBufferGet     (HANDLE_pcmBuffer buf, double *sample);

/** Read one sample, scale and cast to accuracy of n bits */
STATUS_pcmBuffer pcmBufferGetCasted (HANDLE_pcmBuffer buf, long *sample, int bits);

/** Get number of available samples in buffer */
unsigned int pcmBufferGetAvailable (HANDLE_pcmBuffer buf);

/** Get total number of samples stored in pcm buffer */
unsigned int pcmBufferGetWritten (HANDLE_pcmBuffer buf);

/** Reset pcm buffer to zero */
void pcmBufferReset (HANDLE_pcmBuffer buf);

/** Restore no of samples after BufferReset (needed for CRC recovery) */
unsigned int pcmBufferRestore (HANDLE_pcmBuffer buf, unsigned int samples);

/** Get number of channels set in buffer */
unsigned int pcmBufferGetChannels (HANDLE_pcmBuffer buf);

/** Set number of channels */
void pcmBufferSetChannels (HANDLE_pcmBuffer buf, unsigned int channels);

/** Get sampling frequency set in buffer */
unsigned int pcmBufferGetSamplingFreq (HANDLE_pcmBuffer buf);

/** Set sampling frequency */
void pcmBufferSetSamplingFreq (HANDLE_pcmBuffer buf, unsigned int sfreq);

/** Free all allocated resources */
void pcmBufferFree    (HANDLE_pcmBuffer buf);

#endif
