/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ??? 
 * 
 * and edited by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 *
 * $Id: common.c,v 1.25 2007/09/27 12:57:19 matejkml Exp $
 *
 * $Log: common.c,v $
 * Revision 1.25  2007/09/27 12:57:19  matejkml
 * moved most functions from here
 *
 * Revision 1.24  2007/03/29 14:00:08  matejkml
 * removed stdbool.h
 *
 * Revision 1.23  2007/01/12 15:00:53  matejkml
 * Added frameInit function
 *
 * Revision 1.22  2007/01/10 13:36:22  matejkml
 * bugfix2
 *
 * Revision 1.17  2006/03/17 09:50:40  dodenhen
 * fixed little layer12-bug
 *
 * Revision 1.16  2006/03/10 15:11:54  dodenhen
 * Added MPEG_2_5 support
 *
 * Revision 1.15  2004/09/29 14:57:08  kaehleof
 * moved the decoder tables into tables.c and .h;
 * no more tables/ directory needed now;
 * create_tables.c was used to create tables.c and .h
 * from the files in the tables/ directory;
 *
 * Revision 1.14  2003/11/17 15:02:14  tmm01
 *   by Ernst F. Schroeder, look for: efs
 * + added code for decoding free format,
 *   especially in Layer III.
 *   The first sync word is examined, and when
 *   free format is detected, the actual number
 *   of slots is determined from the distance
 *   of the following sync word.
 *
 * Revision 1.13  2003/11/13 12:20:29  tmm01
 *   by Ernst F. Schroeder, look for: efs
 * + added routines for CRC check in Layer III
 *   CRC errors are displayed and counted -
 *   no further action is taken
 *
 * Revision 1.12  2003/10/13 13:52:08  tmm01
 *   by Marc Klein Middelink, look for: mkm
 * + adapted pick_table() to handle free format
 *
 * Revision 1.11  2003/05/13 17:15:32  sps
 *  + bug fix in SubBandSynthesis() (previously introduced while changing
 *    to automatic arrays (was previously malloc))
 *
 *  + revision of the out_fifo interface (only the error code is returned,
 *    the clipping information became a function parameter)
 *
 *  + revised the code such that it can be compiled using g++
 *
 *  + functions returning an error code have now the return type
 *    ERROR_CODE. This concerns
 *     - out_fifo()
 *     - SubBandSynthesis()
 *     - III_get_side_info()
 *     - III_hufman_decode()
 *     - read_syn_window()
 *     - open_bit_stream_r()
 *     - initialize_huffman()
 *     - GetArguments()
 *     - aiff_read_headers()
 *     - aiff_seek_to_sound()
 *     - aiff_write_headers()
 *
 * Revision 1.10  2003/05/09 17:41:46  sps
 * +++ removed memory leaks.
 * +++ revised termination behaviour for unusual cases (file could not be
 * opened, no data could be written into file, memory allocation failed). A
 * couple of functions now return an error code, that leads to a clean
 * termination in the top level function.
 * +++ simplified legacy write functionality.
 * +++ removed some unnecessary "extern" declaration in function
 * declarations in header files
 *
 * Revision 1.9  2003/04/30 17:39:51  sps
 * +++ fixed aiff header for native output +++ fixed rounding for 16 bit
 * native output (by this, separated rounding in individual function
 * castDoubleToLong(), which is now called somewhat later (not anymore in
 * SubBandSynthesis(), but in out_fifo())) +++ made sure that the last
 * samples are written into the output file +++ made sure that decoding
 * starts only if file opening was successfull +++ added AFsp library
 * support, that allows the output of WAVE files. The output of raw and
 * aiff files is still supported, but is in this case also handled by the
 * AFsp library +++ Note: If the AFsp lib is used, any particular support
 * of erroneous signals is disabled +++
 *
 * Revision 1.8  2003/03/11 10:07:18  sps
 * committed contribution from Mike Coleman to remove certain kinds of
 * compiler warnings
 *
 * Revision 1.7  2002/11/21 18:35:25  sps
 * removed log message from header (it caused again a compiler warning)
 *
 * Revision 1.6  2002/11/21 18:31:22  sps
 *
 * Revision 1.5  2002/11/14 13:04:06  tmm01
 * warnings removed (Microsoft Visual Studio V.6.0)  23.10.2 W.Voessing
 *
 * Revision 1.2  1996/03/28 03:13:37  rowlands
 * Merged layers 1-2 and layer 3 revisions
 *
 * Revision 1.1  1996/02/14 03:45:52  rowlands
 * Initial revision
 *
 * Received from FhG
 **********************************************************************/
/**********************************************************************
 *   date   programmers         comment                               *
 * 2/25/91  Doulas Wong,        start of version 1.0 records          *
 *          Davis Pan                                                 *
 * 5/10/91  W. Joseph Carter    Created this file for all common      *
 *                              functions and global variables.       *
 *                              Ported to Macintosh and Unix.         *
 *                              Added Jean-Georges Fritsch's          *
 *                              "bitstream.c" package.                *
 *                              Added routines to handle AIFF PCM     *
 *                              sound files.                          *
 *                              Added "mem_alloc()" and "mem_free()"  *
 *                              routines for memory allocation        *
 *                              portability.                          *
 *                              Added routines to convert between     *
 *                              Apple SANE extended floating point    *
 *                              format and IEEE double precision      *
 *                              floating point format.  For AIFF.     *
 * 02jul91 dpwe (Aware Inc)     Moved allocation table input here;    *
 *                              Tables read from subdir TABLES_PATH.  *
 *                              Added some debug printout fns (Write*)*
 * 7/10/91 Earle Jennings       replacement of the one float by FLOAT *
 *                              port to MsDos from MacIntosh version  *
 * 8/ 5/91 Jean-Georges Fritsch fixed bug in open_bit_stream_r()      *
 *10/ 1/91 S.I. Sudharsanan,    Ported to IBM AIX platform.           *
 *         Don H. Lee,                                                *
 *         Peter W. Farrett                                           *
 *10/3/91  Don H. Lee           implemented CRC-16 error protection   *
 *                              newly introduced functions are        *
 *                              I_CRC_calc, II_CRC_calc and           *
 *                              update_CRC. Additions and revisions   *
 *                              are marked with dhl for clarity       *
 *10/18/91 Jean-Georges Fritsch fixed bug in update_CRC(),            *
 *                              II_CRC_calc() and I_CRC_calc()        *
 * 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
 *                              important fixes involved changing     *
 *                              16-bit ints to long or unsigned in    *
 *                              bit alloc routines for quant of 65535 *
 *                              and passing proper function args.     *
 *                              Removed "Other Joint Stereo" option   *
 *                              and made bitrate be total channel     *
 *                              bitrate, irrespective of the mode.    *
 *                              Fixed many small bugs & reorganized.  *
 * 3/20/92 Jean-Georges Fritsch  fixed bug in start-of-frame search   *
 * 6/15/92 Juan Pineda          added refill_buffer(bs) "n"           *
 *                              initialization                        *
 * 7/08/92 Susanne Ritscher     MS-DOS, MSC6.0 port fixes             *
 * 7/27/92 Mike Li               (re-)Port to MS-DOS                  *
 * 8/19/92 Soren H. Nielsen     Fixed bug in I_CRC_calc and in        *
 *                              II_CRC_calc.  Added function: new_ext *
 *                              for better MS-DOS compatability       *
 * 3/10/93 Kevin Peterson       changed aiff_read_headers to handle   *
 *                              chunks in any order.  now returns     *
 *                              position of sound data in file.       *
 * 3/31/93 Jens Spille          changed IFF_* string compares to use  *
 *                              strcmp()                              *
 * 5/30/93 Masahiro Iwadare     removed the previous modification     *
 *                              for UNIX.                             *
 * 8/27/93 Seymour Shlien,      Fixes in Unix and MSDOS ports,        *
 *         Daniel Lauzon, and                                         *
 *         Bill Truerniet                                             *
 *--------------------------------------------------------------------*
 *  8/24/93 Masahiro Iwadare    Included IS modification in Layer III.*
 *                              Changed for 1 pass decoding.          *
 *  9/07/93 Toshiyuki Ishino    Integrated Layer III with Ver 3.9.    *
 *--------------------------------------------------------------------*
 * 11/20/93 Masahiro Iwadare    Integrated Layer III with Ver 4.0.    *
 *--------------------------------------------------------------------*
 *  7/14/94 Juergen Koller      rewind of bitbuffer added             *
 *--------------------------------------------------------------------*
 *  6/12/95 Soeren H. Nielsen   Bug fix in new_ext().                 *
 *  7/11/95 Soeren H. Nielsen   Changes for MPEG-2 LSF Layer I and II *
 *--------------------------------------------------------------------*
 *  9/23/04 Olaf Kaehler        move tables to tables.h               *
 * -------------------------------------------------------------------*
 * 01/13/06 Eugen Dodenhoeft     added MPEG_2_5 support               *
 **********************************************************************/


#if 0
  /*TODO*/
  CHANNEL_CONFIG_ERROR,     /* error in channel configuration */
  SYNTHESIS_WINDOW_ERROR,   /* error in synthesis window table */
  WRITE_FILE_ERROR,         /* error writing output file */
  READ_FILE_ERROR,          /* error reading input file */
  SIDE_INFO_ERROR,          /* error in side info */
  HUFFMAN_TABLE_ERROR,      /* error in Huffman table */
  COMMAND_LINE_ERROR,       /* error in command line */
  AIFF_PROCESSING_ERROR,    /* error dealing with AIFF header */
  CRC_ERROR,                /* error in bitstream (ep) */
  WRITE_BUFFER_ERROR,       /* error signalizing buffer overflow */
  READ_BUFFER_ERROR         /* error signalizing buffer underrun */
#endif



/***********************************************************************
*
*  Global Include Files
*
***********************************************************************/


#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "common.h"


/***********************************************************************
*
*  Global Variable Definitions
*
***********************************************************************/




/***********************************************************************
*
*  Global Function Definitions
*
***********************************************************************/

/*
void WriteHdr( frameParameters *fr_ps, FILE *s )
{
   frameHeader *info = &fr_ps->header;

   fprintf(s, "HDR: s=FFF, id=%X, l=%X, ep=%s, br=%X, sf=%X, pd=%X, ",
           info->version, info->lay, ((info->error_protection) ? "on" : "off"),
           info->bitrate_index, info->sampling_frequency, info->padding);
   fprintf(s, "pr=%X, m=%X, js=%X, c=%X, o=%X, e=%X\n",
           info->extension, info->mode, info->mode_ext,
           info->copyright, info->original, info->emphasis);
   fprintf(s, "alg.=%s, layer=%s, tot bitrate=%d, sfrq=%.1f\n",
           version_names[info->version], layer_names[info->lay-1],
           bitrate[info->version][info->lay-1][info->bitrate_index],
           s_freq[info->version][info->sampling_frequency]);
   fprintf(s, "mode=%s, sblim=%d, jsbd=%d, ch=%d\n",
           mode_names[info->mode], fr_ps->sblimit, fr_ps->jsbound, fr_ps->channels);
   fflush(s);
}

void WriteBitAlloc(
                   unsigned int bit_alloc[2][SBLIMIT],
                   frameParameters *f_p,
                   FILE *s )
{
int i,j;
int st = f_p->channels;
int sbl = f_p->sblimit;
int jsb = f_p->jsbound;

    fprintf(s, "BITA ");
    for(i=0; i<sbl; ++i) {
        if(i == jsb) fprintf(s,"-");
        for(j=0; j<st; ++j)
            fprintf(s, "%1x", bit_alloc[j][i]);
    }
    fprintf(s, "\n");   fflush(s);
}
*/
/* TODO use for debug */
/*
void WriteScale(
                unsigned int bit_alloc[2][SBLIMIT],
                unsigned int scfsi[2][SBLIMIT],
                unsigned int scalar[2][3][SBLIMIT],
                frameParameters *fr_ps,
                FILE *s )
{
  int stereo  = fr_ps->channels;
  int sblimit = fr_ps->sblimit;
  int lay     = fr_ps->header.lay;
  int i,j,k;

        if(lay == 2) {
            fprintf(s, "SFSI ");
            for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
                if (bit_alloc[k][i])  fprintf(s,"%d",scfsi[k][i]);
            fprintf(s, "\nSCFs ");
            for (k=0;k<stereo;k++) {
                for (i=0;i<sblimit;i++)
                    if (bit_alloc[k][i])
                        switch (scfsi[k][i]) {
                          case 0: for (j=0;j<3;j++)
                                  fprintf(s,"%2d%c",scalar[k][j][i],
                                          (j==2)?';':'-');
                                  break;
                          case 1:
                          case 3: fprintf(s,"%2d-",scalar[k][0][i]);
                                  fprintf(s,"%2d;",scalar[k][2][i]);
                                  break;
                          case 2: fprintf(s,"%2d;",scalar[k][0][i]);
                        }
                fprintf(s, "\n");
            }
        }
        else{   / * lay == 1 *
            fprintf(s, "SCFs ");
            for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
                if (bit_alloc[k][i])  fprintf(s,"%2d;",scalar[k][0][i]);
            fprintf(s, "\n");
        }
}
*/

/*
void WriteSamples(
                  int ch,
                  unsigned int sample[SBLIMIT],
                  unsigned int bit_alloc[SBLIMIT],
                  frameParameters *fr_ps,
                  FILE *s )
{
int i;
int stereo = fr_ps->channels;
int sblimit = fr_ps->sblimit;

        fprintf(s, "SMPL ");
        for (i=0;i<sblimit;i++)
                if ( bit_alloc[i] != 0)
                    fprintf(s, "%d:", sample[i]);
        if(ch==(stereo-1) )     fprintf(s, "\n");
        else                    fprintf(s, "\t");
}

int NumericQ( char *s) / * see if a string lookd like a numeric argument *
{
char    c;

    while( (c = *s++)!='\0' && isspace((int)c)) / * strip leading ws *
        ;
    if( c == '+' || c == '-' )
        c = *s++;               / * perhaps skip leading + or - *
    return isdigit((int)c);
}
*/




