/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ??? 
 * 
 * and edited by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 *
 * $Id: common.h,v 1.26 2007/10/02 08:42:20 matejkml Exp $
 *
 * $Log: common.h,v $
 * Revision 1.26  2007/10/02 08:42:20  matejkml
 * bugfix: added MUTE define
 *
 * Revision 1.25  2007/09/27 12:57:51  matejkml
 * moved most stuff to right places
 *
 * Revision 1.24  2007/03/29 14:00:26  matejkml
 * added debugging
 *
 * Revision 1.23  2007/01/12 15:01:05  matejkml
 * Added frameInit function
 *
 * Revision 1.22  2007/01/10 13:36:22  matejkml
 * bugfix2
 *
 * Revision 1.17  2006/03/10 15:11:55  dodenhen
 * Added MPEG_2_5 support
 *
 * Revision 1.16  2004/10/04 14:08:43  kaehleof
 * added switch for alias reduction in mixed blocks
 *
 * Revision 1.15  2004/09/29 14:57:09  kaehleof
 * moved the decoder tables into tables.c and .h;
 * no more tables/ directory needed now;
 * create_tables.c was used to create tables.c and .h
 * from the files in the tables/ directory;
 *
 * Revision 1.14  2003/11/17 15:02:15  tmm01
 *   by Ernst F. Schroeder, look for: efs
 * + added code for decoding free format,
 *   especially in Layer III.
 *   The first sync word is examined, and when
 *   free format is detected, the actual number
 *   of slots is determined from the distance
 *   of the following sync word.
 *
 * Revision 1.13  2003/11/13 12:20:30  tmm01
 *   by Ernst F. Schroeder, look for: efs
 * + added routines for CRC check in Layer III
 *   CRC errors are displayed and counted -
 *   no further action is taken
 *
 * Revision 1.12  2003/05/13 17:15:33  sps
 *  + bug fix in SubBandSynthesis() (previously introduced while changing
 *    to automatic arrays (was previously malloc))
 *
 *  + revision of the out_fifo interface (only the error code is returned,
 *    the clipping information became a function parameter)
 *
 *  + revised the code such that it can be compiled using g++
 *
 *  + functions returning an error code have now the return type
 *    ERROR_CODE. This concerns
 *     - out_fifo()
 *     - SubBandSynthesis()
 *     - III_get_side_info()
 *     - III_hufman_decode()
 *     - read_syn_window()
 *     - open_bit_stream_r()
 *     - initialize_huffman()
 *     - GetArguments()
 *     - aiff_read_headers()
 *     - aiff_seek_to_sound()
 *     - aiff_write_headers()
 *
 * Revision 1.11  2003/05/09 17:41:47  sps
 * +++ removed memory leaks.
 * +++ revised termination behaviour for unusual cases (file could not be
 * opened, no data could be written into file, memory allocation failed). A
 * couple of functions now return an error code, that leads to a clean
 * termination in the top level function.
 * +++ simplified legacy write functionality.
 * +++ removed some unnecessary "extern" declaration in function
 * declarations in header files
 *
 * Revision 1.10  2003/05/06 16:24:35  sps
 * +++ terminates if the number of channels in the bitstream exceeds the
 * number of initialized channels +++ allows to specify the number of
 * channels to be initialized using the command line option -n +++ performs
 * smart rounding from double to float prior to pass the data to the afsp
 * library write function +++ added function MyClose to assure that all
 * memory is free?d prior to exit
 *
 * Revision 1.9  2003/04/30 17:39:52  sps
 * +++ fixed aiff header for native output +++ fixed rounding for 16 bit
 * native output (by this, separated rounding in individual function
 * castDoubleToLong(), which is now called somewhat later (not anymore in
 * SubBandSynthesis(), but in out_fifo())) +++ made sure that the last
 * samples are written into the output file +++ made sure that decoding
 * starts only if file opening was successfull +++ added AFsp library
 * support, that allows the output of WAVE files. The output of raw and
 * aiff files is still supported, but is in this case also handled by the
 * AFsp library +++ Note: If the AFsp lib is used, any particular support
 * of erroneous signals is disabled +++
 *
 * Revision 1.8  2003/03/27 18:36:28  sps
 * MAX_NAME_SIZE seemed a little bit small meanwhile ...
 * -#define         MAX_NAME_SIZE           81
 * +#define         MAX_NAME_SIZE           512
 *
 * Revision 1.7  2002/11/21 18:35:25  sps
 * removed log message from header (it caused again a compiler warning)
 *
 * Revision 1.6  2002/11/21 18:31:22  sps
 *
 * Revision 1.5  2002/11/14 13:04:07  tmm01
 * warnings removed (Microsoft Visual Studio V.6.0)  23.10.2 W.Voessing
 *
 * Revision 1.3  1996/03/28 03:13:37  rowlands
 * Merged layers 1-2 and layer 3 revisions
 *
 * Revision 1.2  1996/02/14 05:18:36  rowlands
 * Cleanups.
 *
 * Revision 1.1  1996/02/14 03:45:52  rowlands
 * Initial revision
 *
 * Received from FhG
 **********************************************************************/
/**********************************************************************
 *   date   programmers         comment                               *
 * 2/25/91  Doulas Wong,        start of version 1.0 records          *
 *          Davis Pan                                                 *
 * 5/10/91  W. Joseph Carter    Reorganized & renamed all ".h" files  *
 *                              into "common.h" and "encoder.h".      *
 *                              Ported to Macintosh and Unix.         *
 *                              Added additional type definitions for *
 *                              AIFF, double/SANE and "bitstream.c".  *
 *                              Added function prototypes for more    *
 *                              rigorous type checking.               *
 * 27jun91  dpwe (Aware)        Added "alloc_*" defs & prototypes     *
 *                              Defined new struct 'frameParameters'.    *
 *                              Changed info.stereo to info.mode_ext  *
 *                              #define constants for mode types      *
 *                              Prototype arguments if PROTO_ARGS     *
 * 5/28/91  Earle Jennings      added MS_DOS definition               *
 *                              MsDos function prototype declarations *
 * 7/10/91  Earle Jennings      added FLOAT definition as double      *
 *10/ 3/91  Don H. Lee          implemented CRC-16 error protection   *
 * 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
 *                              important fixes involved changing     *
 *                              16-bit ints to long or unsigned in    *
 *                              bit alloc routines for quant of 65535 *
 *                              and passing proper function args.     *
 *                              Removed "Other Joint Stereo" option   *
 *                              and made bitrate be total channel     *
 *                              bitrate, irrespective of the mode.    *
 *                              Fixed many small bugs & reorganized.  *
 *                              Modified some function prototypes.    *
 *                              Changed BUFFER_SIZE back to 4096.     *
 * 7/27/92  Michael Li          (re-)Ported to MS-DOS                 *
 * 7/27/92  Masahiro Iwadare    Ported to Convex                      *
 * 8/07/92  mc@tv.tek.com                                             *
 * 8/10/92  Amit Gulati         Ported to the AIX Platform (RS6000)   *
 *                              AIFF string constants redefined       *
 * 8/27/93 Seymour Shlien,      Fixes in Unix and MSDOS ports,        *
 *         Daniel Lauzon, and                                         *
 *         Bill Truerniet                                             *
 *--------------------------------------------------------------------*
 * 4/23/92  J. Pineda           Added code for Layer III.             *
 * 11/9/92  Amit Gulati         Added defines for layerIII stereo     *
 *                              modes.                                *
 *  8/24/93 Masahiro Iwadare    Included IS modification in Layer III.*
 *                              Changed for 1 pass decoding.          *
 *  9/07/93 Toshiyuki Ishino    Integrated Layer III with Ver 3.9.    *
 *--------------------------------------------------------------------*
 * 11/20/93 Masahiro Iwadare    Integrated Layer III with Ver 4.0.    *
 *--------------------------------------------------------------------*
 *  7/14/94 Juergen Koller      Fix for HPUX an IRIX in AIFF-Strings  *
 *--------------------------------------------------------------------*
 *  6/12/95 Soeren H. Nielsen   Bug fix in new_ext().                 *
 *  7/11/95 Soeren H. Nielsen   Added defs. for MPEG-2 LSF            *
 *--------------------------------------------------------------------*
 *          Roland Bitto	Adapted to MPEG2 low sampling rate    *
 *--------------------------------------------------------------------*
 * 01/09/06 Eugen Dodenhoeft    Added MPEG_2_5 support                *
 **********************************************************************/

#ifndef COMMON_H
#define COMMON_H

/***********************************************************************
*
*  Global Definitions
*
***********************************************************************/

/* General Definitions */

#ifdef MPEG_2_5
#define VERSION_COUNT 4
#else
#define VERSION_COUNT 2
#endif

#define         FALSE                   0
#define         TRUE                    1
#define         NULL_CHAR               '\0'

#define         MAX_U_32_NUM            0xFFFFFFFF

#ifndef PI
#define         PI                      3.14159265358979
#endif

#define         PI4                     PI/4
#define         PI64                    PI/64

#define			MUTE					0
#define         MONO                    1
#define         STEREO                  2
#define         SBLIMIT                 32
#define         SSLIMIT                 18
#define         HAN_SIZE                512
#define         SCALE_BLOCK             12
#define         SCALE_RANGE             64

#define         CRC16_POLYNOMIAL        0x8005







/***********************************************************************
*
*  Global Type Definitions
*
***********************************************************************/

/* Double and SANE Floating Point Type Definitions */

typedef struct  IEEE_DBL_struct {
    unsigned long   hi;
    unsigned long   lo;
} IEEE_DBL;

typedef struct  SANE_EXT_struct {
    unsigned long   l1;
    unsigned long   l2;
    unsigned short  s1;
} SANE_EXT;


/* Layer III side information. */
typedef struct {
	unsigned part2_3_length;
	unsigned big_values;
	unsigned global_gain;
	unsigned scalefac_compress;
	unsigned window_switching_flag;
	unsigned block_type;
	unsigned mixed_block_flag;
	unsigned table_select[3];
	unsigned subblock_gain[3];
	unsigned region0_count;
	unsigned region1_count;
	unsigned preflag;
	unsigned scalefac_scale;
	unsigned count1table_select;
} my_gr_info ;

struct gr_info_s {
  unsigned part2_3_length;
  unsigned big_values;
  unsigned global_gain;
  unsigned scalefac_compress;
  unsigned window_switching_flag;
  unsigned block_type;
  unsigned mixed_block_flag;
  unsigned table_select[3];
  unsigned subblock_gain[3];
  unsigned region0_count;
  unsigned region1_count;
  unsigned preflag;
  unsigned scalefac_scale;
  unsigned count1table_select;
};

struct ch_info_s{
  unsigned scfsi[4];
  struct gr_info_s gr[2];
};

typedef struct III_side_info_s {
  unsigned main_data_begin;
  unsigned private_bits;
  struct ch_info_s ch[2];
} III_side_info_t;

/* Layer III scale factors. */

typedef struct III_scalefac_s {
  int l[23];            /* [cb] */
  int s[3][13];         /* [window][cb] */
} III_scalefac_t[2];    /* [ch] */



typedef struct 
{
  unsigned int steps;
  unsigned int bits;
  unsigned int group;
  unsigned int quant;
} al_table[SBLIMIT][16];


#endif

