/******************************************************************************
 * 
 * This software module was originally developed by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * and edited by 
 * 
 * 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "debug.h"

#define MAX_PREFIX 128

typedef struct TAG_debug
{
  errorLevel  level;
  FILE        *stream;
  int         stackCounter;
  char*       functionStack[255];
  char        prefix[MAX_PREFIX];
} debug_t;


HANDLE_debug debugInit(errorLevel level, FILE* stream)
{
  HANDLE_debug dbg;
  
  if ( (dbg = (HANDLE_debug) malloc(sizeof(debug_t))) == NULL) 
  {
    fprintf(stderr, "ERROR: failed allocating memory for debug object.");
    return NULL; /* MEMORY_ALLOCATION_ERROR */
  }
  
  dbg->stackCounter = 0;
  
  debugSetLevel(dbg, level);
  debugSetStream(dbg, stream);
  debugSetPrefix(dbg, "");
  
  return dbg;
}

void debugSetLevel(HANDLE_debug dbg, errorLevel level)
{
  dbg->level = level;
}

void debugSetStream(HANDLE_debug dbg, FILE *stream)
{
  dbg->stream = stream;
}

void debugEnter(HANDLE_debug dbg, char *function)
{
  if (dbg->stackCounter >= 256)
  {
    fprintf(dbg->stream, "ERROR: debugEnter: stack overflow! Terminating.\n");
    exit(EXIT_FAILURE);
  }
  dbg->functionStack[dbg->stackCounter++] = function;
  debugPrintf(dbg, D_DEBUG_L3, "Entering function");
}

void debugLeave(HANDLE_debug dbg)
{
  char *function;
  if (dbg->stackCounter <= 0)
  {
    fprintf(stderr, "WARNING: debug stack underrun!");
    return;
  }
  debugPrintf(dbg, D_DEBUG_L3, "Leaving function");
  dbg->functionStack[dbg->stackCounter--];
}

void debugPrintf(HANDLE_debug dbg, errorLevel level, char* format, ...)
{
  const char* debugString[4] = {"ERROR", "WARNING", "NOTICE", "DEBUG"};
  const int   dStrRedirect[10] = { 0, 1, 2, 2, 2, 2, 3, 3, 3, 3 };
  
  if (level <= dbg->level)
  {
    /* print prefix */
    if (strlen(dbg->prefix)>0)
    {
      fprintf(dbg->stream, "%s: ", dbg->prefix);
    }
    
    /* in DEBUG mode show function name */
    if (dbg->level >= D_DEBUG && dbg->stackCounter > 0)
    {
      fprintf(dbg->stream, "%s: ", dbg->functionStack[dbg->stackCounter-1]);
    }

    /* print debugString e.g. "ERROR: " */
    if (level < D_NOTICE || dbg->level >= D_DEBUG)
      fprintf(dbg->stream, "%s: ", debugString[dStrRedirect[level]]);

    /* show sting */
    {
      va_list args;
      va_start (args, format);
      vfprintf (dbg->stream, format, args);
      va_end (args);
    }
      
    fprintf(dbg->stream, "\n");
  }
}

void debugSetPrefix(HANDLE_debug dbg, char* format, ...)
{
  va_list args;
  va_start (args, format);
  vsprintf (dbg->prefix, format, args);
  va_end (args);
}

void debugFree(HANDLE_debug dbg)
{
  if (dbg) free(dbg);
}

