/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 *
 *   ???
 * 
 * and edited by 
 * 
 *   Matejko Michael (Fraunhofer IIS) 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**@file
 * This is a documentation of the decoder core.
 */

/******************************************************************************
 * includes
 ******************************************************************************/

#include <stdlib.h>
#include "decLayer123.h"
#include "common.h"
#include "debug.h"
#include "decoder.h"
#include "huffman.h"
#include "bitBuffer.h"
#include "pcmBuffer.h"
#include "frameData.h"

typedef struct TAG_decLayer123
{
  HANDLE_debug      dbg;
  synData           syn;                    /* synthesis data */
  unsigned int      scalefac_buffer[54];
  unsigned int      scalefac_IIP_buffer[54];
  antialiasData     aa;
  mdctData          md;
  double            hybrid_prevblck[2][SBLIMIT][SSLIMIT]; 
  decData           decData;
  
 
  /* statistical data */
  unsigned long     frameCount;     /**< count decoded frames */
  unsigned long     bitCount;       /**< bits delivered to decoder core */
  unsigned long     sampleCount;    /**< samples decoded by decoder core */
  unsigned long     errorCount;     /**< counter for errors occured */
  unsigned long     maxBitrate;
  unsigned long     minBitrate;
  
  /* decoder core options */
  unsigned short    topSb;
  unsigned short    antialiasMixedBlocks;
  
} decLayer123_t;


/******************************************************************************
 * private functions
 ******************************************************************************/

static
int pcmBufferFill(HANDLE_pcmBuffer buf, decData *dd, int num)
{
  int             i,j,l;
  int             stereo = dd->frameData.channels;
  
  for (i = 0; i < num; i++) {          /* loop over samples per subband */
    for (j = 0; j < SBLIMIT; j++) {    /* loop over number of subbands */
      for (l = 0; l < stereo; l++) {   /* loop over number of channels */
        if ( pcmBufferPut(buf, dd->pcm_sample[l][i][j]) != PCM_OK )
          return 0;
      }  /* for (l = 0; l < stereo; l++) */
    }    /* for (j = 0; j < sblimit; j++) */
  }      /* for (i = 0; i < num; i++) */
  
  return 1;
}

/******************************************************************************
 * public functions
 ******************************************************************************/

char *decLayer123Info (FILE *helpStream)
{
  if (helpStream != NULL)
  {
    fprintf(helpStream,
        "MPEG-1/2"
#ifdef MPEG_2_5
        "/2.5"
#endif        
        " audio decoder core\n"
        "options:\n"
        " -s sb    resynth only up to this sb (L1, L2, debugging only)\n"
        " -a       antialias in mixed blocks turned off (ISO/IEC 11172-3 conform)\n"
        " -d       show debug output\n"
        " -dd      show more debug output\n"
    );
  }
  return "MPEG12DecoderCore"; /* TODO */
}


HANDLE_decLayer123 decLayer123Init(char *decPara, FILE* debugStream)
{
  static const char* on_off[2] = {"off", "on"};
  HANDLE_decLayer123 dec;
  HANDLE_debug dbg;
  int i, j;
  char *next;
  
  dbg = debugInit(D_NOTICE, debugStream);
  debugSetPrefix(dbg, "Decoder core"); 
  debugEnter(dbg, "decLayer123Init()");
   
  /* create decoder handler struct */
  if ( (dec = (HANDLE_decLayer123) malloc(sizeof(decLayer123_t))) == NULL) 
  {
    debugPrintf(dbg, D_ERROR, "Error allocating memory");
    debugLeave(dbg);
    return NULL; /* MEMORY_ALLOCATION_ERROR */
  }

  for (i=0;i<HAN_SIZE;i++) for (j=0;j<2;j++) dec->decData.w[j][i] = 0.0;
  
  createSynFilter(&dec->syn);
  III_antialias_init(&dec->aa);
  initMdct(&dec->md);
  III_hybrid_init(dec->hybrid_prevblck);
  dec->decData.tableNumber = -1;
  
  dec->dbg = dbg;
  
  /* Init stat values */
  dec->bitCount = 0;
  dec->sampleCount = 0;
  dec->errorCount = 0;  

  
  /* Parse decoder parameters
   * ======================== */
  
  /* Init values */
  dec->antialiasMixedBlocks = 1;
  dec->topSb = 0;

  for (next=decPara; *next != '\0'; next++)
  {
    switch (*next)
    {
      case '-':
        switch(*(++next))
        {
          case 'd':
            if (*(next+1)=='d')
            {
              next++;
              debugSetLevel(dbg, D_DEBUG);
            }
            else
              debugSetLevel(dbg, D_NOTICE_L1);
            break;
            
          case 's':
            dec->topSb = strtol( next+1, &next, 10 ); next--;
            if(dec->topSb<1 || dec->topSb>SBLIMIT) {
              debugPrintf(dbg, D_WARNING, "-s %i: band not between 1..%d, setting ignored", dec->topSb, SBLIMIT);
              dec->topSb = 0;
              /*
              return NULL;
              */
            }
            break;
          
          case 'a':
            dec->antialiasMixedBlocks = 0;
            break;

          default:
            debugPrintf(dbg, D_WARNING, "Unrecognized decoder parameter string: \"-%c\" in \"%s\"",*next, decPara);
            /*
            debugLeave(dbg);
            return NULL;
            */
        }
        break;
        
      case ' ':
        /* whitespace is ok */
        break;

      default:
        debugPrintf(dbg, D_WARNING, "Unrecognized decoder parameter string \"%c\" in \"%s\"",*next, decPara);
        /*
          debugLeave(dbg);
          return NULL; 
        */
    }
  }
  
  debugPrintf(dbg, D_NOTICE_L1, "Debug output on");
  debugPrintf(dbg, D_NOTICE_L1, "Anti-aliased mixed blocks: %s", on_off[dec->antialiasMixedBlocks]);
  debugPrintf(dbg, D_NOTICE_L1, "Top subband: %d", dec->topSb);

  debugLeave(dbg);
  return dec;
}


ERROR_CODE_decLayer123 decLayer123Frame (
  HANDLE_decLayer123  dec,
  HANDLE_bitBuffer    audioFrame,
  HANDLE_pcmBuffer    pcmBuffer
)
{
  ERROR_CODE_decLayer123  errorCode;
  int               i, j, k; 
  unsigned int      scale_index[2][3][SBLIMIT];
  unsigned int      bit_alloc[2][SBLIMIT], scfsi[2][SBLIMIT];
  unsigned int      crc_error_count;
  III_scalefac_t    III_scalefac;
  III_side_info_t   III_side_info;
  unsigned long     bitBufferErrors;
  unsigned long     currentBitrate;
  int               currentChannels;

 
  debugEnter(dec->dbg, "decLayer123Frame()");
  
  
  /* Clean sample buffer */
  pcmBufferReset(pcmBuffer);
  
  /* Get frame Data */
  getFrameData(&dec->decData.frameData, audioFrame);
  decodeFrameData(&dec->decData);

  currentChannels = dec->decData.frameData.channels;
  currentBitrate  = dec->decData.frameData.bitrate;

  debugPrintf(dec->dbg, D_NOTICE_L1, "Frame [%d] Bitrate: %d", dec->frameCount, currentBitrate);

  
  crc_error_count = 0;
 

  /* Init values in first frame */
  if (dec->frameCount == 0)
  {
    dec->maxBitrate = currentBitrate;
    dec->minBitrate = currentBitrate;
  }


  switch (dec->decData.frameData.lay) 
  {
    case LAYER1 :
      I_decode_bitalloc(audioFrame, bit_alloc, &dec->decData);
      I_decode_scale(audioFrame, bit_alloc, scale_index, &dec->decData);
        
      if (dec->decData.frameData.error_protection) 
      {
        
        if (I_CRC_calc(&dec->decData, bit_alloc)) 
        {
          crc_error_count++;
          dec->errorCount++;
          /* TODO recover ???? */
          debugLeave(dec->dbg);
          return L123_CRC_ERROR;
        }
        else crc_error_count = 0;
      }
        
      for (i=0;i<SCALE_BLOCK;i++) 
      {
        I_buffer_sample(audioFrame, dec->decData.sample, bit_alloc,&dec->decData);
        I_dequantize_sample(dec->decData.sample, dec->decData.fraction,bit_alloc,&dec->decData);
        I_denormalize_sample(dec->decData.fraction,scale_index,&dec->decData);
        
        if(dec->topSb>0)        /* clear channels to 0 */
          for(j=dec->topSb; j<(dec->decData.sblimit); ++j)
            for(k=0; k<currentChannels; ++k)
              dec->decData.fraction[k][0][j] = 0;
        
        for (j=0;j<currentChannels;j++) 
        {
          subBandSynthesis (&dec->syn, &dec->decData.fraction[j][0][0], j,
                                       &dec->decData.pcm_sample[j][0][0]);
        }
        
        if (!pcmBufferFill(pcmBuffer, &dec->decData, 1))
        {
          debugLeave(dec->dbg);
          return L123_UNKNOWN_ERROR; /* TODO */
        }
      }
      
      break;
      
    case LAYER2 : 
      II_decode_bitalloc(audioFrame, bit_alloc, &dec->decData);
      II_decode_scale(audioFrame, scfsi, bit_alloc, scale_index, &dec->decData);

      if (dec->decData.frameData.error_protection)
      {
        if (II_CRC_calc(&dec->decData, bit_alloc, scfsi)) 
        {
          crc_error_count++;
          dec->errorCount++;
          /* recover_CRC_error(dec->pcmBuffer, crc_error_count, dec->samplesPerFrame); */
          /* TODO signalize CRC error!?? */
          break;
        }
        else crc_error_count = 0;
      }

      for (i=0;i<SCALE_BLOCK;i++) 
      {
        II_buffer_sample(audioFrame,dec->decData.sample,bit_alloc,&dec->decData);
        II_dequantize_sample(dec->decData.sample,bit_alloc,dec->decData.fraction,&dec->decData);
        II_denormalize_sample(dec->decData.fraction,scale_index,&dec->decData,i>>2);

        if(dec->topSb>0)        /* debug : clear channels to 0 */
          for(j=dec->topSb; j<dec->decData.sblimit; ++j)
            for(k=0; k<currentChannels; ++k)
              dec->decData.fraction[k][0][j] =
                dec->decData.fraction[k][1][j] =
                dec->decData.fraction[k][2][j] = 0;

        for (j=0;j<3;j++) for (k=0;k<currentChannels;k++) 
        {
          subBandSynthesis (&dec->syn, &dec->decData.fraction[k][j][0], k,
                                       &dec->decData.pcm_sample[k][j][0]);
        }
        
        if (!pcmBufferFill(pcmBuffer, &dec->decData, 3))
        {
          debugLeave(dec->dbg);
          return L123_UNKNOWN_ERROR; /* TODO */
        }
      }
      break;


    case LAYER3:
      {
        int nSlots;
        unsigned int data;
        int gr, ch, ss, sb, main_data_end, flush_main, main_data_read ;
        int  bytes_to_discard ;
        static int frame_start = 0;
        int  maxGr;
        
        if(dec->decData.frameData.version != MPEG_PHASE1) /* MPEG2 or 2.5 */
        {
          maxGr = 1;
        }
        else 
        { 
          maxGr = 2;
        }
        
        if (!III_get_side_info_crc(audioFrame, &III_side_info, &dec->decData))
        {
          debugLeave(dec->dbg);
          return L123_UNKNOWN_ERROR;
        }
        
        /*TODO */
        debugPrintf(dec->dbg, D_NOTICE_L1, "Frame [%d] MainDataBegin: %d", dec->frameCount, III_side_info.main_data_begin);
        
        
        
        
        if (dec->decData.frameData.error_protection) 
        { 
          if (dec->decData.frameData.crc != dec->decData.newCRC) 
          {
            crc_error_count++;
            dec->errorCount++;
            debugPrintf(dec->dbg, D_ERROR, "CRC error");
          }
          else crc_error_count = 0;
        }

        for ( gr=0; gr<maxGr; gr++ ) 
        {
          double lr[2][SBLIMIT][SSLIMIT],ro[2][SBLIMIT][SSLIMIT];
          
          for (ch=0; ch<currentChannels; ch++) {
            long int is[SBLIMIT][SSLIMIT];   /* Quantized samples. */ 
            int part2_start;
            part2_start = bitBufferGetRead(audioFrame);
            
            if(dec->decData.frameData.version == MPEG_PHASE1)
            {
              III_get_scale_factors(audioFrame, &III_scalefac,&III_side_info,gr,ch);
            }
            else 
            {
              III_get_LSF_scale_factors(audioFrame,
                                        &III_scalefac,
                                        &III_side_info,
                                        gr,ch,&dec->decData, 
                                        dec->scalefac_buffer,
                                        dec->scalefac_IIP_buffer);
            }
            if (!III_huffman_decode(audioFrame, is, &III_side_info, ch, gr, part2_start, &dec->decData)) 
            {
              debugLeave(dec->dbg);
              return L123_UNKNOWN_ERROR;
            }
            III_dequantize_sample(is, ro[ch], &III_scalefac,
                                  &(III_side_info.ch[ch].gr[gr]), ch, &dec->decData);
          }

          
          if(dec->decData.frameData.version == MPEG_PHASE1)
          {
            MPEG_1_III_stereo(ro, lr, &III_scalefac, &(III_side_info.ch[0].gr[gr]), &dec->decData);
          }
          else 
          {
            MPEG_2_III_stereo(ro, lr, &III_scalefac, &(III_side_info.ch[0].gr[gr]), &(III_side_info.ch[1].gr[gr]), &dec->decData,dec->scalefac_IIP_buffer);
          }
          
          for (ch=0; ch<currentChannels; ch++) 
          { 
            double re[SBLIMIT][SSLIMIT];
            double hybridIn[SBLIMIT][SSLIMIT];/* Hybrid filter input */
            double hybridOut[SBLIMIT][SSLIMIT];/* Hybrid filter out */
            double polyPhaseIn[SBLIMIT];     /* PolyPhase Input. */
            
            III_reorder (lr[ch],re,&(III_side_info.ch[ch].gr[gr]),
                         &dec->decData);
            
            III_antialias(re, hybridIn, 
                          &(III_side_info.ch[ch].gr[gr]),
                          dec->antialiasMixedBlocks,
                          &dec->aa);
            
            for (sb=0; sb<SBLIMIT; sb++) /* Hybrid synthesis. */ 
            { 
              III_hybrid(hybridIn[sb], hybridOut[sb], sb, ch,&(III_side_info.ch[ch].gr[gr]), &dec->md, dec->hybrid_prevblck);
            }
            
            for (ss=0;ss<18;ss++) /* Frequency inversion for polyphase.*/
              for (sb=0; sb<SBLIMIT; sb++)
                if ((ss%2) && (sb%2))
                  hybridOut[sb][ss] = -hybridOut[sb][ss];
            
            for (ss=0;ss<18;ss++) /* Polyphase synthesis */ 
            { 
              for (sb=0; sb<SBLIMIT; sb++)
                polyPhaseIn[sb] = hybridOut[sb][ss];
              subBandSynthesis (&dec->syn, polyPhaseIn, ch, &dec->decData.pcm_sample[ch][ss][0]);
            }
          }
          /* Output PCM sample points for one granule. */
          if (!pcmBufferFill(pcmBuffer, &dec->decData, 18))
          {
            debugLeave(dec->dbg);
            return L123_UNKNOWN_ERROR; /* TODO */
          }
        }
        break;
      }
  }

  bitBufferErrors = bitBufferGetErrors(audioFrame);
  if (bitBufferErrors > 0)
  {
    debugPrintf(dec->dbg, D_ERROR, "%lu bit buffer errors occured", bitBufferErrors);
    dec->errorCount++;
    debugLeave(dec->dbg);
    return L123_UNDERRUN_ERROR;
  }

  /* set pcm buffer */
  pcmBufferSetChannels(pcmBuffer, dec->decData.frameData.channels);
  pcmBufferSetSamplingFreq(pcmBuffer, dec->decData.frameData.sfreq * 1000);
   
  /* update counters */
  dec->frameCount++;
  dec->bitCount += bitBufferGetRead(audioFrame);
  dec->sampleCount += pcmBufferGetWritten(pcmBuffer) / pcmBufferGetChannels(pcmBuffer);

  /* update bitrate stats */
  if (currentBitrate > dec->maxBitrate) dec->maxBitrate = currentBitrate;
  if (currentBitrate < dec->minBitrate) dec->minBitrate = currentBitrate;
    
  debugLeave(dec->dbg);
  return L123_NO_ERROR;
}


void decLayer123PrintStats (HANDLE_decLayer123 dec)
{
  float bit_per_sample    = (float)dec->bitCount / dec->sampleCount;
  float real_bitrate      = bit_per_sample * dec->decData.frameData.sfreq;
  
  debugSetPrefix(dec->dbg, "");
  debugPrintf(dec->dbg, D_NOTICE,    "Decoder core stats:");
  debugPrintf(dec->dbg, D_NOTICE,    "  content format   : %s %s", dec->decData.frameData.strVersion, dec->decData.frameData.strLayer);
  debugPrintf(dec->dbg, D_NOTICE,    "  channel mode     : %s", dec->decData.frameData.strMode);
  if (dec->maxBitrate == 0)
    debugPrintf(dec->dbg, D_NOTICE,    "  signaled bitrate : free format");
  else if (dec->maxBitrate == dec->minBitrate)
    debugPrintf(dec->dbg, D_NOTICE,    "  signaled bitrate :%8u kbps", dec->maxBitrate);
  else
    debugPrintf(dec->dbg, D_NOTICE,    "  signaled bitrate : variable %lu - %lu kbps", dec->minBitrate, dec->maxBitrate);
  debugPrintf(dec->dbg, D_NOTICE,    "  decoded frames   :%8u frames",    dec->frameCount);
  debugPrintf(dec->dbg, D_NOTICE_L1, "  received bits    :%8u bit",       dec->bitCount);
  debugPrintf(dec->dbg, D_NOTICE_L1, "  decoded samples  :%8u samples",   dec->sampleCount);
  debugPrintf(dec->dbg, D_NOTICE,    "  sample frequency :%8.3f kHz",     dec->decData.frameData.sfreq);
  debugPrintf(dec->dbg, D_NOTICE,    "  bit/sample       :%8.3f b/samp",  bit_per_sample); 
  debugPrintf(dec->dbg, D_NOTICE,    "  real bitrate     :%8.3f kbps",    real_bitrate);
  if (dec->errorCount > 0) 
    debugPrintf(dec->dbg, D_NOTICE,    "   total errors    :%8d", dec->errorCount );
}


void decLayer123Free (HANDLE_decLayer123 dec)
{
  if (dec != NULL) free (dec);
}


