/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ??? 
 * 
 * and edited by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**********************************************************************
* ISO MPEG Audio Subgroup Software Simulation Group (1996)
* ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
*
* $Id: decode.c,v 1.33 2007/11/07 12:44:56 matejkml Exp $
*
* $Log: decode.c,v $
* Revision 1.33  2007/11/07 12:44:56  matejkml
* changed use_afsp to "#if 0"
*
* Revision 1.32  2007/09/27 12:59:20  matejkml
* #structural changes
*
* Revision 1.31  2007/03/29 14:08:43  matejkml
* added free mode support
*
* Revision 1.30  2007/01/12 11:42:50  matejkml
* bugfix: mpeg25 corrected
*
* Revision 1.29  2007/01/11 10:47:46  matejkml
* bugfix2
*
* Revision 1.28  2007/01/10 13:36:23  matejkml
* bugfix2
*
* Revision 1.26  2007/01/05 12:09:12  matejkml
* bugfix
*
* Revision 1.25  2007/01/05 11:59:21  matejkml
* additional changes for decoder split
*
* Revision 1.21  2006/03/10 15:11:55  dodenhen
* Added MPEG_2_5 support
*
* Revision 1.20  2004/10/04 14:08:44  kaehleof
* added switch for alias reduction in mixed blocks
*
* Revision 1.19  2004/09/29 14:57:09  kaehleof
* moved the decoder tables into tables.c and .h;
* no more tables/ directory needed now;
* create_tables.c was used to create tables.c and .h
* from the files in the tables/ directory;
*
* Revision 1.18  2004/08/10 14:34:14  sps
* added some code to allow for investigation of channel configuration
* changes, to be enabled using the define CHANNEL_INVESTIGATION in
* decoder.h
*
// * Revision 1.17  2003/11/27 13:12:04  tmm01
*   by Johannes Boehm / Ernst F. Schroeder, look for: jb&efs
* + removed bug that allowed to write outside the bounds of
*   array is[] while decoding Huffman code words
*
* Revision 1.16  2003/11/13 12:20:30  tmm01
*   by Ernst F. Schroeder, look for:   efs
* + added routines for CRC check in Layer III
*   CRC errors are displayed and counted -
*   no further action is taken
*
* Revision 1.15  2003/08/21 13:19:01  tmm01
*   by Ernst F. Schroeder, look for: efs
* + bug fix in III_dequantize_sample(): different parameter value
*   needed for MPEG-2 LSF
*
* Revision 1.14  2003/07/09 12:15:49  tmm01
* by Ernst F. Schroeder, look for: efs
* + bug fix in MPEG 1 and 2 lsf mixed blocks decoding for intensity stereo mode
*   the decision was wrong, whether the intensity bound is in long or in short
*   blocks.
*
* + 3 major adaptations in MPEG 2 intensity stereo decoding
*   for mixed, short, and long blocks
*   cleaned the code and made it more consistent where identical operations are
*   done, including a number of smaller bug fixes in these areas
*
* + bug fix in MPEG 2 lsf extraction of scalefactors for mixed blocks:
*   the number of long scalefactor bands is only 6 in MPEG 2 lsf, not 8 as in
*   MPEG 1. Also, the position of information in scalefac_IIP_buffer[] was not
*   correct for the scalefactors belonging to the scalefactor bands in short
*   blocks. They have to be moved 3 places up.
*
* Revision 1.13  2003/05/13 17:15:33  sps
*  + bug fix in SubBandSynthesis() (previously introduced while changing
*    to automatic arrays (was previously malloc))
*
*  + revision of the out_fifo interface (only the error code is returned,
*    the clipping information became a function parameter)
*
*  + revised the code such that it can be compiled using g++
*
*  + functions returning an error code have now the return type
*    ERROR_CODE. This concerns
*     - out_fifo()
*     - SubBandSynthesis()
*     - III_get_side_info()
*     - III_hufman_decode()
*     - read_syn_window()
*     - open_bit_stream_r()
*     - initialize_huffman()
*     - GetArguments()
*     - aiff_read_headers()
*     - aiff_seek_to_sound()
*     - aiff_write_headers()
*
* Revision 1.12  2003/05/13 13:53:28  sps
*  + bug fix in SubBandSynthesis() (previously introduced while changing
*    to automatic arrays (was previously malloc)
*
*  + revision of the interface to out_fifo (only the error code is
*    returned, the clipping information became a function parameter)
*
* Revision 1.11  2003/05/09 17:41:47  sps
* +++ removed memory leaks.
* +++ revised termination behaviour for unusual cases (file could not be
* opened, no data could be written into file, memory allocation failed). A
* couple of functions now return an error code, that leads to a clean
* termination in the top level function.
* +++ simplified legacy write functionality.
* +++ removed some unnecessary "extern" declaration in function
* declarations in header files
*
* Revision 1.10  2003/05/06 16:24:35  sps
* +++ terminates if the number of channels in the bitstream exceeds the
* number of initialized channels +++ allows to specify the number of
* channels to be initialized using the command line option -n +++ performs
* smart rounding from double to float prior to pass the data to the afsp
* library write function +++ added function MyClose to assure that all
* memory is free?d prior to exit
*
* Revision 1.9  2003/04/30 17:39:52  sps
* +++ fixed aiff header for native output +++ fixed rounding for 16 bit
* native output (by this, separated rounding in individual function
* castDoubleToLong(), which is now called somewhat later (not anymore in
* SubBandSynthesis(), but in out_fifo())) +++ made sure that the last
* samples are written into the output file +++ made sure that decoding
* starts only if file opening was successfull +++ added AFsp library
* support, that allows the output of WAVE files. The output of raw and
* aiff files is still supported, but is in this case also handled by the
* AFsp library +++ Note: If the AFsp lib is used, any particular support
* of erroneous signals is disabled +++
*
* Revision 1.8  2003/03/11 10:07:19  sps
* committed contribution from Mike Coleman to remove certain kinds of
* compiler warnings
*
* Revision 1.7  2002/11/21 18:35:25  sps
* removed log message from header (it caused again a compiler warning)
*
* Revision 1.6  2002/11/21 18:31:22  sps
*
* Revision 1.5  2002/11/14 13:04:07  tmm01
* warnings removed (Microsoft Visual Studio V.6.0)  23.10.2 W.Voessing
*
* Revision 1.2  1996/03/28 03:13:37  rowlands
* Merged layers 1-2 and layer 3 revisions
*
* Revision 1.1  1996/02/14 03:45:52  rowlands
* Initial revision
*
* Received from FhG
**********************************************************************/
/**********************************************************************
*   date   programmers         comment                               *
* 2/25/91  Douglas Wong,       start of version 1.0 records          *
*          Davis Pan                                                 *
* 3/06/91  Douglas Wong        rename: setup.h to dedef.h            *
*                                      dfilter to defilter           *
*                                      dwindow to dewindow           *
*                              integrated "quantizer", "scalefactor" *
*                              combined window_samples routine into  *
*                              filter samples                        *
* 3/31/91  Bill Aspromonte     replaced read_filter by               *
*                              create_syn_filter and introduced a    *
*                              new Sub-Band Synthesis routine called *
*                              SubBandSynthesis()                    *
* 5/10/91  Vish (PRISM)        Ported to Macintosh and Unix.         *
*                              Changed "out_fifo()" so that last     *
*                              unfilled block is also written out.   *
*                              "create_syn_filter()" was modified so *
*                              that calculation precision is same as *
*                              in specification tables.              *
*                              Changed "decode_scale()" to reflect   *
*                              specifications.                       *
*                              Removed all routines used by          *
*                              "synchronize_buffer()".  This is now  *
*                              replaced by "seek_sync()".            *
*                              Incorporated Jean-Georges Fritsch's   *
*                              "bitstream.c" package.                *
*                              Deleted "reconstruct_sample()".       *
* 27jun91  dpwe (Aware)        Passed outFile and &sampFrames as     *
*                              args to out_fifo() - were global.     *
*                              Moved "alloc_*" reader to common.c.   *
*                              alloc, sblimit, stereo passed via new *
*                              'frameParameters struct (were globals).  *
*                              Added JOINT STEREO decoding, lyrs I&II*
*                              Affects: decode_bitalloc,buffer_samps *
*                              Plus a few other cleanups.            *
* 6/10/91   Earle Jennings     conditional expansion added in        *
*                              II_dequantize_sample to handle range  *
*                              problems in MSDOS version             *
* 8/8/91    Jens Spille        Change for MS-C6.00                   *
*10/1/91    S.I. Sudharsanan,  Ported to IBM AIX platform.           *
*           Don H. Lee,                                              *
*           Peter W. Farrett                                         *
*10/3/91    Don H. Lee         implemented CRC-16 error protection   *
*                              newly introduced functions are        *
*                              buffer_CRC and recover_CRC_error.     *
* 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
*                              important fixes involved changing     *
*                              16-bit ints to long or unsigned in    *
*                              bit alloc routines for quant of 65535 *
*                              and passing proper function args.     *
*                              Removed "Other Joint Stereo" option   *
*                              and made bitrate be total channel     *
*                              bitrate, irrespective of the mode.    *
*                              Fixed many small bugs & reorganized.  *
* 7/27/92  Juan Pineda         Bug fix in SubBandSynthesis()         *
*--------------------------------------------------------------------*
* 6/14/92  Juan Pineda         Layer III decoding routines added.    *
*          Amit Gulati         Follows CD 3-11172 rev2.  Contains    *
*                              hacks deal with evolving available    *
*                              layerIII bitstreams.  Some (minor)    *
*                              modification of prior LI&II code.     *
* 10/25/92 Amit Gulati         Updated layerIII routines. Added code *
*                              for subblock_gain, switched block     *
*                              modes, stereo pre-processing.         *
*                              Corrected sign bits for huffman       *
*                              decoding of quadruples region and     *
*                              adjusted gain factor in III_dequant.  *
* 11/21/92 Amit Gulati         Several layerIII bugs fixed.          *
* 12/15/92 Amit Gulati         Corrected reordering (indexing)       *
*          Stan Searing        within IMDCT routine.                 *
*  8/24/93 Masahiro Iwadare    Included IS modification in Layer III.*
*                              Changed for 1 pass decoding.          *
*  9/07/93 Toshiyuki Ishino    Integrated Layer III with Ver 3.9.    *
*--------------------------------------------------------------------*
* 11/20/93 Masahiro Iwadare    Integrated Layer III with Ver 4.0.    *
*--------------------------------------------------------------------*
*  7/14/94 Juergen Koller      Bug fixes in Layer III code           *
*--------------------------------------------------------------------*
* 08/11/94 IIS                 Bug fixes in Layer III code           *
*--------------------------------------------------------------------*
* 9/20/94  Davis Pan           Modification to avoid premature       *
*                              synchword detection                   *
*--------------------------------------------------------------------*
* 11/09/94 Jon Rowlands        Merged premature synchword detection  *
*                              fix into layer III code version       *
*--------------------------------------------------------------------*
* 07/12/95 Soeren H. Nielsen   Changes for LSF Layer I and II        *
*--------------------------------------------------------------------*
*    8/95  Roland Bitto	Adapted to MPEG2                             *
*  9/8/95  Roalnd Bitto        Bugfix in Function III_stereo         *
*--------------------------------------------------------------------*
* 05/08/02 Walter Voessing     new functions for MPEG1 and MPEG 2    *
*                              Stereo Processing                     *
*                              bugs within 'III_get_scale_factors'   *
*                              fixed                                 *
*                              'out_fifo': bugs fixed and 24 bit     *
*                              output added                          *
*                              'GetArguments': '-C' parameter added  *
*                              (optional 24 bit output)              *
*                              modified usage()                      *
*--------------------------------------------------------------------*
* 20/09/02                     bugfix in functions MPEG_1_III_stereo *
*                              and MPEG_2_III_stereo     W. Voessing *
*--------------------------------------------------------------------*
* 11/10/02                     bugfix in functions MPEG_1_III_stereo *
*                              and MPEG_2_III_stereo     R. Funken   *
*--------------------------------------------------------------------*
* 23/09/04 Olaf Kaehler        move tables to tables.h               *
* -------------------------------------------------------------------*
* 01/03/06 Eugen Dodenhoeft    added Mpeg 2.5 support                *
*********************************************************************/

#include <math.h>
#include <stdlib.h>
#include "debug.h"
#include "decoder.h"
#include "huffman.h"
#include "tables.h"
#include "pcmBuffer.h"


/******************************************************************************
 * private
 ******************************************************************************/

/** JS_bound(lay, modext) */
static const
int jsb_table[3][4] =  
{ 
    { 4, 8, 12, 16 }, 
    { 4, 8, 12, 16 },
    { 0, 4, 8, 16 } 
};

static const
double multiple[64] = {
2.00000000000000, 1.58740105196820, 1.25992104989487,
1.00000000000000, 0.79370052598410, 0.62996052494744, 0.50000000000000,
0.39685026299205, 0.31498026247372, 0.25000000000000, 0.19842513149602,
0.15749013123686, 0.12500000000000, 0.09921256574801, 0.07874506561843,
0.06250000000000, 0.04960628287401, 0.03937253280921, 0.03125000000000,
0.02480314143700, 0.01968626640461, 0.01562500000000, 0.01240157071850,
0.00984313320230, 0.00781250000000, 0.00620078535925, 0.00492156660115,
0.00390625000000, 0.00310039267963, 0.00246078330058, 0.00195312500000,
0.00155019633981, 0.00123039165029, 0.00097656250000, 0.00077509816991,
0.00061519582514, 0.00048828125000, 0.00038754908495, 0.00030759791257,
0.00024414062500, 0.00019377454248, 0.00015379895629, 0.00012207031250,
0.00009688727124, 0.00007689947814, 0.00006103515625, 0.00004844363562,
0.00003844973907, 0.00003051757813, 0.00002422181781, 0.00001922486954,
0.00001525878906, 0.00001211090890, 0.00000961243477, 0.00000762939453,
0.00000605545445, 0.00000480621738, 0.00000381469727, 0.00000302772723,
0.00000240310869, 0.00000190734863, 0.00000151386361, 0.00000120155435,
1E-20
};

/******************************************************************************
 * private functions
 ******************************************************************************/

static
void updateCRC(unsigned int data, unsigned int length, unsigned int *crc )
{
  unsigned int  masking, carry;

  masking = 1 << length;

  while((masking >>= 1)){
    carry = *crc & 0x8000;
    *crc <<= 1;
    if (!carry ^ !(data & masking))
    *crc ^= CRC16_POLYNOMIAL;
  }
  *crc &= 0xffff;
}

/**
 * Read N bits from the bitBuffer and update the CRC 
 */
static
unsigned int getBitsCRC(HANDLE_bitBuffer buf, int N, unsigned int *crc)
{
  unsigned long bits;
  bitBufferRead(buf, &bits, N);
  updateCRC(bits, N, crc);
  return bits;
}

/**
 * Using the decoded info the appropriate possible quantization per subband 
 * table is loaded - choose table, load if necess, return # sb's 
 */
static
int pickTable(decData *dd)
{
  int table;
  int sblim = dd->sblimit; /* return current value if no load */
  int br_per_ch = dd->frameData.bitrate / dd->frameData.channels; 
  int sfrq = (int)dd->frameData.sfreq;

  /* decision rules refer to per-channel bitrates (kbits/sec/chan) */
  if (dd->frameData.version == MPEG_PHASE1) 
  { /* MPEG-1 */
    if ((sfrq == 48 && (br_per_ch == 0 || br_per_ch >= 56)) || (br_per_ch >= 56 && br_per_ch <= 80)) 
      table = 0;
    else if (sfrq != 48 && (br_per_ch == 0 || br_per_ch >= 96)) 
      table = 1;
    else if (sfrq != 32 && br_per_ch <= 48) 
      table = 2;
    else table = 3;
  }
  else 
  { /* MPEG-2 LSF & MPEG_2_5 */
    table = 4;
  }

  if (dd->tableNumber != table) 
  {
    dd->tableNumber = table; 
    sblim = get_bit_alloc(table, &dd->tableAlloc);
  }
  return sblim;
}


/******************************************************************************
 * public functions
 ******************************************************************************/

void decodeFrameData(decData *dd)
{
  frameData *fd = &dd->frameData;
  
  /* calc crc if needed */
  if (fd->error_protection)
  {
    dd->newCRC = 0xffff;
    updateCRC(fd->bitrate_index,       4, &dd->newCRC);
    updateCRC(fd->sampling_frequency,  2, &dd->newCRC);
    updateCRC(fd->padding,             1, &dd->newCRC);
    updateCRC(fd->extension,           1, &dd->newCRC);
    updateCRC(fd->mode,                2, &dd->newCRC);
    updateCRC(fd->mode_ext,            2, &dd->newCRC);
    updateCRC(fd->copyright,           1, &dd->newCRC);
    updateCRC(fd->original,            1, &dd->newCRC);
    updateCRC(fd->emphasis,            2, &dd->newCRC);
  }

  if (fd->lay == LAYER2)
    dd->sblimit = pickTable(dd);
  else 
    dd->sblimit = SBLIMIT;

  if(fd->mode == MPG_MD_JOINT_STEREO)
    dd->jsbound = jsb_table[fd->lay-1][fd->mode_ext];
  else
    dd->jsbound = dd->sblimit;

}

int I_CRC_calc( decData *dd, unsigned int bit_alloc[2][SBLIMIT])
{
  int i, k;
  int stereo  = dd->frameData.channels;
  int jsbound = dd->jsbound;

  for (i=0;i<SBLIMIT;i++)
    for (k=0;k<((i<jsbound)?stereo:1);k++)
      updateCRC(bit_alloc[k][i], 4, &dd->newCRC);

  return (dd->frameData.crc != dd->newCRC);
}

int II_CRC_calc(
                 decData *dd,
                 unsigned int bit_alloc[2][SBLIMIT],
                 unsigned int scfsi[2][SBLIMIT] )
{
  int i, k;
  int stereo  = dd->frameData.channels;
  int sblimit = dd->sblimit;
  int jsbound = dd->jsbound;
  al_table *alloc = &dd->tableAlloc;

  for (i=0;i<sblimit;i++)
    for (k=0;k<((i<jsbound)?stereo:1);k++)
      updateCRC(bit_alloc[k][i], (*alloc)[i][0].bits, &dd->newCRC);

  for (i=0;i<sblimit;i++)
    for (k=0;k<stereo;k++)
      if (bit_alloc[k][i])
        updateCRC(scfsi[k][i], 2, &dd->newCRC);

  return (dd->frameData.crc != dd->newCRC);
}





/*****************************************************************
 *
 * The following routines decode the system information
 *
 ****************************************************************/


/*******************************************************************
 *
 * The bit allocation information is decoded. Layer I
 * has 4 bit per subband whereas Layer II is Ws and bit rate
 * dependent.
 *
 ********************************************************************/

/**************************** Layer II *************/

void II_decode_bitalloc(
                        HANDLE_bitBuffer buf,
                        unsigned int bit_alloc[2][SBLIMIT],
                        decData *dd )
{
    int i,j;
    int stereo = dd->frameData.channels;
    int sblimit = dd->sblimit;
    int jsbound = dd->jsbound;
    al_table *alloc = &dd->tableAlloc;

    for (i=0;i<jsbound;i++) for (j=0;j<stereo;j++)
        bit_alloc[j][i] = (char) bitBufferGet(buf,(*alloc)[i][0].bits);

    for (i=jsbound;i<sblimit;i++) /* expand to 2 channels */
        bit_alloc[0][i] = bit_alloc[1][i] =
        (char) bitBufferGet(buf,(*alloc)[i][0].bits);

    for (i=sblimit;i<SBLIMIT;i++) for (j=0;j<stereo;j++)
        bit_alloc[j][i] = 0;
}

/**************************** Layer I *************/

void I_decode_bitalloc(
                       HANDLE_bitBuffer inputBitBuffer,
                       unsigned int bit_alloc[2][SBLIMIT],
                       decData *dd )
{
    int i,j;
    int stereo  = dd->frameData.channels;
    int jsbound = dd->jsbound;
    int b;

    for (i=0;i<jsbound;i++) for (j=0;j<stereo;j++)
        bit_alloc[j][i] = bitBufferGet(inputBitBuffer,4);
    for (i=jsbound;i<SBLIMIT;i++) {
        b = bitBufferGet(inputBitBuffer,4);
        for (j=0;j<stereo;j++)
            bit_alloc[j][i] = b;
    }
}

/*****************************************************************
 *
 * The following two functions implement the layer I and II
 * format of scale factor extraction. Layer I involves reading
 * 6 bit per subband as scale factor. Layer II requires reading
 * first the scfsi which in turn indicate the number of scale factors
 * transmitted.
 *    Layer I : I_decode_scale
 *   Layer II : II_decode_scale
 *
 ****************************************************************/

/************************** Layer I stuff ************************/

void I_decode_scale(
                    HANDLE_bitBuffer inputBitBuffer,
                    unsigned int bit_alloc[2][SBLIMIT],
                    unsigned int scale_index[2][3][SBLIMIT],
                    decData *dd )
{
    int i,j;
    int stereo = dd->frameData.channels;

    for (i=0;i<SBLIMIT;i++) for (j=0;j<stereo;j++)
        if (!bit_alloc[j][i])
            scale_index[j][0][i] = SCALE_RANGE-1;
        else                    /* 6 bit per scale factor */
            scale_index[j][0][i] =  bitBufferGet(inputBitBuffer,6);

}

/*************************** Layer II stuff ***************************/

void II_decode_scale(
                     HANDLE_bitBuffer inputBitBuffer,
                     unsigned int scfsi[2][SBLIMIT],
                     unsigned int bit_alloc[2][SBLIMIT],
                     unsigned int scale_index[2][3][SBLIMIT],
                     decData *dd )
{
    int i,j;
    int stereo = dd->frameData.channels;
    int sblimit = dd->sblimit;

    for (i=0;i<sblimit;i++) for (j=0;j<stereo;j++) /* 2 bit scfsi */
        if (bit_alloc[j][i]) scfsi[j][i] = (char) bitBufferGet(inputBitBuffer,2);
        for (i=sblimit;i<SBLIMIT;i++) for (j=0;j<stereo;j++)
            scfsi[j][i] = 0;

        for (i=0;i<sblimit;i++) for (j=0;j<stereo;j++) {
            if (bit_alloc[j][i])
                switch (scfsi[j][i]) {
                /* all three scale factors transmitted */
             case 0 : scale_index[j][0][i] = bitBufferGet(inputBitBuffer,6);
                 scale_index[j][1][i] = bitBufferGet(inputBitBuffer,6);
                 scale_index[j][2][i] = bitBufferGet(inputBitBuffer,6);
                 break;
                 /* scale factor 1 & 3 transmitted */
             case 1 : scale_index[j][0][i] =
                          scale_index[j][1][i] = bitBufferGet(inputBitBuffer,6);
                 scale_index[j][2][i] = bitBufferGet(inputBitBuffer,6);
                 break;
                 /* scale factor 1 & 2 transmitted */
             case 3 : scale_index[j][0][i] = bitBufferGet(inputBitBuffer,6);
                 scale_index[j][1][i] =
                     scale_index[j][2][i] =  bitBufferGet(inputBitBuffer,6);
                 break;
                 /* only one scale factor transmitted */
             case 2 : scale_index[j][0][i] =
                          scale_index[j][1][i] =
                          scale_index[j][2][i] = bitBufferGet(inputBitBuffer,6);
                 break;
             default : break;
            }
            else {
                scale_index[j][0][i] = scale_index[j][1][i] =
                    scale_index[j][2][i] = SCALE_RANGE-1;
            }
        }
        for (i=sblimit;i<SBLIMIT;i++) for (j=0;j<stereo;j++) {
            scale_index[j][0][i] = scale_index[j][1][i] =
                scale_index[j][2][i] = SCALE_RANGE-1;
        }
}

/**************************************************************
 *
 *   The following two routines take care of reading the
 * compressed sample from the bit stream for both layer 1 and
 * layer 2. For layer 1, read the number of bits as indicated
 * by the bit_alloc information. For layer 2, if grouping is
 * indicated for a particular subband, then the sample size has
 * to be read from the bits_group and the merged samples has
 * to be decompose into the three distinct samples. Otherwise,
 * it is the same for as layer one.
 *
 **************************************************************/

/******************************* Layer I stuff ******************/

void I_buffer_sample(
                     HANDLE_bitBuffer inputBitBuffer,
                     unsigned int sample[2][3][SBLIMIT],
                     unsigned int bit_alloc[2][SBLIMIT],
                     decData *dd )
{
    int i,j,k;
    int stereo = dd->frameData.channels;
    int jsbound = dd->jsbound;
    unsigned int s;

    for (i=0;i<jsbound;i++) for (j=0;j<stereo;j++)
        if ( (k = bit_alloc[j][i]) == 0)
            sample[j][0][i] = 0;
        else
            sample[j][0][i] = (unsigned int) bitBufferGet(inputBitBuffer,k+1);
        for (i=jsbound;i<SBLIMIT;i++) {
            if ( (k = bit_alloc[0][i]) == 0)
                s = 0;
            else
                s = (unsigned int)bitBufferGet(inputBitBuffer,k+1);
            for (j=0;j<stereo;j++)
                sample[j][0][i]    = s;
        }
}

/*************************** Layer II stuff ************************/

void II_buffer_sample(
                      HANDLE_bitBuffer inputBitBuffer,
                      unsigned int sample[2][3][SBLIMIT],
                      unsigned int bit_alloc[2][SBLIMIT],
                      decData *dd )
{
    int i,j,k,m;
    int stereo = dd->frameData.channels;
    int sblimit = dd->sblimit;
    int jsbound = dd->jsbound;
    al_table *alloc = &dd->tableAlloc;

    for (i=0;i<sblimit;i++) for (j=0;j<((i<jsbound)?stereo:1);j++) {
        if (bit_alloc[j][i]) {
            /* check for grouping in subband */
            if ((*alloc)[i][bit_alloc[j][i]].group==3)
                for (m=0;m<3;m++) {
                    k = (*alloc)[i][bit_alloc[j][i]].bits;
                    sample[j][m][i] = (unsigned int) bitBufferGet(inputBitBuffer,k);
                }
                else {              /* bit_alloc = 3, 5, 9 */
                    unsigned int nlevels, c=0;

                    nlevels = (*alloc)[i][bit_alloc[j][i]].steps;
                    k=(*alloc)[i][bit_alloc[j][i]].bits;
                    c = (unsigned int) bitBufferGet(inputBitBuffer, k);
                    for (k=0;k<3;k++) {
                        sample[j][k][i] = c % nlevels;
                        c /= nlevels;
                    }
                }
        }
        else {                  /* for no sample transmitted */
            for (k=0;k<3;k++) sample[j][k][i] = 0;
        }
        if(stereo == 2 && i>= jsbound) /* joint stereo : copy L to R */
            for (k=0;k<3;k++) sample[1][k][i] = sample[0][k][i];
    }
    for (i=sblimit;i<SBLIMIT;i++) for (j=0;j<stereo;j++) for (k=0;k<3;k++)
        sample[j][k][i] = 0;
}

/**************************************************************
 *
 *   Restore the compressed sample to a factional number.
 *   first complement the MSB of the sample
 *    for layer I :
 *    Use s = (s' + 2^(-nb+1) ) * 2^nb / (2^nb-1)
 *   for Layer II :
 *   Use the formula s = s' * c + d
 *
 **************************************************************/

static double c[17] = { 1.33333333333, 1.60000000000, 1.14285714286,
1.77777777777, 1.06666666666, 1.03225806452,
1.01587301587, 1.00787401575, 1.00392156863,
1.00195694716, 1.00097751711, 1.00048851979,
1.00024420024, 1.00012208522, 1.00006103888,
1.00003051851, 1.00001525902 };

static double d[17] = { 0.500000000, 0.500000000, 0.250000000, 0.500000000,
0.125000000, 0.062500000, 0.031250000, 0.015625000,
0.007812500, 0.003906250, 0.001953125, 0.0009765625,
0.00048828125, 0.00024414063, 0.00012207031,
0.00006103516, 0.00003051758 };

/************************** Layer II stuff ************************/

void II_dequantize_sample(
                          unsigned int sample[2][3][SBLIMIT],
                          unsigned int bit_alloc[2][SBLIMIT],
                          double fraction[2][3][SBLIMIT],
                          decData *dd )
{
    int i, j, k, x;
    int stereo = dd->frameData.channels;
    int sblimit = dd->sblimit;
    al_table *alloc = &dd->tableAlloc;

    for (i=0;i<sblimit;i++)  for (j=0;j<3;j++) for (k=0;k<stereo;k++)
        if (bit_alloc[k][i]) {

            /* locate MSB in the sample */
            x = 0;
#ifndef MS_DOS
            while ((unsigned int)(1L<<x) < (*alloc)[i][bit_alloc[k][i]].steps) x++;
#else
            /* microsoft C thinks an int is a short */
            while (( (unsigned long) (1L<<(long)x) <
                (unsigned long)( (*alloc)[i][bit_alloc[k][i]].steps)
                ) && ( x < 16) ) x++;
#endif

            /* MSB inversion */
            if (((sample[k][j][i] >> (x-1)) & 1) == 1)
                fraction[k][j][i] = 0.0;
            else  fraction[k][j][i] = -1.0;

            /* Form a 2's complement sample */
            fraction[k][j][i] += (double) (sample[k][j][i] & ((1<<(x-1))-1)) /
                (double) (1L<<(x-1));

            /* Dequantize the sample */
            fraction[k][j][i] += d[(*alloc)[i][bit_alloc[k][i]].quant];
            fraction[k][j][i] *= c[(*alloc)[i][bit_alloc[k][i]].quant];
        }
        else fraction[k][j][i] = 0.0;

        for (i=sblimit;i<SBLIMIT;i++) for (j=0;j<3;j++) for(k=0;k<stereo;k++)
            fraction[k][j][i] = 0.0;
}

/***************************** Layer I stuff ***********************/

void I_dequantize_sample(
                         unsigned int sample[2][3][SBLIMIT],
                         double fraction[2][3][SBLIMIT],
                         unsigned int bit_alloc[2][SBLIMIT],
                         decData *dd )
{
    int i, nb, k;
    int stereo = dd->frameData.channels;

    for (i=0;i<SBLIMIT;i++)
        for (k=0;k<stereo;k++)
            if (bit_alloc[k][i]) {
                nb = bit_alloc[k][i] + 1;
                if (((sample[k][0][i] >> (nb-1)) & 1) == 1) fraction[k][0][i] = 0.0;
                else fraction[k][0][i] = -1.0;
                fraction[k][0][i] += (double) (sample[k][0][i] & ((1<<(nb-1))-1)) /
                    (double) (1L<<(nb-1));

                fraction[k][0][i] =
                    (double) (fraction[k][0][i]+1.0/(double)(1L<<(nb-1))) *
                    (double) (1L<<nb) / (double) ((1L<<nb)-1);
            }
            else fraction[k][0][i] = 0.0;
}

/************************************************************
 *
 *   Restore the original value of the sample ie multiply
 *    the fraction value by its scalefactor.
 *
 ************************************************************/

/************************* Layer II Stuff **********************/

void II_denormalize_sample(
                           double fraction[2][3][SBLIMIT],
                           unsigned int scale_index[2][3][SBLIMIT],
                           decData *dd,
                           int x )
{
    int i,j;
    int stereo = dd->frameData.channels;
    int sblimit = dd->sblimit;

    for (i=0;i<sblimit;i++) for (j=0;j<stereo;j++) {
        fraction[j][0][i] *= multiple[scale_index[j][x][i]];
        fraction[j][1][i] *= multiple[scale_index[j][x][i]];
        fraction[j][2][i] *= multiple[scale_index[j][x][i]];
    }
}

/**************************** Layer I stuff ******************************/

void I_denormalize_sample(
                          double fraction[2][3][SBLIMIT],
                          unsigned int scale_index[2][3][SBLIMIT],
                          decData *dd )
{
    int i,j;
    int stereo = dd->frameData.channels;

    for (i=0;i<SBLIMIT;i++) for (j=0;j<stereo;j++)
        fraction[j][0][i] *= multiple[scale_index[j][0][i]];
}

/*****************************************************************
 *
 * The following are the subband synthesis routines. They apply
 * to both layer I and layer II stereo or mono. The user has to
 * decide what parameters are to be passed to the routines.
 *
 ***************************************************************/

/*************************************************************
 *
 *   Pass the subband sample through the synthesis window
 *
 **************************************************************/

/* create in synthesis filter */
void createSynFilter (synData *syn)
{
    int i,k;

    for (i=0; i<64; i++)
        for (k=0; k<32; k++) 
        {
            if ((syn->filter[i][k] = 1e9*cos((double)((PI64*i+PI4)*(2*k+1)))) >= 0)
                modf(syn->filter[i][k]+0.5, &syn->filter[i][k]);
            else
                modf(syn->filter[i][k]-0.5, &syn->filter[i][k]);
            syn->filter[i][k] *= 1e-9;
        }
}

/***************************************************************
 *
 *   Window the restored sample
 *
 ***************************************************************/

void subBandSynthesis (synData *syn, double *bandPtr, int channel, double *samples)
{
    const double *window;
    int       i,j,k;
    double    sum;
    double   *bufOffsetPtr;

    window = get_syn_window();

    /*    if (channel == 0) */
    syn->bufOffset[channel] = (syn->bufOffset[channel] - 64) & 0x3ff;
    bufOffsetPtr = &(syn->buf[channel][syn->bufOffset[channel]]);

    for (i=0; i<64; i++) 
    {
        sum = 0;
        for (k=0; k<32; k++) 
        {
            sum += bandPtr[k] * syn->filter[i][k];
        }
        bufOffsetPtr[i] = sum;
    }

    /*  S(i,j) = D(j+32i) * U(j+32i+((i+1)>>1)*64)  */
    /*  samples(i,j) = MWindow(j+32i) * bufPtr(j+32i+((i+1)>>1)*64)  */
    for (j=0; j<32; j++) 
    {
        sum = 0;
        for (i=0; i<16; i++)  
        {
            k = j + (i<<5);
            sum += window[k] * syn->buf [channel] [( (k + ( ((i+1)>>1) <<6) ) + syn->bufOffset[channel]) & 0x3ff];
        }
        samples[j] = sum;
    }
}
           
#if 0
void  recover_CRC_error(
                        HANDLE_pcmBuffer pcm_frame_buffer,
                        unsigned long    error_count,     
                        unsigned long    samplesPerFrame)
{
  unsigned int i;
  
  if (error_count == 1) {     /* replicate previous error_free frame */
/*
  done = 1;
  flush out fifo 
        out_fifo(pcm_sample, num, fd, done, outFile, psampFrames, Output24, &clip,samplesPerFrame);
         go back to the beginning of the previous frame 
        offset = sizeof(short int) * samplesPerFrame;
        fseek(outFile, -offset, SEEK_CUR);
        done = 0;
        for (i = 0; i < SCALE_BLOCK; i++) {
            fread(pcm_sample, 2, samplesPerSlot, outFile);
            out_fifo(pcm_sample, num, fd, done, outFile, psampFrames, Output24, &clip, samplesPerFrame);
        }
*/
    pcmBufferRestore(pcm_frame_buffer, samplesPerFrame);
  }
  else {                       /* mute the frame */
    pcmBufferReset(pcm_frame_buffer);
    for (i = 0; i < samplesPerFrame; i++) {
      pcmBufferPut(pcm_frame_buffer, MUTE);  /* MUTE value is in decoder.h */
    }
  }
}
*/
#endif

/************************* Layer III routines **********************/


int III_get_side_info_crc(
                             HANDLE_bitBuffer inputBitBuffer,
                             III_side_info_t *si,
                             decData *dd)

/* get side info and calculate CRC at the same time / adapted by efs 2003-11-11 */
{
    
    int ch, gr, i;

    int stereo  = dd->frameData.channels;

    if (dd->frameData.version == MPEG_PHASE1) /* MPEG1 */
    { 
        si->main_data_begin = getBitsCRC(inputBitBuffer, 9, &dd->newCRC);
        if (stereo == 1) {
          si->private_bits = getBitsCRC(inputBitBuffer,5, &dd->newCRC);
        }
        else {
          si->private_bits = getBitsCRC(inputBitBuffer,3, &dd->newCRC);
        }

        for (ch=0; ch<stereo; ch++)
            for (i=0; i<4; i++)
                si->ch[ch].scfsi[i] = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);


            for (gr=0; gr< 2 ; gr++) {
                for (ch=0; ch<stereo; ch++) {
                    si->ch[ch].gr[gr].part2_3_length = getBitsCRC(inputBitBuffer, 12, &dd->newCRC);
                    si->ch[ch].gr[gr].big_values = getBitsCRC(inputBitBuffer, 9, &dd->newCRC);
                    si->ch[ch].gr[gr].global_gain = getBitsCRC(inputBitBuffer, 8, &dd->newCRC);
                    si->ch[ch].gr[gr].scalefac_compress = getBitsCRC(inputBitBuffer, 4, &dd->newCRC);
                    si->ch[ch].gr[gr].window_switching_flag = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                    if (si->ch[ch].gr[gr].window_switching_flag) {
                        si->ch[ch].gr[gr].block_type = getBitsCRC(inputBitBuffer, 2, &dd->newCRC);
                        si->ch[ch].gr[gr].mixed_block_flag = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                        for (i=0; i<2; i++) {
                            si->ch[ch].gr[gr].table_select[i] = getBitsCRC(inputBitBuffer, 5, &dd->newCRC);
                        }
                        for (i=0; i<3; i++) {
                            si->ch[ch].gr[gr].subblock_gain[i] = getBitsCRC(inputBitBuffer, 3, &dd->newCRC);
                        }

                        /* Set region_count parameters since they are implicit in this case. */

                        if (si->ch[ch].gr[gr].block_type == 0) {
                            printf("Side info bad: block_type == 0 in split block.\n");
                           return 0;
                        }
                        else if (si->ch[ch].gr[gr].block_type == 2
                            && si->ch[ch].gr[gr].mixed_block_flag == 0)
                        {
                          si->ch[ch].gr[gr].region0_count = 8; /* MI 9; */
                        }
                        else si->ch[ch].gr[gr].region0_count = 7; /* MI 8; */
                        si->ch[ch].gr[gr].region1_count = 20 -
                            si->ch[ch].gr[gr].region0_count;
                    }
                    else {
                        for (i=0; i<3; i++){
                            si->ch[ch].gr[gr].table_select[i] = getBitsCRC(inputBitBuffer, 5, &dd->newCRC);
                        }
                        si->ch[ch].gr[gr].region0_count = getBitsCRC(inputBitBuffer, 4, &dd->newCRC);
                        si->ch[ch].gr[gr].region1_count = getBitsCRC(inputBitBuffer, 3, &dd->newCRC);
                        si->ch[ch].gr[gr].block_type = 0;
                    }
                    si->ch[ch].gr[gr].preflag = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                    si->ch[ch].gr[gr].scalefac_scale = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                    si->ch[ch].gr[gr].count1table_select = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                }
            }
    }
    else /* Layer 3 LSF */
    {
        si->main_data_begin = getBitsCRC(inputBitBuffer, 8, &dd->newCRC);
        if (stereo == 1) {
            si->private_bits = getBitsCRC(inputBitBuffer,1, &dd->newCRC);
        }
        else si->private_bits = getBitsCRC(inputBitBuffer,2, &dd->newCRC);

        for (gr=0; gr< 1 ; gr++) {
            for (ch=0; ch<stereo; ch++) {
                si->ch[ch].gr[gr].part2_3_length = getBitsCRC(inputBitBuffer, 12, &dd->newCRC);
                si->ch[ch].gr[gr].big_values = getBitsCRC(inputBitBuffer, 9, &dd->newCRC);
                si->ch[ch].gr[gr].global_gain = getBitsCRC(inputBitBuffer, 8, &dd->newCRC);
                si->ch[ch].gr[gr].scalefac_compress = getBitsCRC(inputBitBuffer, 9, &dd->newCRC);   
                si->ch[ch].gr[gr].window_switching_flag = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);

                if (si->ch[ch].gr[gr].window_switching_flag) {
                
                    si->ch[ch].gr[gr].block_type = getBitsCRC(inputBitBuffer, 2, &dd->newCRC);
                    si->ch[ch].gr[gr].mixed_block_flag = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                    for (i=0; i<2; i++) {
                        si->ch[ch].gr[gr].table_select[i] = getBitsCRC(inputBitBuffer, 5, &dd->newCRC);
                    }
                    si->ch[ch].gr[gr].table_select[2] = 0; /* not used, set to zero */

                    for (i=0; i<3; i++) {
                        si->ch[ch].gr[gr].subblock_gain[i] = getBitsCRC(inputBitBuffer, 3, &dd->newCRC);
                    }

                    /* Set region_count parameters since they are implicit in this case. */

                    if (si->ch[ch].gr[gr].block_type == 0) {
                        printf("Side info bad: block_type == 0 in split block.\n");
                        return 0;
                    }
                    else if (si->ch[ch].gr[gr].block_type == 2 && si->ch[ch].gr[gr].mixed_block_flag == 0)
                    {
                        si->ch[ch].gr[gr].region0_count = 8; /* MI 9; */
                    }
                    else
                    {
                        si->ch[ch].gr[gr].region0_count = 7; /* MI 8; */
                    }
                    si->ch[ch].gr[gr].region1_count = 20 - si->ch[ch].gr[gr].region0_count;

                }
                else {
                    for (i=0; i<3; i++) {
                        si->ch[ch].gr[gr].table_select[i] = getBitsCRC(inputBitBuffer, 5, &dd->newCRC);
                    }
                    si->ch[ch].gr[gr].region0_count = getBitsCRC(inputBitBuffer, 4, &dd->newCRC);
                    si->ch[ch].gr[gr].region1_count = getBitsCRC(inputBitBuffer, 3, &dd->newCRC);
                    si->ch[ch].gr[gr].block_type = 0;
                }

                si->ch[ch].gr[gr].scalefac_scale = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
                si->ch[ch].gr[gr].count1table_select = getBitsCRC(inputBitBuffer, 1, &dd->newCRC);
            }
        }
    }
    return 1;
}


struct
{
    int l[5];
    int s[3];
} sfbtable =
{
    {0, 6, 11, 16, 21},
    {0, 6, 12}
};

int slen[2][16] = {{0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4},
    {0, 1, 2, 3, 0, 1, 2, 3, 1, 2, 3, 1, 2, 3, 2, 3}};

struct
{
    int l[23];
    int s[14];
} sfBandIndex[VERSION_COUNT][3] =
{
  /* MPEG-1 */
  {
    {
      {0,4, 8,12,16,20,24,30,36,44, 52, 62, 74, 90,110,134,162,196,238,288,342,418,576},
      {0,4, 8,12,16,22,30,40,52,66, 84,106,136,192} /* 44.1kHz */
    },
    {
      {0,4, 8,12,16,20,24,30,36,42, 50, 60, 72, 88,106,128,156,190,230,276,330,384,576},
      {0,4, 8,12,16,22,28,38,50,64, 80,100,126,192} /* 48kHz */
    },
    {
      {0,4, 8,12,16,20,24,30,36,44, 54, 66, 82,102,126,156,194,240,296,364,448,550,576},
      {0,4, 8,12,16,22,30,42,58,78,104,138,180,192} /* 32kHz */
    }
  },
  /* MPEG-2 */
  {
    {
      {0,6,12,18,24,30,36,44,54,66, 80, 96,116,140,168,200,238,284,336,396,464,522,576},
      {0,4, 8,12,18,24,32,42,56,74,100,132,174,192} /* 22.05kHz */ 
    },
    {
      {0,6,12,18,24,30,36,44,54,66, 80, 96,114,136,162,194,232,278,332,394,464,540,576},
      {0,4, 8,12,18,26,36,48,62,80,104,136,180,192} /* 24kHz */
    },
    {
      {0,6,12,18,24,30,36,44,54,66, 80, 96,116,140,168,200,238,284,336,396,464,522,576},
      {0,4, 8,12,18,26,36,48,62,80,104,134,174,192} /* 16kHz */
    }
  },
#ifdef MPEG_2_5
  /* reserved */
  {
    {
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0}
    },
    {
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0}
    },
    {
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0}
    }
  },
  /* MPEG-2.5 */
  {
    {
      {0,6,12,18,24,30,36,44,54,66,80,96,116,140,168,200,238,284,336,396,464,522,576},
      {0,4, 8,12,18,26,36,48,62,80,104,134,174,192} /* 11.025kHz */ 
    },
 	  {
      {0,6,12,18,24,30,36,44,54,66, 80, 96,116,140,168,200,238,284,336,396,464,522,576},
      {0,4, 8,12,18,26,36,48,62,80,104,134,174,192} /* 12kHz */
    },
 	  {
      {0,12,24,36,48,60,72,88,108,132,160,192,232,280,336,400,476,566,568,570,572,574,576},
  		{0,8,16,24,36,52,72,96,124,160,162,164,166,192} /* 8kHz */
    }
  }
#endif
};



void III_get_scale_factors(
                          HANDLE_bitBuffer mainData,
                          III_scalefac_t *scalefac,
                          III_side_info_t *si,
                          int gr,
                          int ch )
{
  int sfb, i, window;
  struct gr_info_s *gr_info = &(si->ch[ch].gr[gr]);

  
  if (gr_info->window_switching_flag && (gr_info->block_type == 2)) 
  {
    if (gr_info->mixed_block_flag) { /* MIXED */ /* NEW - ag 11/25 */
      for (sfb = 0; sfb < 8; sfb++)
        (*scalefac)[ch].l[sfb] =
          bitBufferGet(mainData, slen[0][gr_info->scalefac_compress]);
      for (sfb = 3; sfb < 6; sfb++)
        for (window=0; window<3; window++)
          (*scalefac)[ch].s[window][sfb] =
            bitBufferGet(mainData, slen[0][gr_info->scalefac_compress]);
      for (sfb = 6; sfb < 12; sfb++)
        for (window=0; window<3; window++)
          (*scalefac)[ch].s[window][sfb] =
            bitBufferGet(mainData, slen[1][gr_info->scalefac_compress]);
      for (sfb=12,window=0; window<3; window++)
        (*scalefac)[ch].s[window][sfb] = 0;
    }
    else {  /* SHORT*/
      for (i=0; i<2; i++)
      {
        for (sfb = sfbtable.s[i]; sfb < sfbtable.s[i+1]; sfb++)
        {
          for (window=0; window<3; window++)
          {
            (*scalefac)[ch].s[window][sfb] = bitBufferGet(mainData, slen[i][gr_info->scalefac_compress]);
          }
        }
      }
      for (sfb=12,window=0; window<3; window++)
      {
        (*scalefac)[ch].s[window][sfb] = 0;
      }
    }
  }
  else {   /* LONG types 0,1,3 */
    for (i=0; i<4; i++) {
      if ((si->ch[ch].scfsi[i] == 0) || (gr == 0))
        for (sfb = sfbtable.l[i]; sfb < sfbtable.l[i+1]; sfb++)
          (*scalefac)[ch].l[sfb] =
            bitBufferGet(mainData, slen[(i<2)?0:1][gr_info->scalefac_compress]);
    }
    (*scalefac)[ch].l[21] = 0;		/* scalefactor not transmitted,  08.05.02 W. Voessing */
    (*scalefac)[ch].l[22] = 0;
  }
}

/****************** new MPEG2 stuf  ***********/

static unsigned nr_of_sfb_block[6][3][4] = {{{6, 5, 5, 5},{ 9, 9, 9, 9 },{6, 9, 9, 9}},
                                            {{6, 5, 7, 3},{ 9, 9, 12, 6},{6, 9, 12, 6}},
                                            {{11, 10, 0, 0},{ 18, 18, 0, 0},{15,18,0,0 }},
                                            {{7, 7, 7, 0},{ 12, 12, 12, 0},{6, 15, 12, 0}},
                                            {{6, 6, 6, 3},{12, 9, 9, 6},{6, 12, 9, 6}},
                                            {{8, 8, 5, 0},{15,12,9,0},{6,18,9,0}}};

void III_get_LSF_scale_data(
                            HANDLE_bitBuffer mainData,
                            III_side_info_t *si,
                            int gr,
                            int ch,
                            decData *dd,
                            unsigned int scalefac_buffer[54],
                            unsigned int scalefac_IIP_buffer[54] )
        {
            short i, j, k;
            short blocktypenumber, blocknumber=0;

            struct gr_info_s *gr_info = &(si->ch[ch].gr[gr]);
            unsigned scalefac_comp, int_scalefac_comp, new_slen[4];

            scalefac_comp =  gr_info->scalefac_compress;
	    
            blocktypenumber = 0;
            if ((gr_info->block_type == 2) && (gr_info->mixed_block_flag == 0)){
                blocktypenumber = 1;
	    }

            if ((gr_info->block_type == 2) && (gr_info->mixed_block_flag == 1))
                blocktypenumber = 2;

            if(!((( dd->frameData.mode_ext == 1) || (dd->frameData.mode_ext == 3)) && (ch == 1)))
            {
                if(scalefac_comp < 400)
                {
                    new_slen[0] = (scalefac_comp >> 4) / 5 ;
                    new_slen[1] = (scalefac_comp >> 4) % 5 ;
                    new_slen[2] = (scalefac_comp % 16) >> 2 ;
                    new_slen[3] = (scalefac_comp % 4);
                    si->ch[ch].gr[gr].preflag = 0;

                    blocknumber = 0;
                }

                else if( scalefac_comp  < 500)
                {
                    new_slen[0] = ((scalefac_comp - 400 )  >> 2) / 5 ;
                    new_slen[1] = ((scalefac_comp - 400) >> 2) % 5 ;
                    new_slen[2] = (scalefac_comp - 400 ) % 4 ;
                    new_slen[3] = 0;
                    si->ch[ch].gr[gr].preflag = 0;
                    blocknumber = 1;

                }

                else if( scalefac_comp  < 512)
                {
                    new_slen[0] = (scalefac_comp - 500 ) / 3 ;
                    new_slen[1] = (scalefac_comp - 500)  % 3 ;
                    new_slen[2] = 0 ;
                    new_slen[3] = 0;
                    si->ch[ch].gr[gr].preflag = 1;
                    blocknumber = 2;

                }
            }

            if((((dd->frameData.mode_ext == 1) || (dd->frameData.mode_ext == 3)) && (ch == 1)))
            {
                /*   intensity_scale = scalefac_comp %2; */
                int_scalefac_comp = scalefac_comp >> 1;

                if(int_scalefac_comp  < 180)
                {
                    new_slen[0] = int_scalefac_comp  / 36 ;
                    new_slen[1] = (int_scalefac_comp % 36 ) / 6 ;
                    new_slen[2] = (int_scalefac_comp % 36) % 6;
                    new_slen[3] = 0;
                    si->ch[ch].gr[gr].preflag = 0;
                    blocknumber = 3;

                }

                else if( int_scalefac_comp  < 244)
                {
                    new_slen[0] = ((int_scalefac_comp - 180 )  % 64 ) >> 4 ;
                    new_slen[1] = ((int_scalefac_comp - 180) % 16) >> 2 ;
                    new_slen[2] = (int_scalefac_comp - 180 ) % 4 ;
                    new_slen[3] = 0;
                    si->ch[ch].gr[gr].preflag = 0;
                    blocknumber = 4;

                }

                else if( int_scalefac_comp  < 255)
                {
                    new_slen[0] = (int_scalefac_comp - 244 ) / 3 ;
                    new_slen[1] = (int_scalefac_comp - 244 )  % 3 ;
                    new_slen[2] = 0 ;
                    new_slen[3] = 0;
                    si->ch[ch].gr[gr].preflag = 0;
                    blocknumber = 5;

                }
            }

            for(i=0;i< 45;i++) scalefac_buffer[i] = 0;
            for(i=0;i< 45;i++) scalefac_IIP_buffer[i] = 0;   /* added for correct stereo processing 08.05.02 W. Voessing */


            k = 0;
            for(i = 0;i < 4;i++)
            {
                for(j = 0; j < (short)nr_of_sfb_block[blocknumber][blocktypenumber][i]; j++)
                {
                    if(new_slen[i] == 0) {
                        scalefac_buffer[k] = 0;
                        scalefac_IIP_buffer[k] = 0;    /* added for correct stereo processing 08.05.02 W. Voessing */
                    }
                    else {
                        scalefac_buffer[k] =  bitBufferGet(mainData, new_slen[i]);
                        scalefac_IIP_buffer[k] =(unsigned int)pow(2, new_slen[i]) - 1;		/* added for correct stereo processing 08.05.02 W. Voessing */
                    }
                    k++;

                }
            }

}



void III_get_LSF_scale_factors(
                               HANDLE_bitBuffer mainData,
                               III_scalefac_t *scalefac,
                               III_side_info_t *si,
                               int gr,
                               int ch,
                               decData *dd,
                               unsigned int scalefac_buffer[54],
                               unsigned int scalefac_IIP_buffer[54] )
{
    int sfb, k = 0, window;
    struct gr_info_s *gr_info = &(si->ch[ch].gr[gr]);

    III_get_LSF_scale_data(mainData, si, gr, ch, dd, scalefac_buffer, scalefac_IIP_buffer);

    if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
        if (gr_info->mixed_block_flag)
        {         
            for (sfb = 0; sfb < 6; sfb++)       /* bug, was (sfb = 0; sfb < 8; sfb++), efs 2003-07-04 */
            {
                (*scalefac)[ch].l[sfb] = scalefac_buffer[k];
                k++;
            }
            for (sfb = 3; sfb < 12; sfb++)
                for (window=0; window<3; window++)
                {
                    (*scalefac)[ch].s[window][sfb] = scalefac_buffer[k];
                    k++;
                }
                for (sfb=12,window=0; window<3; window++)
                    (*scalefac)[ch].s[window][sfb] = 0;

            /* adjust position of "illegal position" information in scalefac_IIP_buffer[] */
            /* in mixed blocks mode for short sfb, move them 3 places up. efs 3002-07-04  */
            for (sfb = 11; sfb >= 3; sfb--)
                for (window = 2; window >= 0; window--)
                    scalefac_IIP_buffer[3*sfb + window] = scalefac_IIP_buffer[3*sfb + window - 3];

        }
        else {  /* SHORT*/
            for (sfb = 0; sfb < 12; sfb++)
            {
                for (window=0; window<3; window++)
                {
                    (*scalefac)[ch].s[window][sfb] = scalefac_buffer[k];
                    k++;
                 }
            }
            for (sfb=12,window=0; window<3; window++)
            {
                (*scalefac)[ch].s[window][sfb] = 0;
            }
        }
    }
    else {   /* LONG types 0,1,3 */
        for (sfb = 0; sfb < 21; sfb++)
        {
            (*scalefac)[ch].l[sfb] = scalefac_buffer[k];
            k++;
        }
        (*scalefac)[ch].l[21] = 0;		/* not transmitted 08.05.02. W. Voessing */
        (*scalefac)[ch].l[22] = 0;

    }
}



int III_huffman_decode(
                             HANDLE_bitBuffer audioFrame,
                             long int is[SBLIMIT][SSLIMIT],
                             III_side_info_t *si,
                             int ch,
                             int gr,
                             int part2_start,
                             decData *dd )
{
    int i, x, y;
    int v, w;
    struct huffcodetab *h;
    int region1Start;
    int region2Start;
    int sfreq;
    int version;
    int currentBit, grBits;
    my_gr_info *gi;

    /*int bt = (*si).ch[ch].gr[gr].window_switching_flag && ((*si).ch[ch].gr[gr].block_type == 2);*/

    gi = (my_gr_info *) &(*si).ch[ch].gr[gr];
    sfreq = dd->frameData.sampling_frequency;
    version = dd->frameData.version;
    
    /* Find region boundary for short block case. */

    if ( ((*si).ch[ch].gr[gr].window_switching_flag) && ((*si).ch[ch].gr[gr].block_type == 2) ) {
        /* Region2. */
#ifdef MPEG_2_5
    if(version == MPEG_PHASE2_5 && sfreq == 2) /* 8kHz */ 
    { 
        region1Start = 3*24;
    }
    else 
    {
        region1Start = 3*12;
    }
#else
      region1Start = 36;
#endif
      region2Start = 576; /* No Region2 for short block case. */
    }
    else {          /* Find region boundary for long block case. */

        region1Start = sfBandIndex[version][sfreq]
            .l[(*si).ch[ch].gr[gr].region0_count + 1]; /* MI */
        region2Start = sfBandIndex[version][sfreq]
            .l[(*si).ch[ch].gr[gr].region0_count +
            (*si).ch[ch].gr[gr].region1_count + 2]; /* MI */
    }


    grBits     = part2_start + (*si).ch[ch].gr[gr].part2_3_length;
    currentBit = bitBufferGetRead(audioFrame);

    /* Read bigvalues area. */
    for (i=0; (unsigned int)i<(*si).ch[ch].gr[gr].big_values*2; i+=2) {
        if      (i<region1Start) h = &(ht[(*si).ch[ch].gr[gr].table_select[0]]);
        else if (i<region2Start) h = &(ht[(*si).ch[ch].gr[gr].table_select[1]]);
        else                h = &(ht[(*si).ch[ch].gr[gr].table_select[2]]);
        huffman_decoder(audioFrame, h, &x, &y, &v, &w);
        is[i/SSLIMIT][i%SSLIMIT] = x;
        is[(i+1)/SSLIMIT][(i+1)%SSLIMIT] = y;
    }

    grBits     = part2_start + (*si).ch[ch].gr[gr].part2_3_length;
    currentBit = bitBufferGetRead(audioFrame);

    /* Read count1 area. */
    h = &(ht[(*si).ch[ch].gr[gr].count1table_select+32]);
    while ((bitBufferGetRead(audioFrame) < part2_start + (*si).ch[ch].gr[gr].part2_3_length ) &&
        ( i < SSLIMIT*SBLIMIT )) {
        huffman_decoder(audioFrame, h, &x, &y, &v, &w);
        is[i/SSLIMIT][i%SSLIMIT] = v;
        is[(i+1)/SSLIMIT][(i+1)%SSLIMIT] = w;
        /* 2003-11-26 jb&efs : adapted to prevent writing to outside of array is[] */
        i += 2;
        if (i >= SSLIMIT*SBLIMIT ) break; /* do not overwrite, if true discard x and y */
        is[i/SSLIMIT][i%SSLIMIT] = x;
        is[(i+1)/SSLIMIT][(i+1)%SSLIMIT] = y;
        i += 2;
    }

    grBits     = part2_start + (*si).ch[ch].gr[gr].part2_3_length;
    currentBit = bitBufferGetRead(audioFrame);

    if (bitBufferGetRead(audioFrame) > part2_start + (*si).ch[ch].gr[gr].part2_3_length)
    {  i -=4;
      bitBufferWind(audioFrame, -(bitBufferGetRead(audioFrame)-part2_start - (*si).ch[ch].gr[gr].part2_3_length));
    }

    /* Dismiss stuffing Bits */
    grBits     = part2_start + (*si).ch[ch].gr[gr].part2_3_length;
    currentBit = bitBufferGetRead(audioFrame);
    if ( currentBit < grBits )
        bitBufferWind(audioFrame, grBits - currentBit);

    /* Zero out rest. */
    for (; i<SSLIMIT*SBLIMIT; i++)
        is[i/SSLIMIT][i%SSLIMIT] = 0;
    return 1;
}


int pretab[22] = {0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,3,3,2,0};

void III_dequantize_sample(
                           long int is[SBLIMIT][SSLIMIT],
                           double xr[SBLIMIT][SSLIMIT],
                           III_scalefac_t *scalefac,
                           struct gr_info_s *gr_info,
                           int ch,
                           decData *dd )
{
    int ss,sb,cb=0,sfreq,version;
    int next_cb_boundary, cb_begin=0, cb_width=0, sign;
    int mixstart;

    mixstart = 8;                                       /* added 2003/08/21  efs */
    if(dd->frameData.version != MPEG_PHASE1) /* MPEG2 or 2.5 */
    {
      mixstart = 6;
    }

    sfreq = dd->frameData.sampling_frequency;
    version = dd->frameData.version;


    /* choose correct scalefactor band per block type, initalize boundary */

    if (gr_info->window_switching_flag && (gr_info->block_type == 2) )
        if (gr_info->mixed_block_flag)
            next_cb_boundary=sfBandIndex[version][sfreq].l[1];  /* LONG blocks: 0,1,3 */
        else {
            next_cb_boundary=sfBandIndex[version][sfreq].s[1]*3; /* pure SHORT block */
            cb_width = sfBandIndex[version][sfreq].s[1];
            cb_begin = 0;
        }
        else
            next_cb_boundary=sfBandIndex[version][sfreq].l[1];  /* LONG blocks: 0,1,3 */

        /* apply formula per block type */

        for (sb=0 ; sb < SBLIMIT ; sb++)
            for (ss=0 ; ss < SSLIMIT ; ss++) {

                if ( (sb*18)+ss == next_cb_boundary)  { /* Adjust critical band boundary */
                    if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
                        if (gr_info->mixed_block_flag)  {
/*                          if (((sb*18)+ss) == sfBandIndex[sfreq].l[8])  {    2003/08/21 efs */
                            if (((sb*18)+ss) == sfBandIndex[version][sfreq].l[mixstart])  {
                                next_cb_boundary=sfBandIndex[version][sfreq].s[4]*3;
                                cb = 3;
                                cb_width = sfBandIndex[version][sfreq].s[cb+1] -
                                    sfBandIndex[version][sfreq].s[cb];
                                cb_begin = sfBandIndex[version][sfreq].s[cb]*3;
                            }
/*                          else if (((sb*18)+ss) < sfBandIndex[sfreq].l[8])   2003/08/21 efs */
                            else if (((sb*18)+ss) < sfBandIndex[version][sfreq].l[mixstart])
                                next_cb_boundary = sfBandIndex[version][sfreq].l[(++cb)+1];
                            else {
                                next_cb_boundary = sfBandIndex[version][sfreq].s[(++cb)+1]*3;
                                cb_width = sfBandIndex[version][sfreq].s[cb+1] -
                                    sfBandIndex[version][sfreq].s[cb];
                                cb_begin = sfBandIndex[version][sfreq].s[cb]*3;
                            }
                        }
                        else  {
                            next_cb_boundary = sfBandIndex[version][sfreq].s[(++cb)+1]*3;
                            cb_width = sfBandIndex[version][sfreq].s[cb+1] -
                                sfBandIndex[version][sfreq].s[cb];
                            cb_begin = sfBandIndex[version][sfreq].s[cb]*3;
                        }
                    }
                    else /* long blocks */
                        next_cb_boundary = sfBandIndex[version][sfreq].l[(++cb)+1];
                }

                /* Compute overall (global) scaling. */

                xr[sb][ss] = pow( 2.0 , (0.25 * (gr_info->global_gain - 210.0)));

                /* Do long/short dependent scaling operations. */

                if (gr_info->window_switching_flag && (
                    ((gr_info->block_type == 2) && (gr_info->mixed_block_flag == 0)) ||
                    ((gr_info->block_type == 2) && gr_info->mixed_block_flag && (sb >= 2)) )) {

                    xr[sb][ss] *= pow(2.0, 0.25 * -8.0 *
                        gr_info->subblock_gain[(((sb*18)+ss) - cb_begin)/cb_width]);
                    xr[sb][ss] *= pow(2.0, 0.25 * -2.0 * (1.0+gr_info->scalefac_scale)
                        * (*scalefac)[ch].s[(((sb*18)+ss) - cb_begin)/cb_width][cb]);
                }
                else {   /* LONG block types 0,1,3 & 1st 2 subbands of switched blocks */
                    xr[sb][ss] *= pow(2.0, -0.5 * (1.0+gr_info->scalefac_scale)
                        * ((*scalefac)[ch].l[cb]
                        + gr_info->preflag * pretab[cb]));
                }

                /* Scale quantized value. */

                sign = (is[sb][ss]<0) ? 1 : 0;
                xr[sb][ss] *= pow( (double) abs(is[sb][ss]), ((double)4.0/3.0) );
                if (sign) xr[sb][ss] = -xr[sb][ss];
            }
}

void III_reorder(
                 double xr[SBLIMIT][SSLIMIT],
                 double ro[SBLIMIT][SSLIMIT],
                 struct gr_info_s *gr_info,
                 decData *dd )
{
    int sfreq, version;
    int sfb, sfb_start, sfb_lines;
    int sb, ss, window, freq, src_line, des_line;

    sfreq = dd->frameData.sampling_frequency;
    version = dd->frameData.version;
    
   
    for(sb=0;sb<SBLIMIT;sb++)
        for(ss=0;ss<SSLIMIT;ss++)
            ro[sb][ss] = 0;

        if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
            if (gr_info->mixed_block_flag) {
                /* NO REORDER FOR LOW 2 SUBBANDS */
                for (sb=0 ; sb < 2 ; sb++)
                    for (ss=0 ; ss < SSLIMIT ; ss++) {
                        ro[sb][ss] = xr[sb][ss];
                    }
                    /* REORDERING FOR REST SWITCHED SHORT */
                    for(sfb=3,sfb_start=sfBandIndex[version][sfreq].s[3],
                        sfb_lines=sfBandIndex[version][sfreq].s[4] - sfb_start;
                    sfb < 13; sfb++,sfb_start=sfBandIndex[version][sfreq].s[sfb],
                        (sfb_lines=sfBandIndex[version][sfreq].s[sfb+1] - sfb_start))
                        for(window=0; window<3; window++)
                            for(freq=0;freq<sfb_lines;freq++) {
                                src_line = sfb_start*3 + window*sfb_lines + freq;
                                des_line = (sfb_start*3) + window + (freq*3);
                                ro[des_line/SSLIMIT][des_line%SSLIMIT] =
                                    xr[src_line/SSLIMIT][src_line%SSLIMIT];
                            }
            }
            else {  /* pure short */
		    
                for(sfb=0,sfb_start=0,sfb_lines=sfBandIndex[version][sfreq].s[1]; sfb < 13; sfb++,sfb_start=sfBandIndex[version][sfreq].s[sfb],
                    (sfb_lines=sfBandIndex[version][sfreq].s[sfb+1] - sfb_start))
                    for(window=0; window<3; window++)
                        for(freq=0;freq<sfb_lines;freq++) {
                            src_line = (sfb_start*3) + window * sfb_lines + freq;
                            des_line = (sfb_start*3) + window + (freq*3);
                            ro[des_line/SSLIMIT][des_line%SSLIMIT] =
                                xr[src_line/SSLIMIT][src_line%SSLIMIT];
                        }
            }
        }
        else {   /*long blocks */
            for (sb=0 ; sb < SBLIMIT ; sb++)
                for (ss=0 ; ss < SSLIMIT ; ss++)
                    ro[sb][ss] = xr[sb][ss];
        }
}

static void III_i_stereo_k_values(
                                  int is_pos,
                                  double io,
                                  int i,
                                  double k[2][576] )
{
    if(is_pos == 0)
    {
        k[0][i] = 1;
        k[1][i] = 1;
    }
    else if ((is_pos % 2) == 1)
    {
        k[0][i] = pow(io,(double)((is_pos + 1)/2));
        k[1][i] = 1;
    }
    else
    {
        k[0][i] = 1;
        k[1][i] = pow(io,(double)(is_pos/2));
    }
}

/*********************************************************/
/* NEW FUNCTIONS FOR MPEG I AND MPEG 2 STEREO PROCESSING */
/*********************************************************/
/* 08.05.02 W.Voessing */

void III_Stereo(
                double xr[2][SBLIMIT][SSLIMIT],
                double lr[2][SBLIMIT][SSLIMIT],
                int Start,
                int Number )
{
    int sb, ss;
    int i;
    for (i = Start; i < (Start + Number); i++) {
        sb = i / 18; ss = i % 18;
	lr[0][sb][ss] = xr[0][sb][ss];
        lr[1][sb][ss] = xr[1][sb][ss];
	
    }

}

void III_M_S(
             double xr[2][SBLIMIT][SSLIMIT],
             double lr[2][SBLIMIT][SSLIMIT],
             int Start,
             int Number )
{
    int sb, ss;
    int i;

    for (i = Start; i < (Start + Number); i++) {
        sb = i / 18; ss = i % 18;
        lr[0][sb][ss] = (xr[0][sb][ss]+xr[1][sb][ss])/(double)1.41421356;  /* Sum */
        lr[1][sb][ss] = (xr[0][sb][ss]-xr[1][sb][ss])/(double)1.41421356;  /* Diff */
    }
}

void III_Intensity(
                   double xr[2][SBLIMIT][SSLIMIT],
                   double lr[2][SBLIMIT][SSLIMIT],
                   double IntensFactor[2][SBLIMIT*SSLIMIT],
                   int Start,
                   int Number )
{
    int sb, ss;
    int i;

    for (i = Start; i < (Start + Number); i++) {
        sb = i / 18; ss = i % 18;                 /* start values for (lr[0], xr[0]), (lr[1], xr[1]) */
        lr[0][sb][ss] = xr[0][sb][ss] * IntensFactor[0][i];
        lr[1][sb][ss] = xr[0][sb][ss] * IntensFactor[1][i];
    }
}
/**************************** NEW MPEG1, LAYER 3 STEREO FUNCTION  08.05.02 W. Voessing ***********************************************/
void MPEG_1_III_stereo(
                       double xr[2][SBLIMIT][SSLIMIT],
                       double lr[2][SBLIMIT][SSLIMIT],
                       III_scalefac_t *scalefac,
                       struct gr_info_s *gr_info,
                       decData *dd )
{
    double TmpFac;
    double MPEG_1_2_Factor[2][SBLIMIT*SSLIMIT];
    int stereo = dd->frameData.channels;
    int i_stereo = (dd->frameData.mode == MPG_MD_JOINT_STEREO) &&
        (dd->frameData.mode_ext & 0x1);
    int ms_stereo = (dd->frameData.mode == MPG_MD_JOINT_STEREO) &&
        (dd->frameData.mode_ext & 0x2);
    int sfreq,version;
    int ch, sb, ss;
    int sfbNo, sfbStart;

    int sbTemp, sfb, sfbTemp;
    int i, j;

    sfreq = dd->frameData.sampling_frequency;
    version = dd->frameData.version;

    /* initialization of lr: */
    for (ch = 0; ch < 2; ch++) {
        for (sb = 0; sb < SBLIMIT; sb++) {
            for (ss = 0; ss < SSLIMIT; ss++) {
                lr[ch][sb][ss] = 0;
            }
        }
    }

    if (stereo == 2)
    {
        if (i_stereo)
        {
            if (gr_info->window_switching_flag && (gr_info->block_type == 2))
            {
                if(gr_info->mixed_block_flag)
                {  /* mixed blocks */
                    /* ------------------------ */
                    /* MIXED BLOCKS PROCESSING: */
                    /* ------------------------ */
                    i = 31;	ss = 17; sb = 0;
                    while ( i >= 0 )
                    {
                        if (xr[1][i][ss] != 0.0)
                        {
                            sb = i * 18 + ss;
                            sbTemp = i;
                            i = -1;
                        }
                        else
                        {
                            ss--;
                            if (ss < 0)
                            {
                                i--;
                                sbTemp = i;
                                ss = 17;
                            }
                        }
                    }

                    if (sb < 36)   /* was (sb <= 36) corrected efs 2003-07-04 */
                    {  /* intensity bound inside long blocks */
                        /* ------------------------------------------------------------ */
                        /* MIXED BLOCKS PROCESSING: INTENSITY BOUND INSIDE LONG BLOCKS: */
                        /* ------------------------------------------------------------ */
                        /* 1. long blocks up to intensity border: not intensity */
                        /* ---------------------------------------------------- */
                        i = 0;
                        while (sfBandIndex[version][sfreq].l[i] < sb) i++;
                        sfb = i;  /* from that sfb on intensity stereo */
                        sfbTemp = sfb;  /* save for later use */

                        i = sfBandIndex[version][sfreq].l[i];
                        sfbStart = i;

                        /* from 0 up to sfbStart do ms_stereo or normal stereo */
                        for (i = 0; i < sfbStart; i++) {
                            if (ms_stereo) III_M_S(xr, lr, 0, sfbStart);
                            else III_Stereo(xr, lr, 0, sfbStart);
                        }

                        /* 2. long blocks from intensity border up to sfb band 8: intensity */
                        /* ---------------------------------------------------------------- */
                        /* calc. is_ratio */
                        for ( ; sfb < 8; sfb++)
                        {
                            sfbStart = sfBandIndex[version][sfreq].l[sfb];
                            sfbNo = sfBandIndex[version][sfreq].l[sfb+1]-sfBandIndex[version][sfreq].l[sfb];
                            TmpFac = tan((double)(*scalefac)[1].l[sfb] * (PI /12));
                            i = sfbStart;
                            for (; sfbNo > 0; sfbNo--)
                            {
                                if ((*scalefac)[1].l[sfb] != 7) {
                                    MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                    MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                }
                                i++;
                            }
                        }

                        /* Start of intensity stereo of remaining sfc bands: */
                        for (; sfbTemp < 8; sfbTemp++)
                        {
                            sfbStart = sfBandIndex[version][sfreq].l[sfbTemp];  /* = Start in 0 ... 575 */
                            sfbNo = sfBandIndex[version][sfreq].l[sfbTemp+1]-sfBandIndex[version][sfreq].l[sfbTemp]; /* No of lines to process */

                            if ((*scalefac)[1].l[sfbTemp] != 7)
                            {
                                III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                            }
                            else if (ms_stereo)
                            {
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            }
                            else
                            {
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                            }
                        }  /* for (; sfbTemp < 8; sfbTemp++) */

                        /* calc. 'is_ratio': */
                        /* ----------------- */
                        for (j = 0; j < 3; j++)
                        {
                            sb = 3;

                            while (sb < 13)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sb+1] - sfBandIndex[version][sfreq].s[sb];
                                sfbStart = 3*sfBandIndex[version][sfreq].s[sb] + j * sfbNo;
                                TmpFac = tan((double)(*scalefac)[1].s[j][sb] * (PI /12));
                                i = sfbStart;

                                for (; sfbNo > 0; sfbNo--)
                                {
                                    if ((*scalefac)[1].s[j][sb] != 7)
                                    {
                                        MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                        MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                    }
                                    i++;
                                }
                                sb++;
                            }

                            /* last scale factor band: */
                            sfbNo = sfBandIndex[version][sfreq].s[12]-sfBandIndex[version][sfreq].s[11];
                            sfbStart = 3*sfBandIndex[version][sfreq].s[11] + j * sfbNo;
                            sfbNo = sfBandIndex[version][sfreq].s[13]-sfBandIndex[version][sfreq].s[12];

                            if (sfbTemp <= 11)
                            {
                                TmpFac = tan((double)(*scalefac)[1].s[j][11] * (PI /12));
                            }
                            else
                            {
                                TmpFac = 0.0;  /* if scalefac[11] is not an intensity position, is_pos = 0 */
                            }

                            i = 3*sfBandIndex[version][sfreq].s[11] + j * sfbNo;

                            for (; sfbNo > 0; sfbNo--)
                            {
                                MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                i++;
                            }

                            /* 3. short blocks from sfbcnt to last sfb do intensity stereo:
                            ------------------------------------------------------------ */
                            for (sfbTemp = 3; sfbTemp < 13; sfbTemp++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfbTemp+1]-sfBandIndex[version][sfreq].s[sfbTemp]; /* No of lines to process */
                                sfbStart = 3 * sfBandIndex[version][sfreq].s[sfbTemp] + j * sfbNo;

                                if ((*scalefac)[1].s[j][sfbTemp] != 7)         /* RF bug fix 30-09-2002  */
                                {
                                    III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                                }
                                else if (ms_stereo)
                                {
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                }
                                else
                                {
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                                }
                            }  /* for (; sfbTemp < 22; sfbTemp++) */
                        } /* for (j = 0; j < 3; j++) */
                    }
                    else
                    {  /* intensity bound outside long blocks */
                        /* ------------------------------------------------------------- */
                        /* MIXED BLOCKS PROCESSING: INTENSITY BOUND OUTSIDE LONG BLOCKS: */
                        /* ------------------------------------------------------------- */
                        /* 1. long blocks up to sfb band 8: not intensity */
                        /* ---------------------------------------------- */
                        /* from 0 to sfb 8 ms_stereo or normal stereo */
                        sfbStart = sfBandIndex[version][sfreq].l[8];

                        for (i = 0; i < sfbStart; i++)
                        {
                            if (ms_stereo)
                            {
                                III_M_S(xr, lr, 0, sfbStart);
                            }
                            else
                            {
                                III_Stereo(xr, lr, 0, sfbStart);
                            }
                        }

                        /* 2. short blocks from sfb band 3 up to intensity border: normal stereo, ms stereo and intensity */
                        /* ---------------------------------------------------------------------------------------------- */
                        for (j = 0; j < 3; j++)
                        {
                            int sfbcnt;
                            sfbcnt = -1;

                            for (sfb = 12; sfb >= 3; sfb--)
                            {
                                int lines;
                                lines = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                i = 3*sfBandIndex[version][sfreq].s[sfb] + (j+1) * lines - 1;

                                while (lines > 0)
                                {
                                    if (xr[1][i/SSLIMIT][i%SSLIMIT] != 0.0)
                                    {
                                        sfbcnt = sfb;
                                        sfb = -10;
                                        lines = -10;
                                    }
                                    lines--;
                                    i--;
                                }
                            }

                            sfbcnt += 1;
                            if (sfbcnt < 3) sfbcnt = 3;
                            sfbTemp = sfbcnt;		 /* for later use */


                            /* do normal stereo or MS stereo from sfb 3 to < sfbcnt:
                            ----------------------------------------------------- */
                            for (sb = 3; sb < sfbcnt; sb++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sb+1]-sfBandIndex[version][sfreq].s[sb];
                                sfbStart = 3*sfBandIndex[version][sfreq].s[sb] + j * sfbNo;

                                if (ms_stereo)
                                {
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                }
                                else
                                {
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                                }
                            }

                            /* from sfbcnt to last sfb calculate is_ratio values:
                            -------------------------------------------------- */
                            sb = sfbcnt;
                            while(sb < 13)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sb+1]-sfBandIndex[version][sfreq].s[sb];
                                sfbStart = 3*sfBandIndex[version][sfreq].s[sb] + j * sfbNo;
                                TmpFac = tan((double)(*scalefac)[1].s[j][sb] * (PI /12));
                                i = sfbStart;

                                for (; sfbNo > 0; sfbNo--)
                                {
                                    if ((*scalefac)[1].s[j][sb] != 7)
                                    {
                                        MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                        MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                    }
                                    i++;
                                }
                                sb++;
                            }

                            /* for the last scale factor band; */
                            sfbNo = sfBandIndex[version][sfreq].s[12]-sfBandIndex[version][sfreq].s[11];
                            sfbStart = 3*sfBandIndex[version][sfreq].s[11] + j * sfbNo;
                            sfbNo = sfBandIndex[version][sfreq].s[13]-sfBandIndex[version][sfreq].s[12];

                            if (sfbcnt <= 11)
                            {
                                TmpFac = tan((double)(*scalefac)[1].s[j][11] * (PI /12));
                            }
                            else
                            {
                                TmpFac = 0.0;  /* if scalefac[11] is not an intensity position, is_pos = 0 */
                            }

                            i = 3*sfBandIndex[version][sfreq].s[11] + j * sfbNo;

                            for (; sfbNo > 0; sfbNo--)
                            {
                                MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                i++;
                            }

                            /* from sfbcnt to last sfb do intensity stereo:
                            -------------------------------------------- */
                            for (; sfbTemp < 13; sfbTemp++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfbTemp+1]-sfBandIndex[version][sfreq].s[sfbTemp]; /* No of lines to process */
                                sfbStart = 3 * sfBandIndex[version][sfreq].s[sfbTemp] + j * sfbNo;

                                if ((*scalefac)[1].s[j][sfbTemp] != 7)  /* RF bug fix 30-09-2002 */
                                {
                                    III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                                }
                                else if (ms_stereo)
                                {
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                }
                                else
                                {
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                                }
                            }  /* for (; sfbTemp < 22; sfbTemp++) */
                        } /* for (j = 0; j < 3; j++) */
                    }
                }  /* if (gr_info->mixed_block_flag) */
                else
                {
                    /* short blocks */
                    /* ----------------------- */
                    /* SHORT BLOCK PROCESSING: */
                    /* ----------------------- */
                    for (j = 0; j < 3; j++)
                    {
                        int sfbcnt;
                        sfbcnt = -1;

                        for (sfb = 12; sfb >= 0; sfb--)
                        {
                            int lines;
                            lines = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                            i = 3*sfBandIndex[version][sfreq].s[sfb] + (j+1) * lines - 1;

                            while (lines > 0)
                            {
                                if (xr[1][i/SSLIMIT][i%SSLIMIT] != 0.0)
                                {
                                    sfbcnt = sfb;
                                    sfb = -10;
                                    lines = -10;
                                }
                                lines--;
                                i--;
                            }
                        }

                        sfbcnt += 1;
                        sfbTemp = sfbcnt;		 /* for later use */

                       /* do normal stereo or MS stereo from 0 to sfbcnt:
                        ----------------------------------------------- */
                        /*for (sb = sfbcnt; sb > 0; sb--) { */
                        for (sb = 0; sb < sfbcnt; sb++)
                        {
                            sfbNo = sfBandIndex[version][sfreq].s[sb+1]-sfBandIndex[version][sfreq].s[sb];
                            sfbStart = 3*sfBandIndex[version][sfreq].s[sb] + j * sfbNo;

                            if (ms_stereo)
                            {
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            }
                            else
                            {
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                            }
                        }

                        /* calc. 'is_ratio':
                        ----------------- */
                        sb = sfbcnt;
                        while(sb < 13)
                        {
                            sfbNo = sfBandIndex[version][sfreq].s[sb+1]-sfBandIndex[version][sfreq].s[sb];
                            sfbStart = 3*sfBandIndex[version][sfreq].s[sb] + j * sfbNo;
                            TmpFac = tan((double)(*scalefac)[1].s[j][sb] * (PI /12));
                            i = sfbStart;

                            for (; sfbNo > 0; sfbNo--)
                            {
                                if ((*scalefac)[1].s[j][sb] != 7)
                                {
                                    MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                                    MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                                }
                                i++;
                            }
                            sb++;
                        }

                        sfbNo = sfBandIndex[version][sfreq].s[12]-sfBandIndex[version][sfreq].s[11];
                        sfbStart = 3*sfBandIndex[version][sfreq].s[11] + j * sfbNo;
                        sfbNo = sfBandIndex[version][sfreq].s[13]-sfBandIndex[version][sfreq].s[12];

                        if (sfbcnt <= 11)
                        {
                            TmpFac = tan((double)(*scalefac)[1].s[j][11] * (PI /12));
                        }
                        else
                        {
                            TmpFac = 0.0;  /* if scalefac[11] is not an intensity position, is_pos = 0 */
                        }

                        /*  18-09-2002 RF bug fix */
                        i = 3*sfBandIndex[version][sfreq].s[12] + j * sfbNo;

                        for (; sfbNo > 0; sfbNo--)
                        {
                            MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                            MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                            i++;
                        }

                        /* from sfbcnt to last sfb do intensity stereo:
                        -------------------------------------------- */
                        for (; sfbTemp < 13; sfbTemp++)
                        {
                            sfbNo = sfBandIndex[version][sfreq].s[sfbTemp+1]-sfBandIndex[version][sfreq].s[sfbTemp]; /* No of lines to process */
                            sfbStart = 3 * sfBandIndex[version][sfreq].s[sfbTemp] + j * sfbNo;

                            if ((*scalefac)[1].s[j][sfbTemp] != 7)  /* RF bug fix 30-09-2002 */
                            {
                                III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo); /* corrected 20.09.02 W. Voessing */
                            }
                            else if (ms_stereo)
                            {
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            }
                            else
                            {
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                            }
                        }  /* for (; sfbTemp < 22; sfbTemp++) */
                    } /* for (j = 0; j < 3; j++) */
                } /* if( gr_info->mixed_block_flag) */
            }  /* if (gr_info->window_switching_flag && (gr_info->block_type == 2)) */
            else
            { /* long blocks */
                /* ---------------------- */
                /* LONG BLOCK PROCESSING:	*/
                /* ---------------------- */
                i = 31;
                ss = 17;
                sb = 0;

                while ( i >= 0 )
                {
                    if (xr[1][i][ss] != 0.0)
                    {
                        sb = i * 18 + ss;
                        sbTemp = i;
                        /*  i = -1     patched RF    24-09-2002   */
                        i = -2;
                    }
                    else
                    {
                        ss--;
                        if (ss < 0)
                        {
                            i--;
                            sbTemp = i;
                            ss = 17;
                        }
                    }
                }

                /*  patched RF    24-09-2002   */
                if (sb == 0)
                {
                    if (i == -1)
                    {
                        /*  all xr[1][][] are 0: set IS bound sfb to 0  */
                        i = 0;
                    }
                    else
                    {
                        /*  xr[1][0][0] is unequal 0 and all others are 0: set IS bound sfb to 1 */
                        i = 1;
                    }
                }
                else
                {
                    i = 0;
                    while ( sfBandIndex[version][sfreq].l[i] <= sb )
                        i++;
                }

                sfb = i;  /* from that sfb on intensity stereo */
                sfbTemp = sfb;  /* save for later use */

                i = sfBandIndex[version][sfreq].l[i];
                sfbStart = i;

                /* cal. 'is_ratio': */
                for ( ; sfb < 21; sfb++)
                {
                    sb = sfBandIndex[version][sfreq].l[sfb+1] - sfBandIndex[version][sfreq].l[sfb];
                    TmpFac = tan((double)(*scalefac)[1].l[sfb] * (PI / 12));

                    for ( ; sb > 0; sb--)
                    {
                        if ((*scalefac)[1].l[sfb] != 7)
                        {
                            MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                            MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                        }
                        i++;
                    }
                }

                /* last scale factor band: */
                sfb = sfBandIndex[version][sfreq].l[20];

                if (sfbTemp <= 20)
                {
                    TmpFac = tan((double)(*scalefac)[1].l[20] * (PI /12));
                }
                else
                {
                    TmpFac = 0.0;  /* if scalefac[20] is not an intensity position, is_pos = 0 */
                }

                for (sb = 576 - sfBandIndex[version][sfreq].l[21]; sb > 0; sb--)
                {
                    MPEG_1_2_Factor[0][i] = TmpFac / (1 + TmpFac);
                    MPEG_1_2_Factor[1][i] = 1 / (1 + TmpFac);
                    i++;
                }

                /* from 0 to sfbStart ms_stereo or normal stereo */
                if (ms_stereo)
                {
                    III_M_S(xr, lr, 0, sfbStart);
                }
                else
                {
                    III_Stereo(xr, lr, 0, sfbStart);
                }

                /* now intensity stereo of the remaining sfb's: */
                for (; sfbTemp < 22; sfbTemp++)
                {
                    sfbStart = sfBandIndex[version][sfreq].l[sfbTemp];
                    sfbNo = sfBandIndex[version][sfreq].l[sfbTemp+1]-sfBandIndex[version][sfreq].l[sfbTemp]; /* No of lines to process */

                    if ((*scalefac)[1].l[sfbTemp] != 7)
                    {
                        III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                    }
                    else if (ms_stereo)
                    {
                        III_M_S(xr, lr, sfbStart, sfbNo);
                    }
                    else
                    {
                        III_Stereo(xr, lr, sfbStart, sfbNo);
                    }
                }  /* for (; sfbTemp < 22; sfbTemp++) */
            }  /* if (gr_info->window_switching_flag && (gr_info->block_type == 2)) */
        }  /* if (i_stereo)  */
        else
        {  /* normal stereo or ms stereo */
            /* ------------------------------- */
            /* NORMAL OR MS STEREO PROCESSING: */
            /* ------------------------------- */
            if (ms_stereo)
            {
                III_M_S(xr, lr, 0, 576);
            }
            else
            {
                III_Stereo(xr, lr, 0, 576);
            }
        } /* if (i_stereo) */
   }  /* if (stereo == 2) */
   else
   {  /* mono processing */
       /* ---------------- */
       /* MONO PROCESSING: */
       /* ---------------- */
       for (sb = 0; sb < SBLIMIT; sb++)
       {
           for (ss = 0; ss < SSLIMIT; ss++)
           {
               lr[0][sb][ss] = xr[0][sb][ss];
           }
       }
   }  /* if (stereo == 2) */
}
/****************************************************************************************************************/

/**************************** NEW MPEG2, LAYER 3 STEREO FUNCTION 08.05.02 W. Voessing ***************************/
void MPEG_2_III_stereo(
                       double xr[2][SBLIMIT][SSLIMIT],
                       double lr[2][SBLIMIT][SSLIMIT],
                       III_scalefac_t *scalefac,
                       struct gr_info_s *gr_info_l,
                       struct gr_info_s *gr_info_r,
                       decData *dd, 
                       unsigned int scalefac_IIP_buffer[54])
{
    double MPEG_1_2_Factor[2][SBLIMIT*SSLIMIT];
    int stereo = dd->frameData.channels;
    int i_stereo = (dd->frameData.mode == MPG_MD_JOINT_STEREO) &&
        (dd->frameData.mode_ext & 0x1);
    int ms_stereo = (dd->frameData.mode == MPG_MD_JOINT_STEREO) &&
        (dd->frameData.mode_ext & 0x2);
    int sfreq,version;
    int ch, sb, ss;
    int sfbNo, sfbStart;

    int sbTemp, sfb, sfbTemp;
    int i, j;

    double io;
    
    if ((gr_info_r->scalefac_compress % 2) == 1)
    {
        io = (double)0.707106781188;
    }
    else
    {
        io = (double)0.840896415256;
    }
    
    sfreq = dd->frameData.sampling_frequency;
    version = dd->frameData.version;

         
    /* initialization of lr: */
    for (ch = 0; ch < 2; ch++)
    {
        for (sb = 0; sb < SBLIMIT; sb++)
        {
            for (ss = 0; ss < SSLIMIT; ss++)
            {
                lr[ch][sb][ss] = 0;
            }
        }
    }
  
/*    if (gr_info_l->block_type == 2) return;*/
/*    if (gr_info_r->block_type != 2) return;*/
    
    if (stereo == 2)
    {
        if (i_stereo)
        {
            if (gr_info_l->window_switching_flag && (gr_info_l->block_type == 2))
            {
                if(gr_info_l->mixed_block_flag)
                {   
		    /* ------------------------ */
                    /* MIXED BLOCKS PROCESSING: */
                    /* ------------------------ */
		    /*  start of corrected version by efs 2003-07-04  */
                    i = 31;
                    ss = 17;
                    sb = -1;
                    while ( i >= 0 )
                    {
                        if (xr[1][i][ss] != 0.0)
                        {
                            sb = i * 18 + ss;
                            i = -1;
                        }
                        else
                        {
                            ss--;
                            if (ss < 0)
                            {
                                i--;
                                ss = 17;
                            }
                        }
                    }   /* now sb is the number of highest line with value != 0      */
                        /* can be between -1 (all lines zero) and 575 (no line zero) */

                    if (sb < 36)    /*  was (sb <= 36)  */
                    {
                        /* ------------------------------------------------------------ */
                        /* MIXED BLOCKS PROCESSING: INTENSITY BOUND INSIDE LONG BLOCKS: */
                        /* ------------------------------------------------------------ */
                        /* 1. long blocks up to intensity border: Stereo or M/S */
                        /* ---------------------------------------------------- */
                        i = 0;
                        while (sfBandIndex[version][sfreq].l[i] <= sb) i++;
                        sfbTemp = i;  /* from that (long) sfb on we have intensity stereo */

                        sfbNo = sfBandIndex[version][sfreq].l[sfbTemp];            /* number of lines to process */
                        sfbStart = 0;                                     /* start of sfb */

                        /* from sfbStart up sfbNo lines do ms_stereo or normal stereo */
                        
			for (i = 0; i < sfbStart; i++)
                        {
                            if (ms_stereo)
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            else
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                        }

                        /* 2. long blocks from intensity border up to sfb band 6: intensity */
                        /* ---------------------------------------------------------------- */
                        /* calc. MPEG_1_2_Factor[0], MPEG_1_2_Factor[1] */
                        for (sfb = sfbTemp; sfb < 6; sfb++)
                        {
                            sfbStart = sfBandIndex[version][sfreq].l[sfb];
                            sfbNo = sfBandIndex[version][sfreq].l[sfb+1]-sfBandIndex[version][sfreq].l[sfb];

                            i = sfbStart;
                            for (; sfbNo > 0; sfbNo--)
                            {
                                if ((unsigned int)((*scalefac)[1].l[sfb]) != scalefac_IIP_buffer[sfb])
                                {
                                    III_i_stereo_k_values((*scalefac)[1].l[sfb],io,i,MPEG_1_2_Factor);
                                    i++;
                                }
                            }
                        }

                        for ( sfb = sfbTemp; sfb < 6; sfb++)
                        {
                            sfbStart = sfBandIndex[version][sfreq].l[sfb];  /* = Start in 0 ... 575 */
                            sfbNo = sfBandIndex[version][sfreq].l[sfb+1]-sfBandIndex[version][sfreq].l[sfb]; /* No of lines to process */

                            if ((unsigned int)((*scalefac)[1].l[sfb]) != scalefac_IIP_buffer[sfb])
                                III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                            else if (ms_stereo)
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            else
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                        }

                        /* 3. now process all sfb with short blocks (3...12), all in intensity mode */
                        /* ------------------------------------------------------------------------ */

                        for (j = 0; j < 3; j++)
                        {
                        /* first calculate directional factors for intensity stereo,
                           for all sfb in intensity mode, but only
                           if they do not have "illegal" position: */
                        /* to do this for all sfb we have to get information for last scale factor band: */
		        /* here we clearly have more than one sfb in intensity mode,
		           so copy factors and legal/illegal information from sfb11 to sfb12 */
                            sfb = 12;
                            ((*scalefac)[1].s[j][sfb]) = ((*scalefac)[1].s[j][sfb-1]);
                            scalefac_IIP_buffer[3*sfb + j] = scalefac_IIP_buffer[3*(sfb-1) + j];  /* legal/illegal in sfb 12 same as in sfb 11 */

                            for (sfb = 3 ; sfb <= 12; sfb++)
                            {
                                if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])   /* RF bug fix 30-09-2002 */
                                {
                                    sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                    i = 3*sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                                    for (sb = 0; sb < sfbNo; sb++)
                                    {
                                        III_i_stereo_k_values((*scalefac)[1].s[j][sfb],io,i,MPEG_1_2_Factor);
                                        i++;
                                    }
                                }
                            }

                            for (sfb = 3; sfb < 13; sfb++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb]; /* No of lines to process */
                                sfbStart = 3 * sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                                if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])   /* RF bug fix 30-9-2002 */
                                    III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo); /* corrected 20.09.02 W. Voessing */
                                else if (ms_stereo)
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                else
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                            }
                        } /* for (j = 0; j < 3; j++) */
                    }
                    else
                    {
                        /* ------------------------------------------------------------- */
                        /* MIXED BLOCKS PROCESSING: INTENSITY BOUND OUTSIDE LONG BLOCKS: */
                        /* ------------------------------------------------------------- */
                        /* 1. long blocks up to sfb band 6: Stereo or M/S */
                        /* ---------------------------------------------- */

                        sfbNo = sfBandIndex[version][sfreq].l[6];                  /* number of lines to process */
                        sfbStart = 0;                                     /* start of sfb */
                        /* from sfbStart up sfbNo lines do ms_stereo or normal stereo */
   	                if (ms_stereo)
                            III_M_S(xr, lr, sfbStart, sfbNo);
                        else
                            III_Stereo(xr, lr, sfbStart, sfbNo);

                        /* 2. short blocks, do for all 3  */
                        /* ------------------------------ */
                        for (j = 0; j < 3; j++)
                        {
                            int sfbcnt;
                            sfbcnt = -1;

                            for (sfb = 12; sfb >= 3; sfb--)
                            {
                                int lines;
                                lines = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                i = 3*sfBandIndex[version][sfreq].s[sfb] + (j+1) * lines - 1;

                                while (lines > 0)
                                {
                                    if (xr[1][i/SSLIMIT][i%SSLIMIT] != 0.0)
                                    {
                                        sfbcnt = sfb;
                                        sfb = -10;
                                        lines = -10;
                                    }
                                    lines--;
                                    i--;
                                }
                            }

                            sfbcnt += 1;
                            if (sfbcnt < 3)
                            {
                                sfbcnt = 3;   /* should not be necessary */
                            }

                            sfbTemp = sfbcnt; /* from this (short) sfb on we have intensity mode        */
                                              /* can have values between 3 (all short sfb in intensity) */
                                              /* and 13 (no short sfb in intensity mode)                */

                            /* 3. from sfbTemp to last sfb calculate is_ratio values:    */
                            /* -----------------------------------------------------     */
                            /* first calculate directional factors for intensity stereo, */
                            /* for all sfb in intensity mode, but only                   */
                            /* if they do not have "illegal" position:                   */

                            /* to do this for all sfb we have to get information for last scale factor band: */
                            /*  get factors for last scale factor band: */
                            sfb = 12;
                            if (sfbTemp < sfb)
	                    /* more than one sfb in intensity mode,
		               copy factors and legal/illegal information from sfb11 to sfb12 */
		            {
		                ((*scalefac)[1].s[j][sfb]) = ((*scalefac)[1].s[j][sfb-1]);
                                scalefac_IIP_buffer[3*sfb + j] = scalefac_IIP_buffer[3*(sfb-1) + j];   /* legal/illegal in sfb 12 same as in sfb 11 */
		            }
                            else if (sfbTemp == sfb)
                            /* only sfb 12 in intensity mode, use factors corresponding to is_pos[12] == 0 */
                            {
		                ((*scalefac)[1].s[j][sfb]) = 0;
		                scalefac_IIP_buffer[3*sfb + j] = 1;	/* the scf value 0 in sfb12 is "legal" */
        	            }
                            /* if sfbTemp > sfb (no sfb in intensity mode): do nothing */

                            for (sfb = sfbTemp ; sfb <= 12; sfb++)
                            {
                                if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])   /* RF bug fix 30-09-2002 */
                                {
                                    sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                    i = 3*sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                                    for (sb = 0; sb < sfbNo; sb++)
                                    {
                                        III_i_stereo_k_values((*scalefac)[1].s[j][sfb],io,i,MPEG_1_2_Factor);
                                        i++;
                                    }
                                }
                            }

                            /* 4. do normal stereo or MS stereo from sfb 3 to < sfbTemp: */
                            /* --------------------------------------------------------- */
                            for (sfb = 3; sfb < sfbTemp; sfb++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                sfbStart = 3*sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;

                                if (ms_stereo)
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                else
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                            }

                            /* 5. now intensity stereo processing of the remaining sfb's: */
                            /* ---------------------------------------------------------- */
                            for (sfb = sfbTemp; sfb < 13; sfb++)
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb]; /* No of lines to process */
                                sfbStart = 3 * sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                                if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])    /* RF bug fix 30-09-2002 */
                                    III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
                                else if (ms_stereo)
                                    III_M_S(xr, lr, sfbStart, sfbNo);
                                else
                                    III_Stereo(xr, lr, sfbStart, sfbNo);
                            }
			/*  end of correction by efs 2003-07-04 */
                        } /* for (j = 0; j < 3; j++) */
                    }  /* if intensity bound inside or outside long blocks */
                }  /* if (gr_info->mixed_block_flag) */
                else
                {  /* short blocks  */
                    /* ----------------------- */
                    /* SHORT BLOCK PROCESSING: */
                    /* ----------------------- */
                    
		    for (j = 0; j < 3; j++)
                    {
                        int sfbcnt;
                        sfbcnt = -1;

                        for (sfb = 12; sfb >= 0; sfb--)
                        {
                            int lines;
                            lines = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                            i = 3*sfBandIndex[version][sfreq].s[sfb] + (j+1) * lines - 1;

                            while (lines > 0)
                            {
                                if (xr[1][i/SSLIMIT][i%SSLIMIT] != 0.0)
                                {
                                    sfbcnt = sfb;
                                    sfb = -10;
                                    lines = -10;
                                }
                                lines--;
                                i--;
                            }
                        }

                        sfbcnt += 1;

                        /*  start of corrected version by efs 2003-07-04  */
                        sfbTemp = sfbcnt; /* from this (short) sfb on we have intensity mode        */
                                          /* can have values between 3 (all short sfb in intensity) */
                                          /* and 13 (no short sfb in intensity mode)                */

                        /* first calculate directional factors for intensity stereo,
                           for all sfb in intensity mode, but only
                           if they do not have "illegal" position: */

                        /* to do this for all sfb we have to get information for last scale factor band: */
                        /* get factors for last scale factor band: */
                        sfb = 12;
                        if (sfbTemp < sfb)
		                /* more than one sfb in intensity mode,
		                   copy factors and legal/illegal information from sfb11 to sfb12 */
		                {
		                    ((*scalefac)[1].s[j][sfb]) = ((*scalefac)[1].s[j][sfb-1]);
		                    scalefac_IIP_buffer[3*sfb + j] = scalefac_IIP_buffer[3*(sfb-1) + j];  /* legal/illegal in sfb 12 same as in sfb 11 */
		                }
                        else if (sfbTemp == sfb)
                        /* only sfb 12 in intensity mode, use factors corresponding to is_pos[12] == 0 */
                        {
		                    ((*scalefac)[1].s[j][sfb]) = 0;
		                    scalefac_IIP_buffer[3*sfb + j] = 1;	/* the scf value 0 in sfb12 is "legal" */
        	            }
                        /* if sfbTemp > sfb (no sfb in intensity mode): do nothing */

                        for (sfb = sfbTemp ; sfb <= 12; sfb++)
                        {
                            if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])   /* RF bug fix 30-09-2002 */
                            {
                                sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                                i = 3*sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                                for (sb = 0; sb < sfbNo; sb++)
                                {
                                    III_i_stereo_k_values((*scalefac)[1].s[j][sfb],io,i,MPEG_1_2_Factor);
                                    i++;
                                }
                            }
                        }

                        /* Now process audio samples */
                        /* first process lower sfb's not in intensity mode */
                        for (sfb = 0; sfb < sfbTemp; sfb++)
                        {
                            sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb];
                            sfbStart = 3*sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                            if (ms_stereo)
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            else
                                III_Stereo(xr, lr, sfbStart, sfbNo);
                        }

                        /* now intensity stereo processing of the remaining sfb's: */
                        for (sfb = sfbTemp; sfb < 13; sfb++)
                        {
                            sfbNo = sfBandIndex[version][sfreq].s[sfb+1]-sfBandIndex[version][sfreq].s[sfb]; /* No of lines to process */
                            sfbStart = 3 * sfBandIndex[version][sfreq].s[sfb] + j * sfbNo;
                            if ((unsigned int)((*scalefac)[1].s[j][sfb]) != scalefac_IIP_buffer[3*sfb + j])   /* RF bug fix 30-9-2002 */
                                III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo); /* corrected 20.09.02 W. Voessing */
                            else if (ms_stereo)
                                III_M_S(xr, lr, sfbStart, sfbNo);
                            else {
				    III_Stereo(xr, lr, sfbStart, sfbNo);
			    }
                        }
                       /*  end of correction by efs 2003-07-04 */
                    } /* for (j = 0; j < 3; j++) */
                } /* if( gr_info->mixed_block_flag) */
            }  /* if (gr_info->window_switching_flag && (gr_info->block_type == 2)) */
            else
            { /* long blocks */
                /* ---------------------- */
                /* LONG BLOCK PROCESSING:	*/
                /* ---------------------- */
                i = 31;
                ss = 17;
                sb = 0;

                while ( i >= 0 )
                {
                    if (xr[1][i][ss] != 0.0)
                    {
                        sb = i * 18 + ss;
                        sbTemp = i;
                        /*  i = -1     patched RF    24-09-2002   */
                        i = -2;
                    }
                    else
                    {
                        ss--;
                        if (ss < 0)
                        {
                            i--;
                            sbTemp = i;
                            ss = 17;
                        }
                    }
                }

                /*  patched RF    24-09-2002   */
                if (sb == 0)
                {
                    if (i == -1)
                    {
                        /*  all xr[1][][] are 0: set IS bound sfb to 0  */
                        i = 0;
                    }
                    else
                    {
                        /*  xr[1][0][0] is unequal 0 and all others are 0: set IS bound sfb to 1 */
                        i = 1;
                    }
                }
                else
                {
                    i = 0;
                    while ( sfBandIndex[version][sfreq].l[i] <= sb )
                        i++;
                }

		/*  corrected version by efs 2003-07-04  */
                sfbTemp = i;  /* from this (long) sfb on we have intensity mode        */
                              /* can have values between 0 (all long sfb in intensity) */
                              /* and 22 (no long sfb in intensity mode)                */

                /* first calculate directional factors for intensity stereo,
                   for all sfb in intensity mode, but only if they
                   do not have "illegal" position: */

                /* to do this for all sfb we have to get information for last scale factor band: */
                sfb = 21;
                if (sfbTemp < sfb)
                /* more than one sfb in intensity mode, */
                /* copy factors and legal/illegal information from sfb20 to sfb21 */
                {
                    ((*scalefac)[1].l[sfb]) = ((*scalefac)[1].l[sfb-1]);
                    scalefac_IIP_buffer[sfb] = scalefac_IIP_buffer[sfb-1];	/* legal/illegal in sfb 21 same as in sfb 20 */
                }
                else if (sfbTemp == sfb)
                /* only sfb 21 in intensity mode, is_pos[21] = 0 */
                {
                    ((*scalefac)[1].l[sfb]) = 0;
                    scalefac_IIP_buffer[sfb] = 1;	/* the scf value 0 in sfb21 is "legal" */
                }
                /* if sfbTemp > sfb (no sfb in intensity mode): do nothing */

                for (sfb = sfbTemp ; sfb <= 21; sfb++)
                {
                    if ((unsigned int)((*scalefac)[1].l[sfb]) != scalefac_IIP_buffer[sfb])
                    {
                        sfbNo = sfBandIndex[version][sfreq].l[sfb+1] - sfBandIndex[version][sfreq].l[sfb];
                        i = sfBandIndex[version][sfreq].l[sfb];
                        for (sb=0 ; sb < sfbNo; sb++)
                        {
                            III_i_stereo_k_values((*scalefac)[1].l[sfb],io,i,MPEG_1_2_Factor);
                            i++;
                        }
                    }
                }

                /* Now process audio samples */
                /* first process lower sfb's not in intensity mode */
                sfb=0;
                sfbNo = sfBandIndex[version][sfreq].l[sfbTemp]-sfBandIndex[version][sfreq].l[sfb];
                sfbStart = sfBandIndex[version][sfreq].l[sfb];
                if (ms_stereo)
                    III_M_S(xr, lr, sfbStart, sfbNo);
                else {
                    III_Stereo(xr, lr, sfbStart, sfbNo);
		}

                /* now intensity stereo processing of the remaining sfb's: */
                for (sfb = sfbTemp; sfb < 22; sfb++)
                {
                    sfbNo = sfBandIndex[version][sfreq].l[sfb+1]-sfBandIndex[version][sfreq].l[sfb]; /* number of lines to process */
                    sfbStart = sfBandIndex[version][sfreq].l[sfb];                          /* start of sfb */


                    if ((unsigned int)((*scalefac)[1].l[sfb]) != scalefac_IIP_buffer[sfb]) /* "legal" position ? */
		    {
                        III_Intensity(xr, lr, MPEG_1_2_Factor, sfbStart, sfbNo);
		    }
                    else if (ms_stereo)
		    {
                        III_M_S(xr, lr, sfbStart, sfbNo);
		    }
                    else
		    {
                        III_Stereo(xr, lr, sfbStart, sfbNo);
		    }
                }  /* for (sfb = sfbTemp; sfb < 22; sfb++) */
/*  end of correction by efs 2003-07-04 */

            }  /* if (gr_info->window_switching_flag && (gr_info->block_type == 2)) */
        }  /* if (i_stereo) */
        else
        {  /* normal stereo or ms stereo */
            /* ------------------------------- */
            /* NORMAL OR MS STEREO PROCESSING: */
            /* ------------------------------- */
            
	
	    if (ms_stereo)
	    {
                III_M_S(xr, lr, 0, 576);
	    }
            else 
	    {
                III_Stereo(xr, lr,0,576); 
	    }
        } /* if (i_stereo) */
    }  /* if (stereo == 2) */
    else
    {  /* mono processing */
        /* ---------------- */
        /* MONO PROCESSING: */
        /* ---------------- */
        for (sb = 0; sb < SBLIMIT; sb++)
        {
            for (ss = 0; ss < SSLIMIT; ss++)
            {
                lr[0][sb][ss] = xr[0][sb][ss];
            }
        }
    }  /* if (stereo == 2) */
}
/****************************************************************************************************************/

double Ci[8]={-0.6,-0.535,-0.33,-0.185,-0.095,-0.041,-0.0142,-0.0037};

void III_antialias_init(antialiasData *aa)
{
    int i;
    double sq;
    for (i=0;i<8;i++) 
    {
        sq=sqrt(1.0+Ci[i]*Ci[i]);
        aa->cs[i] = 1.0/sq;
        aa->ca[i] = Ci[i]/sq;
    }
}

void III_antialias(
                   double xr[SBLIMIT][SSLIMIT],
                   double hybridIn[SBLIMIT][SSLIMIT],
                   struct gr_info_s *gr_info,
                   int antialias_mixed_blocks,
                   antialiasData *aa)
{
    double        bu,bd;  /* upper and lower butterfly inputs */
    int           ss,sb,sblim;

    /* clear all inputs */

    for(sb=0;sb<SBLIMIT;sb++)
        for(ss=0;ss<SSLIMIT;ss++)
            hybridIn[sb][ss] = xr[sb][ss];

    if  (gr_info->window_switching_flag && (gr_info->block_type == 2) &&
         !gr_info->mixed_block_flag ) return;

    if ( gr_info->window_switching_flag && gr_info->mixed_block_flag &&
         (gr_info->block_type == 2))
      sblim = (antialias_mixed_blocks?1:0);
    else
      sblim = SBLIMIT-1;

    /* 31 alias-reduction operations between each pair of sub-bands */
    /* with 8 butterflies between each pair                         */

    for(sb=0;sb<sblim;sb++)
      for(ss=0;ss<8;ss++) {
        bu = xr[sb][17-ss];
        bd = xr[sb+1][ss];
        hybridIn[sb][17-ss] = (bu * aa->cs[ss]) - (bd * aa->ca[ss]);
        hybridIn[sb+1][ss] = (bd * aa->cs[ss]) + (bu * aa->ca[ss]);
      }

}

void initMdct(mdctData *md)
{
    int i;
    
    /* type 0 - normal window */
    for(i=0;i<36;i++)
        md->win[0][i] = sin( PI/36 *(i+0.5) );

    /* type 1 - start window */
    for(i=0;i<18;i++)
        md->win[1][i] = sin( PI/36 *(i+0.5) );
    for(i=18;i<24;i++)
        md->win[1][i] = 1.0;
    for(i=24;i<30;i++)
        md->win[1][i] = sin( PI/12 *(i+0.5-18) );
    for(i=30;i<36;i++)
        md->win[1][i] = 0.0;

    /* type 3 - stop window */
    for(i=0;i<6;i++)
        md->win[3][i] = 0.0;
    for(i=6;i<12;i++)
        md->win[3][i] = sin( PI/12 *(i+0.5-6) );
    for(i=12;i<18;i++) 
        md->win[3][i] =1.0;
    for(i=18;i<36;i++) 
        md->win[3][i] = sin( PI/36*(i+0.5) );

    /* type 2*/
    for(i=0;i<12;i++)
        md->win[2][i] = sin( PI/12*(i+0.5) ) ;
    for(i=12;i<36;i++)
        md->win[2][i] = 0.0 ;

    for (i=0; i<4*36; i++)
        md->COS[i] = cos(PI/(2*36) * i);
}

void inv_mdct(
              double in[18],
              double out[36],
              int block_type,
              mdctData *md )
{
    /*------------------------------------------------------------------*/
    /*                                                                  */
    /*    Function: Calculation of the inverse MDCT                     */
    /*    In the case of short blocks the 3 output vectors are already  */
    /*    overlapped and added in this modul.                           */
    /*                                                                  */
    /*    New layer3                                                    */
    /*                                                                  */
    /*------------------------------------------------------------------*/

    int     i, m, N, p;
    double  tmp[12],sum;

    for(i=0;i<36;i++)
        out[i]=0;
    
    if(block_type == 2 /* 2 */){
	 N=12;
        for(i=0;i<3;i++){
            for(p= 0;p<N;p++){
                sum = 0.0;
                for(m=0;m<N/2;m++)
                    sum += in[i+3*m] * cos( PI/(2*N)*(2*p+1+N/2)*(2*m+1));
                tmp[p] = sum * md->win[block_type][p] ;
            }
            for(p=0;p<N;p++) {
                out[6*i+p+6] += tmp[p];
	    }
        }
    }
    else{
        N=36;
        for(p= 0;p<N;p++){
            sum = 0.0;
            for(m=0;m<N/2;m++)
	    {
                sum += in[m] * md->COS[((2*p+1+N/2)*(2*m+1))%(4*36)];
	    }
            out[p] = sum * md->win[block_type][p];
        }
    }
}

void III_hybrid_init( double prevblck[2][SBLIMIT][SSLIMIT] )
{
        int i,j,k;

        for(i=0;i<2;i++)
          for(j=0;j<SBLIMIT;j++)
            for(k=0;k<SSLIMIT;k++)
              prevblck[i][j][k]=0.0;
}

void III_hybrid(
                double fsIn[SSLIMIT],   /* freq samples per subband in */
                double tsOut[SSLIMIT],  /* time samples per subband out */
                int sb,
                int ch,
                struct gr_info_s *gr_info,
                mdctData *md,
                double prevblck[2][SBLIMIT][SSLIMIT] )
{
    int ss;
    double rawout[36];
    int bt;


    bt = (gr_info->window_switching_flag && gr_info->mixed_block_flag &&
        (sb < 2)) ? 0 : gr_info->block_type;
    
    /* inverse mdct */
    inv_mdct( fsIn, rawout, bt, md);

    /* overlap addition */
    for(ss=0; ss<SSLIMIT; ss++) {
        tsOut[ss] = rawout[ss]+ prevblck[ch][sb][ss];
        prevblck[ch][sb][ss] = rawout[ss+18];
    }
}

