/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ??? 
 * 
 * and edited by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 *
 * $Id: decoder.h,v 1.22 2007/11/07 12:45:14 matejkml Exp $
 *
 * $Log: decoder.h,v $
 * Revision 1.22  2007/11/07 12:45:14  matejkml
 * changed use_afsp to "#if 0"
 *
 * Revision 1.21  2007/09/27 12:59:42  matejkml
 * structural changes
 *
 * Revision 1.20  2007/03/29 14:04:21  matejkml
 * mask for free mode added
 *
 * Revision 1.19  2007/01/10 13:36:23  matejkml
 * bugfix2
 *
 * Revision 1.14  2006/03/10 15:11:56  dodenhen
 * Added MPEG_2_5 support
 *
 * Revision 1.13  2004/10/04 14:08:45  kaehleof
 * added switch for alias reduction in mixed blocks
 *
 * Revision 1.12  2004/08/10 14:34:15  sps
 * added some code to allow for investigation of channel configuration
 * changes, to be enabled using the define CHANNEL_INVESTIGATION in
 * decoder.h
 *
 * Revision 1.11  2003/11/13 12:20:31  tmm01
 *   by Ernst F. Schroeder, look for: efs
 * + added routines for CRC check in Layer III
 *   CRC errors are displayed and counted -
 *   no further action is taken
 *
 * Revision 1.10  2003/05/13 17:15:34  sps
 *  + bug fix in SubBandSynthesis() (previously introduced while changing
 *    to automatic arrays (was previously malloc))
 *
 *  + revision of the out_fifo interface (only the error code is returned,
 *    the clipping information became a function parameter)
 *
 *  + revised the code such that it can be compiled using g++
 *
 *  + functions returning an error code have now the return type
 *    ERROR_CODE. This concerns
 *     - out_fifo()
 *     - SubBandSynthesis()
 *     - III_get_side_info()
 *     - III_huffman_decode()
 *     - read_syn_window()
 *     - open_bit_stream_r()
 *     - initialize_huffman()
 *     - GetArguments()
 *     - aiff_read_headers()
 *     - aiff_seek_to_sound()
 *     - aiff_write_headers()
 *
 * Revision 1.9  2003/05/13 13:53:29  sps
 *  + bug fix in SubBandSynthesis() (previously introduced while changing
 *    to automatic arrays (was previously malloc)
 *
 *  + revision of the interface to out_fifo (only the error code is
 *    returned, the clipping information became a function parameter)
 *
 * Revision 1.8  2003/05/09 17:41:48  sps
 * +++ removed memory leaks.
 * +++ revised termination behaviour for unusual cases (file could not be
 * opened, no data could be written into file, memory allocation failed). A
 * couple of functions now return an error code, that leads to a clean
 * termination in the top level function.
 * +++ simplified legacy write functionality.
 * +++ removed some unnecessary "extern" declaration in function
 * declarations in header files
 *
 * Revision 1.7  2003/04/30 17:39:53  sps
 * +++ fixed aiff header for native output +++ fixed rounding for 16 bit
 * native output (by this, separated rounding in individual function
 * castDoubleToLong(), which is now called somewhat later (not anymore in
 * SubBandSynthesis(), but in out_fifo())) +++ made sure that the last
 * samples are written into the output file +++ made sure that decoding
 * starts only if file opening was successfull +++ added AFsp library
 * support, that allows the output of WAVE files. The output of raw and
 * aiff files is still supported, but is in this case also handled by the
 * AFsp library +++ Note: If the AFsp lib is used, any particular support
 * of erroneous signals is disabled +++
 *
 * Revision 1.6  2003/03/11 10:07:19  sps
 * committed contribution from Mike Coleman to remove certain kinds of
 * compiler warnings
 *
 * Revision 1.5  2002/11/14 13:04:08  tmm01
 * warnings removed (Microsoft Visual Studio V.6.0)  23.10.2 W.Voessing
 *
 * Revision 1.2  1996/03/28 03:13:37  rowlands
 * Merged layers 1-2 and layer 3 revisions
 *
 * Revision 1.1  1996/02/14 03:45:52  rowlands
 * Initial revision
 *
 * Received from FhG
 **********************************************************************/
/**********************************************************************
 *   date   programmers         comment                               *
 * 2/25/91  Doulas Wong,        start of version 1.0 records          *
 *          Davis Pan                                                 *
 * 5/10/91  Vish (PRISM)        Renamed and regrouped all ".h" files  *
 *                              into "common.h" and "decoder.h".      *
 *                              Ported to Macintosh and Unix.         *
 * 27jun91  dpwe (Aware)        New prototype for out_fifo()          *
 *                              Moved "alloc_*" stuff to common.h     *
 *                              Use ifdef PROTO_ARGS for prototypes   *
 *                              prototypes reflect frameParameters struct*
 * 10/3/91  Don H. Lee          implemented CRC-16 error protection   *
 * 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
 *                              important fixes involved changing     *
 *                              16-bit ints to long or unsigned in    *
 *                              bit alloc routines for quant of 65535 *
 *                              and passing proper function args.     *
 *                              Removed "Other Joint Stereo" option   *
 *                              and made bitrate be total channel     *
 *                              bitrate, irrespective of the mode.    *
 *                              Fixed many small bugs & reorganized.  *
 *                              Modified some function prototypes.    *
 * 08/07/92 Mike Coleman        Made small changes for portability    *
 * 9/07/93  Toshiyuki Ishino    Integrated with Layer III.            *
 * 11/04/94 Jon Rowlands        fix protos for usage() and            *
 *                              recover_CRC_error()                   *
 *          Roland Bitto 	Adapted to MPEG2 low bitrate        	      *
 *--------------------------------------------------------------------*
 * 05/08/02 Walter Voessing     new functions for MPEG1 and MPEG 2    *
 *                              Stereo Processing                     *  
 *                              bugs within 'III_get_scale_factors'   *   
 *                              fixed                                 *
 *                              'out_fifo': bugs fixed and 24 bit     *
 *                                          output added	      *
 *                              'GetArguments': '-C' parameter added  *
 *                              (optional 24 bit output)              *
 *                              modified usage()                      *
 **********************************************************************/

#ifndef DECODER_H
#define DECODER_H
 
#include "common.h"
#include "pcmBuffer.h"
#include "frameData.h"

#define   DFLT_OPEXT        ".dec"  /* default output file name extension */

/*
  this define enables some debug output to investigate changes in the 
  channel configuration

#define CHANNEL_INVESTIGATION
*/

/*
 NOTE: The value of a multiple-character constant is
 implementation-defined.
*/
#if !defined(MS_DOS) && !defined(AIX)
#define   FILTYP_DEC_AIFF   'AIFF'
#define   FILTYP_DEC_BNRY   'TEXT'
#define   CREATR_DEC_AIFF   'Sd2a'
/*
  The following character constant is ASCII '????'
  It is declared in hex because the character
  constant contains a trigraph, causing an error in
  parsing with ANSI preprocessors.
*/
#define   CREATR_DEC_BNRY   0x3f3f3f3f
#else
#define   FILTYP_DEC_AIFF   "AIFF"
#define   FILTYP_DEC_BNRY   "TEXT"
#define   CREATR_DEC_AIFF   "Sd2a"
#define   CREATR_DEC_BNRY   "????"
#endif

typedef struct 
{
    double buf[2][2*HAN_SIZE];
    double filter[64][SBLIMIT];
    int    bufOffset[2];
} synData;

typedef struct
{
    double cs[8];
    double ca[8];
} antialiasData;

typedef struct
{
    double win[4][36];
    double COS[4*36];
} mdctData;


typedef double PCM[2][SSLIMIT][SBLIMIT];  
typedef unsigned int SAM[2][3][SBLIMIT];
typedef double FRA[2][3][SBLIMIT];
typedef double VE[2][HAN_SIZE];


/** Data struct for decoder core */
typedef struct
{
  frameData         frameData;

  int               jsbound;        /**< First band of joint stereo coding */
  int               sblimit;        /**< Total number of sub bands */
  int               tableNumber;    /**< Number of table as loaded */
  al_table          tableAlloc;     /**< Bit allocation table read in */
  unsigned int      newCRC;         /**< New calculation of CRC */
  
  PCM               pcm_sample;
  SAM               sample;
  FRA               fraction;
  VE                w;
  
} decData;


void decodeFrameData(decData *dd);
/* The following functions are in the file "decode.c" */
int I_CRC_calc(
                decData *dd,
                unsigned int bit_alloc[2][SBLIMIT]);
int II_CRC_calc(
                 decData *dd,
                 unsigned int bit_alloc[2][SBLIMIT],
                 unsigned int scfsi[2][SBLIMIT]);

void   II_decode_bitalloc(HANDLE_bitBuffer, unsigned int[2][SBLIMIT],
                       decData *);
void   I_decode_bitalloc(HANDLE_bitBuffer, unsigned int[2][SBLIMIT],
                       decData *);
void   I_decode_scale(HANDLE_bitBuffer, unsigned int[2][SBLIMIT],
                       unsigned int[2][3][SBLIMIT], decData *);
void   II_decode_scale(HANDLE_bitBuffer, unsigned int[2][SBLIMIT],
                       unsigned int[2][SBLIMIT], unsigned int[2][3][SBLIMIT],
                       decData *);
void   I_buffer_sample(HANDLE_bitBuffer, unsigned int[2][3][SBLIMIT],
                       unsigned int[2][SBLIMIT], decData *);
void   II_buffer_sample(HANDLE_bitBuffer, unsigned int[2][3][SBLIMIT],
                       unsigned int[2][SBLIMIT], decData *);
void   read_quantizer_table(double[17], double[17]);
void   II_dequantize_sample(unsigned int[2][3][SBLIMIT], 
                       unsigned int[2][SBLIMIT], double[2][3][SBLIMIT],
                       decData *);
void   I_dequantize_sample(unsigned int[2][3][SBLIMIT],
                       double[2][3][SBLIMIT], unsigned int[2][SBLIMIT],
                       decData *);
void   read_scale_factor(double[SCALE_RANGE]);
void   II_denormalize_sample(double[2][3][SBLIMIT],
                       unsigned int[2][3][SBLIMIT], decData *, int);
void   I_denormalize_sample(double[2][3][SBLIMIT],
                       unsigned int[2][3][SBLIMIT], decData *);


void createSynFilter (synData *);
void subBandSynthesis (synData *, double*, int, double*);

int read_syn_window(double[HAN_SIZE]);
void   window_sample(double*, double*);

int fill_pcm_frame_buffer(
                      HANDLE_pcmBuffer buf,
                      double           pcm_sample[2][SSLIMIT][SBLIMIT],
                      int              num,
                      decData        *fd,
                      unsigned long    *psampFrames /* for aiff header */);
                      

void   buffer_CRC(HANDLE_bitBuffer, unsigned int*);
#if 0
void   recover_CRC_error(HANDLE_pcmBuffer, unsigned long, unsigned long);
#endif
void   III_dequantize_sample(long int[SBLIMIT][SSLIMIT],
                             double [SBLIMIT][SSLIMIT], III_scalefac_t *,
                             struct gr_info_s *, int, decData *);
void   III_antialias_init(antialiasData *aa);
void   III_antialias(double[SBLIMIT][SSLIMIT], double[SBLIMIT][SSLIMIT], 
                     struct gr_info_s *,
                     int antialias_mixed_blocks, antialiasData *aa);
void   initMdct(mdctData *md);
void   inv_mdct(double[18], double[36], int, mdctData *md);
void   III_hybrid_init( double prevblck[2][SBLIMIT][SSLIMIT] );
void   III_hybrid(double[SSLIMIT], double[SSLIMIT] , int, int,
                       struct gr_info_s *, mdctData *, double prevblck[2][SBLIMIT][SSLIMIT]);
int III_get_side_info_crc(HANDLE_bitBuffer,
                             III_side_info_t *,
                             decData *);


void III_get_scale_factors(HANDLE_bitBuffer,
                           III_scalefac_t *,
                           III_side_info_t *, 
                           int, 
                           int);

void III_get_LSF_scale_factors(HANDLE_bitBuffer, III_scalefac_t *scalefac, III_side_info_t *si, int gr, int ch, decData *dd,unsigned int scalefac_buffer[54],unsigned int scalefac_IIP_buffer[54]);
int III_huffman_decode(HANDLE_bitBuffer, long int is[SBLIMIT][SSLIMIT], III_side_info_t *si, int ch, int gr, int part2_start, decData *dd);
void III_reorder (double xr[SBLIMIT][SSLIMIT], double ro[SBLIMIT][SSLIMIT], struct gr_info_s *gr_info, decData *dd); 

void III_get_LSF_scale_data(
                            HANDLE_bitBuffer mainData,
                            III_side_info_t *si,
                            int gr,
                            int ch,
                            decData *dd,
                            unsigned int scalefac_buffer[54],
                            unsigned int scalefac_IIP_buffer[54] );

void III_Intensity(
                   double xr[2][SBLIMIT][SSLIMIT],
                   double lr[2][SBLIMIT][SSLIMIT],
                   double IntensFactor[2][SBLIMIT*SSLIMIT],
                   int Start,
                   int Number );

void III_M_S(
             double xr[2][SBLIMIT][SSLIMIT],
             double lr[2][SBLIMIT][SSLIMIT],
             int Start,
             int Number );

void III_Stereo(
                double xr[2][SBLIMIT][SSLIMIT],
                double lr[2][SBLIMIT][SSLIMIT],
                int Start,
                int Number );


/*********************************************************/
/* NEW FUNCTIONS FOR MPEG I AND MPEG 2 STEREO PROCESSING */
/*********************************************************/
/* WV, 24.10.02 */

void MPEG_1_III_stereo(double xr[2][SBLIMIT][SSLIMIT], double lr[2][SBLIMIT][SSLIMIT], III_scalefac_t *scalefac,
			        								struct gr_info_s *gr_info, decData *dd);
void MPEG_2_III_stereo(double xr[2][SBLIMIT][SSLIMIT], double lr[2][SBLIMIT][SSLIMIT], III_scalefac_t *scalefac,
											        struct gr_info_s *gr_info_l, struct gr_info_s *gr_info_r, decData *dd,unsigned int scalefac_IIP_buffer[54]);

#endif /* #ifndes DEC_LAYER3_H */
