/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ???
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * and edited by 
 *
 *  
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#include <stdlib.h>
#include "fileWriter.h"
#include "debug.h"

#ifdef USELIB_AFsp
#include <libtsp.h>
#include <libtsp/AFpar.h>
#include <libtsp/nucleus.h>
#else
#include <netinet/in.h>
#include "portableio.h"
#endif /* USELIB_AFsp */

typedef struct TAG_fileWriter
{
  HANDLE_debug      dbg;
  FORMAT_fileWriter format;
  
  unsigned int      outChannels;  /* stores the initial channel configuration */
  unsigned int      channelsBefore;
  
  unsigned int      samplingFreq;
  unsigned int      clipped;
  unsigned int      writtenSampleFrames;
  char              *filename;
  
  int               infoPrinted;
  int               initOutFile;
  
#ifdef USELIB_AFsp
  AFILE             *outFile;
#else
  FILE              *outFile;
#endif
 
} fileWriter_t;



/*****************************************************************************
 * private functions
 *****************************************************************************/



#ifndef USELIB_AFsp

/* Seek past some Audio Interchange File Format (AIFF) headers to sound data. */
static
int aiff_seek_to_sound_data( FILE *file_ptr )
{
  if ( fseek(file_ptr, AIFF_FORM_HEADER_SIZE + AIFF_COMM_HEADER_SIZE + AIFF_SSND_HEADER_SIZE , SEEK_SET) != 0 ) 
  {
    return 0;
  }
  return 1;
}

/* Write Audio Interchange File Format (AIFF) headers. */
static
int aiff_write_headers( FILE *file_ptr, IFF_AIFF *aiff_ptr )
{
  int chunkSize;
  int sampleBytes = (aiff_ptr->sampleSize / 8) + (aiff_ptr->sampleSize % 8 ? 1 : 0);

  if ( fseek(file_ptr, 0L, SEEK_SET) != 0 ) 
  {
    return 0;
  }

  /* write FORM chunk */
  chunkSize = AIFF_FORM_HEADER_SIZE + AIFF_COMM_HEADER_SIZE + AIFF_SSND_HEADER_SIZE - 8 + aiff_ptr->numChannels * aiff_ptr->numSampleFrames * sampleBytes;
/*     chunkSize = AIFF_SSND_HEADER_SIZE - 8 + aiff_ptr->numChannels * aiff_ptr->numSampleFrames * sampleBytes; */
  Write32BitsHighLow( file_ptr, IFF_ID_FORM );
  Write32BitsHighLow( file_ptr, chunkSize );
  Write32BitsHighLow( file_ptr, IFF_ID_AIFF );
  /* write COMM chunk */
  Write32BitsHighLow( file_ptr, IFF_ID_COMM );
  Write32BitsHighLow( file_ptr, AIFF_COMM_HEADER_SIZE - 8 ); /* chunk size */
  Write16BitsHighLow( file_ptr, aiff_ptr->numChannels );
  Write32BitsHighLow( file_ptr, aiff_ptr->numSampleFrames );
  Write16BitsHighLow( file_ptr, aiff_ptr->sampleSize );
  WriteIeeeExtendedHighLow( file_ptr, aiff_ptr->sampleRate );
  /* write SSND chunk header */
  chunkSize = 8 + aiff_ptr->numChannels * aiff_ptr->numSampleFrames * sampleBytes;
  Write32BitsHighLow( file_ptr, IFF_ID_SSND );
  Write32BitsHighLow( file_ptr, chunkSize );
  Write32BitsHighLow( file_ptr, 0 ); /* offset */
  Write32BitsHighLow( file_ptr, 0 ); /* block size */
  return 1;
}

#endif /* ifndef USELIB_AFsp */



static
int initOutFile(HANDLE_fileWriter fw)
{

#ifdef USELIB_AFsp

  unsigned short     dFormat = 0;
  unsigned short     fType = 0;
  
  if (fw->format & FW_INT24) 
    dFormat = FD_INT24;
  else
    dFormat = FD_INT16;
    
  switch (fw->format & FW_FF_MASK)
  {
    case FW_RAW:
      fType = FTW_NH_EB;
      debugPrintf(fw->dbg, D_NOTICE_L2, "Opening RAW file");
      break;
    case FW_WAVE:
      fType = FTW_WAVE;
      debugPrintf(fw->dbg, D_NOTICE_L2, "Opening WAVE file");
      break;
    case FW_AIFF:
      fType = FTW_AIFF;
      debugPrintf(fw->dbg, D_NOTICE_L2, "Opening AIFF file");
      break;
    case FW_ESPS:
      /* TODO supported??? #ifdef ESPS ?? */
      break;
    default:
      debugPrintf(fw->dbg, D_WARNING, "No file format selected. Using WAVE format.");
      fType = FTW_WAVE;
      break;
  }
  
  /*  (AFoptions ())->ErrorHalt = 0;*/
  
  fw->outFile = AFopnWrite(fw->filename, fType, dFormat, fw->outChannels, fw->samplingFreq, NULL);
  if (fw->outFile == NULL ) 
  {
    debugPrintf(fw->dbg, D_ERROR, "Could not create '%s'", fw->filename);
    return 0;
  }

#else /* USELIB_AFsp */
  
  if ((fw->outFile = fopen(fw->filename, "w+b")) == NULL) 
  {
    debugPrintf(fw->dbg, D_ERROR, "Could not create '%s'", fw->filename);
    return 0;
  }

  if (fw->format & FW_AIFF)
  {
    if (!aiff_seek_to_sound_data(fw->outFile))
    {
      debugPrintf(fw->dbg, D_ERROR, "Could not seek to PCM sound data");
      return 0;
    }
  }

#ifdef ESPS
  if (fw->format & FW_ESPS)
  {
    debugPrintf(fw->dbg, D_NOTICE, "Writing ESPS header");
    esps_write_header(fw->outFile, (long) sampleFrames, (double) fw->samplingFreq, fw->outChannels, fw->filename );
  }
#endif /* ESPS */

#endif /* USELIB_AFsp */

  return 1;

} /* initOutFile() */







/*****************************************************************************
 * public functions
 *****************************************************************************/

HANDLE_fileWriter fileWriterInit(char *filename, FORMAT_fileWriter format, HANDLE_debug debug)
{
  HANDLE_fileWriter fw;
  
  debugEnter(debug, "fileWriterInit()");
  
  /* allocate handler struct */
  if ((fw = (HANDLE_fileWriter) malloc(sizeof(fileWriter_t))) == NULL)
  {
    debugPrintf(debug, D_ERROR, "Could not allocate memory for Handler");
    debugLeave(debug);
    return NULL;
  }
  
  fw->dbg = debug;
  fw->filename = filename;
  fw->initOutFile = 0;
  fw->format = format;
  if (format & FW_FORCE_MONO) 
    fw->outChannels = 1; 
  else if (format & FW_FORCE_STEREO) 
    fw->outChannels = 2;
  else
    fw->outChannels = 0;
  fw->infoPrinted = 0;
  fw->clipped = 0;
  
  debugLeave(debug);
  return fw;
}




ERROR_CODE_fileWriter fileWriterOutBuffer(HANDLE_fileWriter fw, HANDLE_pcmBuffer pcmBuffer)
{
  ERROR_CODE_fileWriter ret;
  STATUS_pcmBuffer  pcm;
  unsigned int      deliveredSamples  = pcmBufferGetAvailable(pcmBuffer);
  unsigned int      writtenSamples    = 0;
  unsigned int      channelsInBuf     = pcmBufferGetChannels(pcmBuffer);
  int               monoToStereo      = 0;
  int               stereoToMono      = 0;
  long              sample            = 0; 
  long              leftSample        = 0;
  int               i, leftChannel;
#ifdef USELIB_AFsp
  float             sampleOut;
  int               bits              = 24;
#else
  unsigned long     bigWord;
  int               bits              = (fw->format & FW_INT24)? 24 : 16;
  unsigned int      subwords          = bits / 8;
  unsigned char     *wordStart        = ((unsigned char*)&bigWord)+(4-subwords);  
  int               m;
#endif

  debugEnter(fw->dbg, "fileWriterOutBuffer()");
  
  /* Initialize out file */  
  if (!fw->initOutFile)
  {
    if (!fw->outChannels)
      fw->outChannels = channelsInBuf;
      
    if (!fw->samplingFreq)
      fw->samplingFreq = pcmBufferGetSamplingFreq(pcmBuffer);
      
    if (!initOutFile(fw))
    {
      debugPrintf (fw->dbg, D_ERROR, "Error while initializing output file");
      debugLeave (fw->dbg);
      return FW_INIT_ERROR;
    }
    fw->initOutFile = 1;
  }

  monoToStereo = ((fw->outChannels == 2) && (channelsInBuf == 1))?1:0;
  stereoToMono = ((fw->outChannels == 1) && (channelsInBuf == 2))?1:0;

  if ( fw->channelsBefore && channelsInBuf != fw->channelsBefore )
  {
    debugPrintf (fw->dbg, D_WARNING, "Implicit channel re-configuration from %d to %d", fw->channelsBefore, channelsInBuf);
  }
  fw->channelsBefore = channelsInBuf;
  
  if ( !fw->infoPrinted && stereoToMono )
  {
    debugPrintf (fw->dbg, D_WARNING, "Forcing mono! Channels will be mixed from stereo to mono");
    fw->infoPrinted = 1;
  }
  else
  if ( !fw->infoPrinted && monoToStereo )
  {
    debugPrintf (fw->dbg, D_WARNING, "Forcing stereo! Mono will be duplicated to stereo");
    fw->infoPrinted = 1;
  }
  
   
  
  leftChannel = 1;
  
  while (1) 
  {
    /* get next scaled and castet sample from pcm buffer */
    pcm = pcmBufferGetCasted(pcmBuffer, &sample, bits);
    if      (pcm == PCM_EMPTY)   break;         /* break if no sample available */
    else if (pcm == PCM_CLIPPED) fw->clipped++; /* count clipped samples */
    
    if (stereoToMono)
    {
      if (leftChannel)
      {
        leftSample  = sample;
        leftChannel = 0;
        continue; /* get right sample */
      }
      else
      {
        /* mean of two channels */
        sample      = (leftSample + sample) / 2;
        leftChannel = 1;
        deliveredSamples--;
      }
    }

#ifdef USELIB_AFsp

    /* write sample using AFsp-Lib */
    sampleOut = (float) sample/(1<<23);
    
    for (i=0; i<=monoToStereo; i++)
    {
      if (!AFfWriteData(fw->outFile, &sampleOut, 1)) break;
    }

#else /* USELIB_AFsp */

    /* convert sample to word written in big endian */
    bigWord = htonl(sample);

    /* write out words to file. break if no sample available */
    for (i=0; i<=monoToStereo; i++)
    {
      if (fwrite(wordStart, sizeof (char), subwords, fw->outFile) < subwords ) break;
    }
    
#endif /* USELIB_AFsp */
    
    writtenSamples++;

  } /* while(1) */
  
  fw->writtenSampleFrames += writtenSamples / channelsInBuf;
      
  if (writtenSamples != deliveredSamples)
  {
    debugPrintf (fw->dbg, D_ERROR, "Could only write %u from %u samples", writtenSamples, deliveredSamples);
    debugLeave (fw->dbg);
    return FW_WRITE_ERROR;
  }

  debugPrintf (fw->dbg, D_NOTICE_L2, "Successfully written %u samples", writtenSamples / channelsInBuf);
  debugLeave (fw->dbg);
  return FW_NO_ERROR;
}


void fileWriterClose(HANDLE_fileWriter fw)
{
  if (!fw) return;
  
  debugEnter (fw->dbg, "fileWriterClose()");

#ifndef USELIB_AFsp
  if (fw->format & FW_AIFF) 
  {
    IFF_AIFF pcm_aiff_data;
    
    pcm_aiff_data.numChannels       = fw->outChannels; /* TODO channelsInit ?? */
    pcm_aiff_data.numSampleFrames   = fw->writtenSampleFrames;
    if (fw->format & FW_INT24) 
    { 
      pcm_aiff_data.sampleSize      = 24; 
    } 
    else 
    { 
      pcm_aiff_data.sampleSize      = 16;
    } 
    pcm_aiff_data.sampleRate        = fw->samplingFreq;
    pcm_aiff_data.sampleType        = IFF_ID_SSND;
    pcm_aiff_data.blkAlgn.offset    = 0;
    pcm_aiff_data.blkAlgn.blockSize = 0;

    debugPrintf(fw->dbg, D_NOTICE_L2, "Writing AIFF header");
    if ( !aiff_write_headers(fw->outFile, &pcm_aiff_data)) 
    {
      debugPrintf(fw->dbg, D_ERROR, "Could not write AIFF headers");
    }
  }
#endif /* USELIB_AFsp */

  /* Close file */
#ifdef USELIB_AFsp
  if (fw->outFile) AFclose(fw->outFile);
#else
  if (fw->outFile) fclose(fw->outFile);
#endif

  if(fw->clipped > 0)
  { 
    debugPrintf(fw->dbg, D_NOTICE,"%d output samples clipped", fw->clipped);
  }

  debugLeave (fw->dbg);
}


unsigned long fileWriterGetWritten(HANDLE_fileWriter fw)
{
  return fw->writtenSampleFrames;
}

unsigned long fileWriterGetSamplingFreq(HANDLE_fileWriter fw)
{
  return fw->samplingFreq;
}


void fileWriterFree(HANDLE_fileWriter fw)
{
  if (fw)
  {
    free(fw);
  }
}



