/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * and edited by 
 *
 *  
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#ifndef FILEWRITER_H_
#define FILEWRITER_H_

#include "bitBuffer.h"
#include "pcmBuffer.h"
#include "debug.h"

typedef enum 
{
  /* File format */
  FW_RAW = 0,
  FW_WAVE = 1,
  FW_AIFF = 2,
  FW_ESPS = 4,
  FW_FF_MASK = 7,
  
  /* 24 Bit per sample on/off */
  FW_INT24 = 8,
  
  /* Forcing mono or stereo */
  FW_FORCE_MONO = 16,
  FW_FORCE_STEREO = 32
  
} FORMAT_fileWriter;
/* just add values to create format */

typedef enum
{
  FW_NO_ERROR = 0,
  FW_INIT_ERROR,
  FW_WRITE_ERROR
} ERROR_CODE_fileWriter;


/* AIFF Definitions */

#define IFF_ID_FORM 0x464f524d /* "FORM" */
#define IFF_ID_AIFF 0x41494646 /* "AIFF" */
#define IFF_ID_COMM 0x434f4d4d /* "COMM" */
#define IFF_ID_SSND 0x53534e44 /* "SSND" */
#define IFF_ID_MPEG 0x4d504547 /* "MPEG" */

#define AIFF_FORM_HEADER_SIZE 12
#define AIFF_COMM_HEADER_SIZE 26
#define AIFF_SSND_HEADER_SIZE 16


typedef struct  blockAlign_struct {
    unsigned long   offset;
    unsigned long   blockSize;
} blockAlign;

typedef struct  IFF_AIFF_struct {
    short           numChannels;
    unsigned long   numSampleFrames;
    short           sampleSize;
    double          sampleRate;
    unsigned long   sampleType;
    blockAlign      blkAlgn;
} IFF_AIFF;


typedef struct TAG_fileWriter* HANDLE_fileWriter;

HANDLE_fileWriter fileWriterInit(char *filename, FORMAT_fileWriter format, HANDLE_debug debug);
ERROR_CODE_fileWriter fileWriterOutBuffer(HANDLE_fileWriter fw, HANDLE_pcmBuffer pcmBuffer);
void fileWriterClose(HANDLE_fileWriter fw);
void fileWriterFree(HANDLE_fileWriter fw);
unsigned long fileWriterGetSamplingFreq(HANDLE_fileWriter fw);
unsigned long fileWriterGetWritten(HANDLE_fileWriter fw);



#endif /*FILEWRITER_H_*/
