/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   Matejko Michael (Fraunhofer IIS) 
 * 
 * and edited by 
 * 
 * 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#ifndef FRAMEDATA_H_
#define FRAMEDATA_H_

#include "bitBuffer.h"

/** MPEG Header Definitions - Audio ID/IDex Values */
typedef enum
{
  MPEG_PHASE1   = 0,
  MPEG_PHASE2   = 1,
  MPEG_RSVD_VER = 2, /*reserved*/
  MPEG_PHASE2_5 = 3
} mpegVersion;

/** MPEG Header Definitions - Layer Values */
typedef enum
{
  LAYER1 = 1,
  LAYER2 = 2,
  LAYER3 = 3
} mpegLayer;

/** MPEG Header Definitions - Mode Values */
typedef enum
{
  MPG_MD_STEREO       = 0,
  MPG_MD_JOINT_STEREO = 1,
  MPG_MD_DUAL_CHANNEL = 2,
  MPG_MD_MONO         = 3
} mpegMode;

/** Mode Extention */
typedef enum
{
  MPG_MD_LR_LR  = 0,
  MPG_MD_LR_I   = 1,
  MPG_MD_MS_LR  = 2,
  MPG_MD_MS_I   = 3
} mpegModeExt;


typedef struct 
{
  /* header data */
  mpegVersion   version;            /**< Audio version ID */
  mpegLayer     lay;                /**< Layer index */
  int           error_protection;   /**< protection bit */
  int           bitrate_index;      /**< bitrate index */
  int           sampling_frequency; /**< sampling rate index */
  int           padding;            /**< padding bit */
  int           extension;          /**< private bit (only informative) */
  mpegMode      mode;               /**< channel mode */
  mpegModeExt   mode_ext;           /**< mode extension */
  int           copyright;          /**< copyright bit */
  int           original;           /**< original bit */
  int           emphasis;           /**< emphasis */
  
  /* decoded data */
  double        sfreq;              /**< sampling frequency */
  int           bitrate;            /**< bitrate */
  int           channels;           /**< 1 for mono, 2 for stereo */
  int           slotbits;           /**< bits per slot */
  int           samples;            /**< count of samples per frame */
  const char *  strVersion;         /**< Version name */
  const char *  strLayer;           /**< Layer name */
  const char *  strMode;            /**< Stereo Mode name */
  
  /* CRC word */
  unsigned int  crc;                /**< CRC value from bitstream */
  
  /* main data begin pionter for layer3 */
  unsigned int main_data_begin;     /**< main data begin */
  
} frameData;

int getFrameData(frameData *fd, HANDLE_bitBuffer buf);

#endif /*FRAMEDATA_H_*/
