/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   ??? 
 * 
 * and edited by 
 * 
 *
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 *
 * $Id: huffman.c,v 1.15 2007/09/27 13:00:02 matejkml Exp $
 *
 * $Log: huffman.c,v $
 * Revision 1.15  2007/09/27 13:00:02  matejkml
 * added copyright info
 *
 * Revision 1.14  2007/01/10 13:36:24  matejkml
 * bugfix2
 *
 * Revision 1.10  2004/11/10 17:45:31  sps
 * removed obsolete comments
 *
 * Revision 1.9  2004/09/29 14:57:10  kaehleof
 * moved the decoder tables into tables.c and .h;
 * no more tables/ directory needed now;
 * create_tables.c was used to create tables.c and .h
 * from the files in the tables/ directory;
 *
 * Revision 1.8  2003/05/13 17:15:34  sps
 *  + bug fix in SubBandSynthesis() (previously introduced while changing
 *    to automatic arrays (was previously malloc))
 *
 *  + revision of the out_fifo interface (only the error code is returned,
 *    the clipping information became a function parameter)
 *
 *  + revised the code such that it can be compiled using g++
 *
 *  + functions returning an error code have now the return type
 *    ERROR_CODE. This concerns
 *     - out_fifo()
 *     - SubBandSynthesis()
 *     - III_get_side_info()
 *     - III_hufman_decode()
 *     - read_syn_window()
 *     - open_bit_stream_r()
 *     - initialize_huffman()
 *     - GetArguments()
 *     - aiff_read_headers()
 *     - aiff_seek_to_sound()
 *     - aiff_write_headers()
 *
 * Revision 1.7  2003/05/09 17:41:48  sps
 * +++ removed memory leaks.
 * +++ revised termination behaviour for unusual cases (file could not be
 * opened, no data could be written into file, memory allocation failed). A
 * couple of functions now return an error code, that leads to a clean
 * termination in the top level function.
 * +++ simplified legacy write functionality.
 * +++ removed some unnecessary "extern" declaration in function
 * declarations in header files
 *
 * Revision 1.6  2003/03/11 10:07:20  sps
 * committed contribution from Mike Coleman to remove certain kinds of
 * compiler warnings
 *
 * Revision 1.5  2002/11/14 13:04:08  tmm01
 * warnings removed (Microsoft Visual Studio V.6.0)  23.10.2 W.Voessing
 *
 * Revision 1.2  1996/03/28 03:13:37  rowlands
 * Merged layers 1-2 and layer 3 revisions
 *
 * Revision 1.1  1996/02/14 03:45:52  rowlands
 * Initial revision
 *
 * Received from FhG
 **********************************************************************/
/**********************************************************************
 *   date   programmers                comment                        *
 *27.2.92   F.O.Witte                  (ITT Intermetall)              *
 *				       email: otto.witte@itt-sc.de    *
 *				       tel:   ++49 (761)517-125	      *
 *				       fax:   ++49 (761)517-880	      *
 *12.6.92   J. Pineda                  Added sign bit to decoder.     *
 * 08/24/93 M. Iwadare                 Changed for 1 pass decoding.   *
 *--------------------------------------------------------------------*
 *  7/14/94 Juergen Koller      Bug fixes in Layer III code           *
 *--------------------------------------------------------------------*
 * 23/09/04 Olaf Kaehler        move tables to tables.h               *
 *********************************************************************/	

#include "common.h"
#include "huffman.h"
#include "stdlib.h"
#include "bitBuffer.h"
     
HUFFBITS dmask = 1 << (sizeof(HUFFBITS)*8-1);
unsigned int hs = sizeof(HUFFBITS)*8;

/* do the huffman-decoding 						*/
/* note! for counta,countb -the 4 bit value is returned in y, discard x */
int huffman_decoder(
                    HANDLE_bitBuffer mainData,
                    struct huffcodetab *h,	/* pointer to huffman code record	*/
                    /* unsigned */ int *x,	/* returns decoded x value 		*/
                    /* unsigned */ int *y,	/* returns decoded y value		*/
                    int *v,
                    int *w )
{  
  HUFFBITS level;
  int point = 0;
  int error = 1;
  level     = dmask;
  if (h->val == NULL) return 2;

  /* table 0 needs no bits */
  if ( h->treelen == 0)
  {  *x = *y = 0;
     return 0;
  }

  /* Lookup in Huffman table. */

  do {
    if (h->val[point][0]==0) {   /*end of tree*/
      *x = h->val[point][1] >> 4;
      *y = h->val[point][1] & 0xf;

      error = 0;
      break;
    } 
    if (bitBufferGet(mainData, 1)) {
      while (h->val[point][1] >= MXOFF) point += h->val[point][1]; 
      point += h->val[point][1];
    }
    else {
      while (h->val[point][0] >= MXOFF) point += h->val[point][0]; 
      point += h->val[point][0];
    }
    level >>= 1;
  } while (level  || ((unsigned int)point < h->treelen) );
  
  /* Check for error. */

  if (error) { /* set x and y to a medium value as a simple concealment */
    printf("Illegal Huffman code in data.\n");
    *x = ((h->xlen-1) << 1);
    *y = ((h->ylen-1) << 1);
  }

  /* Process sign encodings for quadruples tables. */

  if (h->tablename[0] == '3'
      && (h->tablename[1] == '2' || h->tablename[1] == '3')) {
     *v = (*y>>3) & 1;
     *w = (*y>>2) & 1;
     *x = (*y>>1) & 1;
     *y = *y & 1;

     if (*v)
        if (bitBufferGet(mainData, 1) == 1) *v = -*v;
     if (*w)
        if (bitBufferGet(mainData, 1) == 1) *w = -*w;
     if (*x)
        if (bitBufferGet(mainData, 1) == 1) *x = -*x;
     if (*y)
        if (bitBufferGet(mainData, 1) == 1) *y = -*y;
     }
     
  /* Process sign and escape encodings for dual tables. */
  
  else {
  
      /* x and y are reversed in the test bitstream.
         Reverse x and y here to make test bitstream work. */
	 
/*    removed 11/11/92 -ag  
		{int i=*x; *x=*y; *y=i;} 
*/      
     if (h->linbits)
       if ((h->xlen-1) == (unsigned int)*x) 
         *x += bitBufferGet(mainData, h->linbits);
     if (*x)
        if (bitBufferGet(mainData, 1) == 1) *x = -*x;
     if (h->linbits)	  
       if ((h->ylen-1) == (unsigned int)*y)
         *y += bitBufferGet(mainData, h->linbits);
     if (*y)
        if (bitBufferGet(mainData, 1) == 1) *y = -*y;
     }
	  
  return error;  
}
