/******************************************************************************
 *
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Decoder - Lower Sampling Frequency Extension
 * 
 * 
 * This software module was originally developed by 
 * 
 *   Matejko Michael (Fraunhofer IIS)
 * 
 * and edited by 
 * 
 * 
 * 
 * in the course of development of the ISO/IEC 13818-3 for reference purposes
 * and its performance may not have been optimized. This software module is
 * an implementation of one or more tools as specified by the ISO/IEC 13818-3.
 * ISO/IEC gives You a royalty-free, worldwide, non-exclusive, copyright
 * license to copy, distribute, and make derivative works of this software
 * module ormodifications thereof for use in implementations of the 
 * ISO/IEC 13818-3  in products that satisfy conformance criteria (if any).
 * Those intending to use this software module in products are advised that 
 * its use may infringe existing patents. ISO/IEC have no liability for use of
 * this software module or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual
 * and image-coding related ITU Recommendations and/or ISO/IEC International
 * Standards. 
 * 
 ******************************************************************************/

/******************************************************************************
 * ATTENTION: Documentation is generated by DoxyGen using "make docs"
 ******************************************************************************/

#include "pcmBuffer.h"
#include <stdlib.h>

typedef struct TAG_pcmBuffer
{
  double            *ptr;     /* pointer to pcm buffer */
  unsigned int      size;     /* size of buffer */
  unsigned int      fill;     /* nr of samples filled */
  unsigned int      rpos;     /* read position */
  unsigned int      channels; /* no of channels in buffer */
  unsigned int      sfreq;    /* sampling freqency */
} pcmBuffer_t;


HANDLE_pcmBuffer pcmBufferInit (unsigned int size)
{
  HANDLE_pcmBuffer buf;
  
  /* create handler struct */
  if ( ( buf = (HANDLE_pcmBuffer) malloc(sizeof(pcmBuffer_t)) ) == NULL )
    return NULL;
  
  /* allocate buffer */
  if ( ( buf->ptr = (double *) malloc(sizeof(double) * size) ) == NULL ) {
    free(buf);
    return NULL;
  }
      
  buf->size = size;
  buf->fill = 0;
  
  return buf;
}


STATUS_pcmBuffer pcmBufferPut (HANDLE_pcmBuffer buf, double sample)
{
  if (buf->fill >= buf->size)
    return PCM_FULL;
    
  buf->ptr[buf->fill++] = sample;
  return PCM_OK;
}


STATUS_pcmBuffer pcmBufferGet (HANDLE_pcmBuffer buf, double *sample)
{
  if (buf->rpos >= buf->fill)
    return PCM_EMPTY;
    
  *sample = buf->ptr[buf->rpos++];
  return PCM_OK;
}


STATUS_pcmBuffer pcmBufferGetCasted (HANDLE_pcmBuffer buf, long *sample, int bits)
{
  STATUS_pcmBuffer ret;
  double sampleIn;
  long   scale = (1<<(bits-1));
  
  /* get sample */
  ret = pcmBufferGet(buf, &sampleIn);
  if (ret != PCM_OK) return ret;
  
  /* scale and cast sample
     The following rounding turncates towards zero for both positive and negative values.
     The result is a cross-over distorsion. */
  if(sampleIn > 0) 
  {
    *sample = (long)(sampleIn * (double)scale + (double)0.5);
  }
  else 
  {
    *sample = (long)(sampleIn * (double)scale - (double)0.5);
  }
  
  /* clip sample if it exceeds scaling range */
  if (*sample >= scale) 
  {
    *sample = (scale-1);
    return PCM_CLIPPED;
  }
  else if (*sample < -scale) 
  {
    *sample = (-scale);
    return PCM_CLIPPED;
  }

  return PCM_OK;
}

unsigned int pcmBufferGetAvailable (HANDLE_pcmBuffer buf)
{
  return buf->fill - buf->rpos;
}

unsigned int pcmBufferGetWritten (HANDLE_pcmBuffer buf)
{
  return buf->fill; /* / buf->channels;*/
}

/* Restore no of samples after BufferReset (needed for CRC recovery) 
 * very easy, because samples still in buffer!
 */
unsigned int pcmBufferRestore (HANDLE_pcmBuffer buf, unsigned int samples)
{
  if ( (buf->fill + samples) > buf->size )
    return 0;
    
  buf->fill += samples;
  return 1;
}


void pcmBufferReset (HANDLE_pcmBuffer buf)
{
  buf->fill = 0;
  buf->rpos = 0;
}

unsigned int pcmBufferGetChannels (HANDLE_pcmBuffer buf)
{
  return buf->channels;
}

void pcmBufferSetChannels (HANDLE_pcmBuffer buf, unsigned int channels)
{
  buf->channels = channels;
}

unsigned int pcmBufferGetSamplingFreq (HANDLE_pcmBuffer buf)
{
  return buf->sfreq;
}

void pcmBufferSetSamplingFreq (HANDLE_pcmBuffer buf, unsigned int sfreq)
{
  buf->sfreq = sfreq;
}

void pcmBufferFree (HANDLE_pcmBuffer buf)
{
  if (buf) 
  {
    free(buf->ptr);
    free(buf);
  }
}
